@@ -1,6 +1,6 @@
 from collections import defaultdict
 import logging
-from typing import List
+from typing import Dict, List, Tuple, Set, Optional, Iterable, Union, Type, TYPE_CHECKING
 
 import networkx
 
@@ -13,9 +13,17 @@ from ...calling_conventions import SimRegArg, SimStackArg, SimFunctionArgument
 from ...sim_type import SimTypeChar, SimTypeInt, SimTypeLongLong, SimTypeShort, SimTypeFunction, SimTypeBottom
 from ...sim_variable import SimVariable, SimStackVariable, SimRegisterVariable
 from ...knowledge_plugins.key_definitions.constants import OP_BEFORE, OP_AFTER
+from ...procedures.stubs.UnresolvableCallTarget import UnresolvableCallTarget
+from ...procedures.stubs.UnresolvableJumpTarget import UnresolvableJumpTarget
 from .. import Analysis, register_analysis
+from ..cfg.cfg_base import CFGBase
 from .ailgraph_walker import AILGraphWalker
-from .optimization_passes import get_default_optimization_passes
+from .ailblock_walker import AILBlockWalker
+from .optimization_passes import get_default_optimization_passes, OptimizationPassStage
+
+if TYPE_CHECKING:
+    from angr.knowledge_plugins.cfg import CFGModel
+    from .peephole_optimizations import PeepholeOptimizationStmtBase, PeepholeOptimizationExprBase
 
 
 l = logging.getLogger(name=__name__)
@@ -30,11 +38,9 @@ class Clinic(Analysis):
                  exception_edges=False,
                  sp_tracker_track_memory=True,
                  optimization_passes=None,
+                 cfg=None,
+                 peephole_optimizations: Optional[Iterable[Union[Type['PeepholeOptimizationStmtBase'],Type['PeepholeOptimizationExprBase']]]]=None,
                  ):
-
-        # Delayed import
-        import ailment.analyses  # pylint:disable=redefined-outer-name,unused-import,import-outside-toplevel
-
         if not func.normalized:
             raise ValueError("Decompilation must work on normalized function graphs.")
 
@@ -44,13 +50,15 @@ class Clinic(Analysis):
         self.arg_list = None
         self.variable_kb = None
 
-        self._func_graph = None  # type: networkx.DiGraph
+        self._func_graph: Optional[networkx.DiGraph] = None
         self._ail_manager = None
         self._blocks_by_addr_and_size = { }
 
         self._remove_dead_memdefs = remove_dead_memdefs
         self._exception_edges = exception_edges
         self._sp_tracker_track_memory = sp_tracker_track_memory
+        self._cfg: Optional['CFGModel'] = cfg
+        self.peephole_optimizations = peephole_optimizations
 
         # sanity checks
         if not self.kb.functions:
@@ -104,6 +112,13 @@ class Clinic(Analysis):
         # Set up the function graph according to configurations
         self._set_function_graph()
 
+        # Remove alignment blocks
+        self._remove_alignment_blocks()
+
+        # if the graph is empty, don't continue
+        if not self._func_graph:
+            return
+
         # Make sure calling conventions of all functions have been recovered
         self._recover_calling_conventions()
 
@@ -116,10 +131,24 @@ class Clinic(Analysis):
         # Convert VEX blocks to AIL blocks and then simplify them
 
         self._convert_all()
-        ail_graph = self._simplify_blocks(stack_pointer_tracker=spt)
+
+        ail_graph = self._make_ailgraph()
+
+        # Fix "fake" indirect jumps and calls
+        ail_graph = self._replace_single_target_indirect_transitions(ail_graph)
+
+        # Make returns
+        ail_graph = self._make_returns(ail_graph)
+
+        # Simplify blocks
+        ail_graph = self._simplify_blocks(ail_graph, stack_pointer_tracker=spt)
+
+        # Run simplification passes
+        ail_graph = self._run_simplification_passes(ail_graph,
+                                                    stage=OptimizationPassStage.AFTER_SINGLE_BLOCK_SIMPLIFICATION)
 
         # Simplify the entire function for the first time
-        self._simplify_function(ail_graph)
+        self._simplify_function(ail_graph, unify_variables=False)
 
         # clear _blocks_by_addr_and_size so no one can use it again
         # TODO: Totally remove this dict
@@ -129,7 +158,7 @@ class Clinic(Analysis):
         self._make_callsites(ail_graph, stack_pointer_tracker=spt)
 
         # Simplify the entire function for the second time
-        self._simplify_function(ail_graph)
+        self._simplify_function(ail_graph, unify_variables=True)
 
         # Make function arguments
         arg_list = self._make_argument_list()
@@ -141,7 +170,7 @@ class Clinic(Analysis):
         self._make_function_prototype(arg_list, variable_kb)
 
         # Run simplification passes
-        ail_graph = self._run_simplification_passes(ail_graph)
+        ail_graph = self._run_simplification_passes(ail_graph, stage=OptimizationPassStage.AFTER_GLOBAL_SIMPLIFICATION)
 
         self.graph = ail_graph
         self.arg_list = arg_list
@@ -151,6 +180,16 @@ class Clinic(Analysis):
     def _set_function_graph(self):
         self._func_graph = self.function.graph_ex(exception_edges=self._exception_edges)
 
+    @timethis
+    def _remove_alignment_blocks(self):
+        """
+        Alignment blocks are basic blocks that only consist of nops. They should not be included in the graph.
+        """
+        for node in list(self._func_graph.nodes()):
+            if self._func_graph.in_degree(node) == 0 and \
+                    CFGBase._is_noop_block(self.project.arch, self.project.factory.block(node.addr, node.size)):
+                self._func_graph.remove_node(node)
+
     @timethis
     def _recover_calling_conventions(self):
         self.project.analyses.CompleteCallingConventions()
@@ -203,23 +242,70 @@ class Clinic(Analysis):
         return ail_block
 
     @timethis
-    def _simplify_blocks(self, stack_pointer_tracker=None):
+    def _replace_single_target_indirect_transitions(self, ail_graph: networkx.DiGraph) -> networkx.DiGraph:
+        """
+        Remove single-target indirect jumps and calls and replace them with direct jumps or calls.
+        """
+        if self._cfg is None:
+            return ail_graph
+
+        for block in ail_graph.nodes():
+            if not block.statements:
+                continue
+            last_stmt = block.statements[-1]
+            if isinstance(last_stmt, ailment.Stmt.Call) and not isinstance(last_stmt.target, ailment.Expr.Const):
+                # indirect call
+                # consult CFG to see if this is a call with a single successor
+                node = self._cfg.get_any_node(block.addr)
+                if node is None:
+                    continue
+                successors = self._cfg.get_successors(node, excluding_fakeret=True, jumpkind='Ijk_Call')
+                if len(successors) == 1 and \
+                        not isinstance(self.project.hooked_by(successors[0].addr), UnresolvableCallTarget):
+                    # found a single successor - replace the last statement
+                    new_last_stmt = last_stmt.copy()
+                    new_last_stmt.target = ailment.Expr.Const(None, None, successors[0].addr, last_stmt.target.bits)
+                    block.statements[-1] = new_last_stmt
+
+            elif isinstance(last_stmt, ailment.Stmt.Jump) and not isinstance(last_stmt.target, ailment.Expr.Const):
+                # indirect jump
+                # consult CFG to see if there is a jump with a single successor
+                node = self._cfg.get_any_node(block.addr)
+                if node is None:
+                    continue
+                successors = self._cfg.get_successors(node, excluding_fakeret=True, jumpkind='Ijk_Boring')
+                if len(successors) == 1 and \
+                        not isinstance(self.project.hooked_by(successors[0].addr), UnresolvableJumpTarget):
+                    # found a single successor - replace the last statement
+                    new_last_stmt = last_stmt.copy()
+                    new_last_stmt.target = ailment.Expr.Const(None, None, successors[0].addr, last_stmt.target.bits)
+                    block.statements[-1] = new_last_stmt
+
+        return ail_graph
+
+    @timethis
+    def _make_ailgraph(self) -> networkx.DiGraph:
+        graph = self._function_graph_to_ail_graph(self._func_graph)
+        return graph
+
+    @timethis
+    def _simplify_blocks(self, ail_graph: networkx.DiGraph, stack_pointer_tracker=None):
         """
         Simplify all blocks in self._blocks.
 
+        :param ail_graph:               The AIL function graph.
         :param stack_pointer_tracker:   The RegisterDeltaTracker analysis instance.
         :return:                        None
         """
 
-        # First of all, let's simplify blocks one by one
+        blocks_by_addr_and_size = {}
 
-        for key in self._blocks_by_addr_and_size:
-            ail_block = self._blocks_by_addr_and_size[key]
+        for ail_block in ail_graph.nodes():
             simplified = self._simplify_block(ail_block, stack_pointer_tracker=stack_pointer_tracker)
-            self._blocks_by_addr_and_size[key] = simplified
+            key = ail_block.addr, ail_block.original_size
+            blocks_by_addr_and_size[key] = simplified
 
-        # Update the function graph so that we can use reaching definitions
-        graph = self._function_graph_to_ail_graph(self._func_graph)
+        graph = self._function_graph_to_ail_graph(self._func_graph, blocks_by_addr_and_size=blocks_by_addr_and_size)
         return graph
 
     def _simplify_block(self, ail_block, stack_pointer_tracker=None):
@@ -235,13 +321,25 @@ class Clinic(Analysis):
             ail_block,
             remove_dead_memdefs=self._remove_dead_memdefs,
             stack_pointer_tracker=stack_pointer_tracker,
+            peephole_optimizations=self.peephole_optimizations,
         )
         return simp.result_block
 
     @timethis
-    def _simplify_function(self, ail_graph):
+    def _simplify_function(self, ail_graph, unify_variables=False, max_iterations: int=4) -> None:
         """
-        Simplify the entire function.
+        Simplify the entire function until it reaches a fixed point.
+        """
+
+        for _ in range(max_iterations):
+            simplified = self._simplify_function_once(ail_graph, unify_variables=unify_variables)
+            if not simplified:
+                break
+
+    @timethis
+    def _simplify_function_once(self, ail_graph, unify_variables=False):
+        """
+        Simplify the entire function once.
 
         :return:    None
         """
@@ -254,30 +352,41 @@ class Clinic(Analysis):
             self.function,
             func_graph=ail_graph,
             remove_dead_memdefs=self._remove_dead_memdefs,
-            reaching_definitions=rd
+            reaching_definitions=rd,
+            unify_variables=unify_variables,
         )
+        if not simp.simplified:
+            return False
 
         def _handler(node):
             return simp.blocks.get(node, None)
 
         AILGraphWalker(ail_graph, _handler, replace_nodes=True).walk()
 
-    @timethis
-    def _run_simplification_passes(self, ail_graph):
+        return True
 
-        blocks_map = defaultdict(set)
+    @timethis
+    def _run_simplification_passes(self, ail_graph, stage:int=OptimizationPassStage.AFTER_GLOBAL_SIMPLIFICATION):
+
+        addr_and_idx_to_blocks: Dict[Tuple[int,Optional[int]],ailment.Block] = { }
+        addr_to_blocks: Dict[int,Set[ailment.Block]] = defaultdict(set)
 
         # update blocks_map to allow node_addr to node lookup
         def _updatedict_handler(node):
-            blocks_map[node.addr].add(node)
+            addr_and_idx_to_blocks[(node.addr, node.idx)] = node
+            addr_to_blocks[node.addr].add(node)
         AILGraphWalker(ail_graph, _updatedict_handler).walk()
 
         # Run each pass
         for pass_ in self._optimization_passes:
 
+            if pass_.STAGE != stage:
+                continue
+
             analysis = getattr(self.project.analyses, pass_.__name__)
 
-            a = analysis(self.function, blocks=blocks_map, graph=ail_graph)
+            a = analysis(self.function, blocks_by_addr=addr_to_blocks, blocks_by_addr_and_idx=addr_and_idx_to_blocks,
+                         graph=ail_graph)
             if a.out_graph:
                 # use the new graph
                 ail_graph = a.out_graph
@@ -330,7 +439,10 @@ class Clinic(Analysis):
             csm = self.project.analyses.AILCallSiteMaker(block, reaching_definitions=rd)
             if csm.result_block:
                 ail_block = csm.result_block
-                simp = self.project.analyses.AILBlockSimplifier(ail_block, stack_pointer_tracker=stack_pointer_tracker)
+                simp = self.project.analyses.AILBlockSimplifier(ail_block,
+                                                                stack_pointer_tracker=stack_pointer_tracker,
+                                                                peephole_optimizations=self.peephole_optimizations,
+                                                                )
                 return simp.result_block
             return None
 
@@ -338,6 +450,48 @@ class Clinic(Analysis):
 
         return ail_graph
 
+    @timethis
+    def _make_returns(self, ail_graph: networkx.DiGraph) -> networkx.DiGraph:
+        """
+        Work on each return statement and fill in its return expressions.
+        """
+
+        if self.function.calling_convention is None:
+            # unknown calling convention. cannot do much about return expressions.
+            return ail_graph
+
+        # Block walker
+
+        def _handle_Return(stmt_idx: int, stmt: ailment.Stmt.Return, block: Optional[ailment.Block]):  # pylint:disable=unused-argument
+            if block is not None \
+                    and not stmt.ret_exprs \
+                    and self.function.calling_convention.ret_val is not None:
+                new_stmt = stmt.copy()
+                ret_val = self.function.calling_convention.ret_val
+                if isinstance(ret_val, SimRegArg):
+                    reg = self.project.arch.registers[ret_val.reg_name]
+                    new_stmt.ret_exprs.append(ailment.Expr.Register(None, None, reg[0],
+                                                                    reg[1] * self.project.arch.byte_width))
+                else:
+                    l.warning("Unsupported type of return expression %s.",
+                              type(self.function.calling_convention.ret_val))
+                block.statements[stmt_idx] = new_stmt
+
+
+        def _handler(block):
+            walker = AILBlockWalker()
+            # we don't need to handle any statement besides Returns
+            walker.stmt_handlers.clear()
+            walker.expr_handlers.clear()
+            walker.stmt_handlers[ailment.Stmt.Return] = _handle_Return
+            walker.walk(block)
+
+        # Graph walker
+
+        AILGraphWalker(ail_graph, _handler, replace_nodes=True).walk()
+
+        return ail_graph
+
     @timethis
     def _make_function_prototype(self, arg_list: List[SimVariable], variable_kb):
         if self.function.prototype is not None:
@@ -388,8 +542,9 @@ class Clinic(Analysis):
         vr = self.project.analyses.VariableRecoveryFast(self.function,  # pylint:disable=unused-variable
                                                         func_graph=ail_graph, kb=tmp_kb, track_sp=False,
                                                         func_args=arg_list)
-        # clean up existing types
+        # clean up existing types for this function
         tmp_kb.variables[self.function.addr].remove_types()
+        # TODO: Type inference for global variables
         # run type inference
         try:
             tp = self.project.analyses.Typehoon(vr.type_constraints, kb=tmp_kb, var_mapping=vr.var_to_typevar)
@@ -398,7 +553,9 @@ class Clinic(Analysis):
             l.warning("Typehoon analysis failed. Variables will not have types. Please report to GitHub.",
                       exc_info=True)
 
-        # TODO: The current mapping implementation is kinda hackish...
+        # Unify SSA variables
+        tmp_kb.variables[self.function.addr].unify_variables()
+        tmp_kb.variables[self.function.addr].assign_unified_variable_names(labels=self.kb.labels, reset=True)
 
         # Link variables to each statement
         for block in ail_graph.nodes():
@@ -447,13 +604,25 @@ class Clinic(Analysis):
                 self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt, stmt.condition)
 
             elif stmt_type is ailment.Stmt.Call:
-                if stmt.args:
-                    for arg in stmt.args:
-                        self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt,
-                                                     arg)
-                if stmt.ret_expr:
-                    self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt,
-                                                 stmt.ret_expr)
+                self._link_variables_on_call(variable_manager, global_variables, block, stmt_idx, stmt, is_expr=False)
+
+            elif stmt_type is ailment.Stmt.Return:
+                self._link_variables_on_return(variable_manager, global_variables, block, stmt_idx, stmt)
+
+    def _link_variables_on_return(self, variable_manager, global_variables, block: ailment.Block, stmt_idx: int,
+                                  stmt: ailment.Stmt.Return):
+        if stmt.ret_exprs:
+            for ret_expr in stmt.ret_exprs:
+                self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt, ret_expr)
+
+    def _link_variables_on_call(self, variable_manager, global_variables, block, stmt_idx, stmt, is_expr=False):
+        if stmt.args:
+            for arg in stmt.args:
+                self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt,
+                                             arg)
+        if not is_expr and stmt.ret_expr:
+            self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt,
+                                         stmt.ret_expr)
 
     def _link_variables_on_expr(self, variable_manager, global_variables, block, stmt_idx, stmt, expr):
         """
@@ -477,10 +646,15 @@ class Clinic(Analysis):
                 expr.variable_offset = offset
 
         elif type(expr) is ailment.Expr.Load:
-            # import ipdb; ipdb.set_trace()
             variables = variable_manager.find_variables_by_atom(block.addr, stmt_idx, expr)
             if len(variables) == 0:
+                # this is a local variable
                 self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt, expr.addr)
+                if 'reference_variable' in expr.addr.tags and expr.addr.reference_variable is not None:
+                    # copy over the variable to this expr since the variable on a constant is supposed to be a
+                    # reference variable.
+                    expr.variable = expr.addr.reference_variable
+                    expr.variable_offset = expr.addr.reference_variable_offset
             else:
                 if len(variables) > 1:
                     l.error("More than one variable are available for atom %s. Consider fixing it using phi nodes.",
@@ -524,16 +698,22 @@ class Clinic(Analysis):
             variables = global_variables.get_global_variables(expr.value)
             if variables:
                 var = next(iter(variables))
-                expr.variable = var
-                expr.variable_offset = 0
+                expr.tags['reference_variable'] = var
+                expr.tags['reference_variable_offset'] = None
 
-    def _function_graph_to_ail_graph(self, func_graph):
+        elif isinstance(expr, ailment.Stmt.Call):
+            self._link_variables_on_call(variable_manager, global_variables, block, stmt_idx, expr, is_expr=True)
+
+    def _function_graph_to_ail_graph(self, func_graph, blocks_by_addr_and_size=None):
+
+        if blocks_by_addr_and_size is None:
+            blocks_by_addr_and_size = self._blocks_by_addr_and_size
 
         node_to_block_mapping = {}
         graph = networkx.DiGraph()
 
         for node in func_graph.nodes():
-            ail_block = self._blocks_by_addr_and_size.get((node.addr, node.size), node)
+            ail_block = blocks_by_addr_and_size.get((node.addr, node.size), node)
             node_to_block_mapping[node] = ail_block
 
             if ail_block is not None:
