@@ -9,7 +9,7 @@ from ...calling_conventions import DEFAULT_CC, SimRegArg, SimStackArg
 from ...knowledge_plugins.key_definitions.atoms import Register, Tmp, MemoryLocation
 from ...knowledge_plugins.key_definitions.constants import OP_BEFORE, OP_AFTER
 from ...knowledge_plugins.key_definitions.dataset import DataSet
-from ...knowledge_plugins.key_definitions.undefined import Undefined, undefined
+from ...knowledge_plugins.key_definitions.undefined import Undefined, UNDEFINED
 from ...knowledge_plugins.key_definitions.live_definitions import Definition
 from .external_codeloc import ExternalCodeLocation
 from .rd_state import ReachingDefinitionsState
@@ -88,7 +88,7 @@ class SimEngineRDAIL(
         dst = stmt.dst
 
         if src is None:
-            src = DataSet(undefined, dst.bits)
+            src = DataSet(UNDEFINED, dst.bits)
 
         if isinstance(dst, ailment.Tmp):
             self.state.kill_and_add_definition(Tmp(dst.tmp_idx, dst.size), self._codeloc(), src)
@@ -149,7 +149,10 @@ class SimEngineRDAIL(
             self.state.kill_definitions(Register(*self.arch.registers['cc_dep2']), self._codeloc())
             self.state.kill_definitions(Register(*self.arch.registers['cc_ndep']), self._codeloc())
 
-    def _ail_handle_Call(self, stmt):
+    def _ail_handle_Call(self, stmt: ailment.Stmt.Call):
+        self._handle_Call_base(stmt, is_expr=False)
+
+    def _handle_Call_base(self, stmt: ailment.Stmt.Call, is_expr: bool=False):
         target = self._expr(stmt.target)  # pylint:disable=unused-variable
 
         ip = Register(self.arch.ip_offset, self.arch.bytes)
@@ -183,7 +186,6 @@ class SimEngineRDAIL(
             cc = self.project.factory.cc()
 
         killed_vars = [ Register(*self.arch.registers[reg_name]) for reg_name in cc.CALLER_SAVED_REGS ]
-        return_reg_offset, return_reg_size = self.arch.registers[cc.RETURN_VAL.reg_name]
 
         # Add uses
         if used_exprs is None:
@@ -191,8 +193,23 @@ class SimEngineRDAIL(
         for expr in used_exprs:
             self._expr(expr)
 
-        # Return value is redefined here, so it is not a dummy value
-        self.state.kill_definitions(Register(return_reg_offset, return_reg_size), self._codeloc(), dummy=False)
+        # Add definition
+        return_reg_offset = None
+        if not is_expr:
+            if stmt.ret_expr is not None:
+                if isinstance(stmt.ret_expr, ailment.Expr.Register):
+                    return_reg_offset = stmt.ret_expr.reg_offset
+                    return_reg_size = stmt.ret_expr.size
+                    self.state.kill_and_add_definition(Register(return_reg_offset, return_reg_size), self._codeloc(),
+                                                       None)
+                else:
+                    l.warning("Unsupported ret_expr type %s. Please report to GitHub.", stmt.ret_expr.__class__)
+
+            else:
+                # Return value is redefined here, so it is not a dummy value
+                return_reg_offset, return_reg_size = self.arch.registers[cc.RETURN_VAL.reg_name]
+                self.state.kill_definitions(Register(return_reg_offset, return_reg_size), self._codeloc(), dummy=False)
+
         # Kill those ones that should be killed
         for var in killed_vars:
             if var.reg_offset == return_reg_offset:
@@ -207,7 +224,16 @@ class SimEngineRDAIL(
             self.state.kill_definitions(Register(*self.arch.registers['cc_dep2']), self._codeloc())
             self.state.kill_definitions(Register(*self.arch.registers['cc_ndep']), self._codeloc())
 
-    def _ail_handle_Return(self, stmt):  # pylint:disable=unused-argument
+    def _ail_handle_Return(self, stmt: ailment.Stmt.Return):  # pylint:disable=unused-argument
+
+        if stmt.ret_exprs:
+            # Handle return expressions
+            for ret_expr in stmt.ret_exprs:
+                self._expr(ret_expr)
+
+            return
+
+        # No return expressions are available.
         # consume registers that are potentially useful
         # TODO: Consider the calling convention of the current function
 
@@ -256,6 +282,10 @@ class SimEngineRDAIL(
 
         return super(SimEngineRDAIL, self)._ail_handle_Tmp(expr)
 
+    def _ail_handle_CallExpr(self, expr: ailment.Stmt.Call):
+        self._handle_Call_base(expr, is_expr=True)
+        return DataSet(UNDEFINED, expr.bits)
+
     def _ail_handle_Register(self, expr):
 
         reg_offset = expr.reg_offset
@@ -338,14 +368,14 @@ class SimEngineRDAIL(
                     if any(type(d) is Undefined for d in data):
                         l.info('Stack access at offset %#x undefined, ins_addr = %#x.', addr.offset, self.ins_addr)
                 else:
-                    data.add(undefined)
+                    data.add(UNDEFINED)
 
                 self.state.add_use(MemoryLocation(addr, size), self._codeloc())
             else:
                 l.debug('Memory address %r undefined or unsupported at pc %#x.', addr, self.ins_addr)
 
         if len(data) == 0:
-            data.add(undefined)
+            data.add(UNDEFINED)
 
         return DataSet(data, bits)
 
@@ -388,7 +418,7 @@ class SimEngineRDAIL(
                 r = DataSet(converted, expr.to_bits)
 
         if r is None:
-            r = DataSet(undefined, expr.to_bits)
+            r = DataSet(UNDEFINED, expr.to_bits)
 
         return r
 
@@ -401,7 +431,7 @@ class SimEngineRDAIL(
     def _ail_handle_BinaryOp(self, expr):
         r = super()._ail_handle_BinaryOp(expr)
         if isinstance(r, ailment.Expr.BinaryOp):
-            return DataSet(undefined, r.bits)
+            return DataSet(UNDEFINED, r.bits)
         return r
 
     def _ail_handle_Cmp(self, expr):
@@ -416,9 +446,13 @@ class SimEngineRDAIL(
     _ail_handle_CmpEQ = _ail_handle_Cmp
     _ail_handle_CmpNE = _ail_handle_Cmp
     _ail_handle_CmpLE = _ail_handle_Cmp
+    _ail_handle_CmpLEs = _ail_handle_Cmp
     _ail_handle_CmpLT = _ail_handle_Cmp
+    _ail_handle_CmpLTs = _ail_handle_Cmp
     _ail_handle_CmpGE = _ail_handle_Cmp
+    _ail_handle_CmpGEs = _ail_handle_Cmp
     _ail_handle_CmpGT = _ail_handle_Cmp
+    _ail_handle_CmpGTs = _ail_handle_Cmp
 
     def _ail_handle_Const(self, expr):
         return DataSet(expr.value, expr.bits)
@@ -432,7 +466,7 @@ class SimEngineRDAIL(
                        )
 
     def _ail_handle_DirtyExpression(self, expr):  # pylint:disable=no-self-use
-        return DataSet(undefined, expr.bits // 8)
+        return DataSet(UNDEFINED, expr.bits // 8)
 
     #
     # User defined high-level statement handlers
