@@ -386,13 +386,9 @@ class SimSuccessors:
         :return: None
         """
 
-        try:
-            bypass = o.BYPASS_UNSUPPORTED_SYSCALL in state.options
-            stub = state.project.simos.syscall(state, allow_unsupported=bypass)
-            if stub: # can be None if simos is not a subclass of SimUserspace
-                state.ip = stub.addr # fix the IP
-        except AngrUnsupportedSyscallError:
-            pass # the syscall is not supported. don't do anything
+        stub = state.project.simos.syscall(state, allow_unsupported=True)
+        if stub: # can be None if simos is not a subclass of SimUserspace
+            state.ip = stub.addr # fix the IP
 
     def _finalize(self):
         """
@@ -522,5 +518,5 @@ from ..errors import SimSolverModeError, AngrUnsupportedSyscallError, AngrSyscal
 from ..calling_conventions import SYSCALL_CC
 from ..state_plugins.sim_action_object import _raw_ast
 from ..state_plugins.callstack import CallStack
-from ..storage.memory import DUMMY_SYMBOLIC_READ_VALUE
+from ..storage import DUMMY_SYMBOLIC_READ_VALUE
 from .. import sim_options as o
