@@ -1,10 +1,14 @@
-from typing import List
+from typing import List, Dict
 import logging
+import cle
+
+from capstone import CS_GRP_CALL, CS_GRP_IRET, CS_GRP_JUMP, CS_GRP_RET
 
 from . import ExplorationTechnique
 from .. import BP_BEFORE, BP_AFTER, sim_options
 from ..errors import AngrTracerError
 
+
 l = logging.getLogger(name=__name__)
 
 
@@ -34,14 +38,15 @@ class RepHook:
     def __init__(self, mnemonic):
         self.mnemonic = mnemonic
 
-    def _inline_call(self, state, procedure, *arguments, **kwargs):
+    @staticmethod
+    def _inline_call(state, procedure, *arguments, **kwargs):
         e_args = [state.solver.BVV(a, state.arch.bits) if isinstance(a, int) else a for a in arguments]
         p = procedure(project=state.project, **kwargs)
         return p.execute(state, None, arguments=e_args)
 
     def run(self, state):
 
-        from .. import SIM_PROCEDURES
+        from .. import SIM_PROCEDURES # pylint: disable=import-outside-toplevel
 
         dst = state.regs.edi if state.arch.name == "X86" else state.regs.rdi
 
@@ -105,7 +110,7 @@ class RepHook:
                 state.regs.rcx = 0
 
         else:
-            import ipdb; ipdb.set_trace()
+            raise NotImplementedError("Unsupported mnemonic %s" % self.mnemonic)
 
 
 class Tracer(ExplorationTechnique):
@@ -130,6 +135,8 @@ class Tracer(ExplorationTechnique):
                                 in order to set the predecessors list correctly. If you turn this
                                 on you may want to enable the LAZY_SOLVES option.
     :param mode:                Tracing mode.
+    :param aslr:                Whether there are aslr slides. if not, tracer uses trace address
+                                as state address.
 
     :ivar predecessors:         A list of states in the history before the final state.
     """
@@ -140,15 +147,19 @@ class Tracer(ExplorationTechnique):
             keep_predecessors=1,
             crash_addr=None,
             copy_states=False,
-            mode=TracingMode.Strict):
-        super(Tracer, self).__init__()
+            fast_forward_to_entry=True,
+            mode=TracingMode.Strict,
+            aslr=True):
+        super().__init__()
         self._trace = trace
         self._resiliency = resiliency
         self._crash_addr = crash_addr
         self._copy_states = copy_states
         self._mode = mode
+        self._aslr = aslr
+        self._fast_forward_to_entry = fast_forward_to_entry
 
-        self._aslr_slides = {}
+        self._aslr_slides = {}  # type: Dict[cle.Backend, int]
         self._current_slide = None
 
         # keep track of the last basic block we hit
@@ -163,40 +174,100 @@ class Tracer(ExplorationTechnique):
         if self._mode == TracingMode.Permissive and not self._copy_states:
             raise ValueError('"copy_states" must be True when tracing in permissive mode.')
 
+    def _locate_entry_point(self, angr_addr):
+        # ...via heuristics
+        indices = set()
+        threshold = 0x40000
+        while not indices and threshold > 0x2000:
+            for idx, addr in enumerate(self._trace):
+                if ((addr - angr_addr) & 0xfff) == 0 and (idx == 0 or abs(self._trace[idx-1] - addr) > threshold):
+                    indices.add(idx)
+
+            indices = set(i for i in indices if self._filter_idx(angr_addr, i))
+            threshold //= 2
+
+        return indices
+
+    def _identify_aslr_slides(self):
+        """
+        libraries can be mapped differently in the original run(in the trace) and in angr
+        this function identifies the difference(called aslr slides) of each library to help angr translate
+        original address and address in angr back and forth
+        """
+        if self._aslr:
+            # if we don't know whether there is any slide, we need to identify the slides via heuristics
+            for obj in self.project.loader.all_elf_objects:
+                # heuristic 1: non-PIC  objects are loaded without aslr slides
+                if not obj.pic:
+                    self._aslr_slides[obj] = 0
+                    continue
+
+                # heuristic 2: library objects with custom_base_addr are loaded at the correct locations
+                if obj._custom_base_addr:
+                    l.info("%s is assumed to be loaded at the address matching the one in the trace", obj)
+                    self._aslr_slides[obj] = 0
+                    continue
+
+                # heuristic 3: entry point of an object should appear in the trace
+                possibilities = None
+                for entry in obj.initializers + ([obj.entry] if obj.is_main_bin else []):
+                    indices = self._locate_entry_point(entry)
+                    slides = {self._trace[idx] - entry for idx in indices}
+                    if possibilities is None:
+                        possibilities = slides
+                    else:
+                        possibilities.intersection_update(slides)
+
+                if possibilities is None:
+                    continue
+
+                if len(possibilities) == 0:
+                    raise AngrTracerError("Trace does not seem to contain object initializers for %s. Do you want to have a Tracer(aslr=False)?" % obj)
+                if len(possibilities) == 1:
+                    self._aslr_slides[obj] = next(iter(possibilities))
+                else:
+                    raise AngrTracerError("Trace seems ambiguous with respect to what the ASLR slides are for %s. This is surmountable, please open an issue." % obj)
+        else:
+            # if we know there is no slides, just trust the address in the loader
+            for obj in self.project.loader.all_elf_objects:
+                self._aslr_slides[obj] = 0
+            self._current_slide = 0
+
+    def _filter_idx(self, angr_addr, idx):
+        slide = self._trace[idx] - angr_addr
+        block = self.project.factory.block(angr_addr)
+        legal_next = block.vex.constant_jump_targets
+        if legal_next:
+            return any(a + slide == self._trace[idx + 1] for a in legal_next)
+        else:
+            # the intuition is that if the first block of an initializer does an indirect jump,
+            # it's probably a call out to another binary (notably __libc_start_main)
+            # this is an awful fucking heuristic but it's as good as we've got
+            return abs(self._trace[idx] - self._trace[idx + 1]) > 0x1000
+
     def setup(self, simgr):
         simgr.populate('missed', [])
         simgr.populate('traced', [])
         simgr.populate('crashed', [])
         simgr.populate('desync', [])
 
-        self.project = simgr._project
         if len(simgr.active) != 1:
             raise AngrTracerError("Tracer is being invoked on a SimulationManager without exactly one active state")
 
-        # calc ASLR slide for main binary and find the entry point in one fell swoop
-        # ...via heuristics
-        for idx, addr in enumerate(self._trace):
-            if self.project.loader.main_object.pic:
-                if ((addr - self.project.entry) & 0xfff) == 0 and (idx == 0 or abs(self._trace[idx-1] - addr) > 0x100000):
-                    break
-            else:
-                if addr == self.project.entry:
-                    break
+        self._identify_aslr_slides()
+
+        if self._fast_forward_to_entry:
+            idx = self._trace.index(self._translate_state_addr(self.project.entry))
+            # step to entry point
+            while simgr.one_active.addr != self.project.entry:
+                simgr.step(extra_stop_points={self.project.entry})
+                if len(simgr.active) == 0:
+                    raise AngrTracerError("Could not step to the first address of the trace - simgr is empty")
+                if len(simgr.active) > 1:
+                    raise AngrTracerError("Could not step to the first address of the trace - state split. Do you want to have a Tracer(fast_forward_to_entry=False)?")
+                simgr.drop(stash='unsat')
         else:
-            raise AngrTracerError("Could not identify program entry point in trace!")
-
-        # pylint: disable=undefined-loop-variable
-        # pylint doesn't know jack shit
-        self._current_slide = self._aslr_slides[self.project.loader.main_object] = self._trace[idx] - self.project.entry
-
-        # step to entry point
-        while self._trace and self._trace[idx] != simgr.one_active.addr + self._current_slide:
-            simgr.step(extra_stop_points={self._trace[idx] - self._current_slide})
-            if len(simgr.active) == 0:
-                raise AngrTracerError("Could not step to the first address of the trace - simgr is empty")
-            elif len(simgr.active) > 1:
-                raise AngrTracerError("Could not step to the first address of the trace - state split")
-            simgr.drop(stash='unsat')
+            idx = 0
 
         # initialize the state info
         simgr.one_active.globals['trace_idx'] = idx
@@ -250,17 +321,21 @@ class Tracer(ExplorationTechnique):
         # optimization:
         # look forward, is it a rep stos/movs instruction?
         # if so, we add a temporary hook to speed up constraint solving
-        block = self.project.factory.block(state.addr)
-        if len(block.capstone.insns) == 1 and (
-                block.capstone.insns[0].mnemonic.startswith("rep m") or
-                block.capstone.insns[0].mnemonic.startswith("rep s")
-        ) and not self.project.is_hooked(state.addr):
-            insn = block.capstone.insns[0]
-            self.project.hook(state.addr, RepHook(insn.mnemonic.split(" ")[1]).run, length=insn.size)
+        if not self.project.is_hooked(state.addr):
+            block = self.project.factory.block(state.addr)
+
+            if len(block.capstone.insns) == 1 and (
+                    block.capstone.insns[0].mnemonic.startswith("rep m") or
+                    block.capstone.insns[0].mnemonic.startswith("rep s")
+            ):
+                insn = block.capstone.insns[0]
+                self.project.hook(state.addr, RepHook(insn.mnemonic.split(" ")[1]).run, length=insn.size)
 
         # perform the step. ask qemu to stop at the termination point.
         stops = set(kwargs.pop('extra_stop_points', ())) | {self._trace[-1]}
         succs_dict = simgr.step_state(state, extra_stop_points=stops, **kwargs)
+        if None not in succs_dict and simgr.errored:
+            raise simgr.errored[-1].error
         sat_succs = succs_dict[None]  # satisfiable states
         succs = sat_succs + succs_dict['unsat']  # both satisfiable and unsatisfiable states
 
@@ -361,6 +436,15 @@ class Tracer(ExplorationTechnique):
             try:
                 if self._compare_addr(self._trace[idx + 1], succ.addr):
                     res.append(succ)
+                else:
+                    *_, last_description = succ.history.descriptions
+                    if 'Unicorn' in last_description:
+                        # A new state was created in SimEngineUnicorn. Check every recent basic block to see if any
+                        # match the next expected index
+                        for bbl_addr in succ.history.recent_bbl_addrs:
+                            if self._compare_addr(self._trace[idx + 1], bbl_addr):
+                                res.append(succ)
+                                break
             except AngrTracerError:
                 pass
 
@@ -394,16 +478,22 @@ class Tracer(ExplorationTechnique):
                     continue
 
 
-                if self._compare_addr(self._trace[idx], addr):
+                if self._compare_addr(self._trace[idx], addr) or self._check_qemu_unicorn_desync(state, idx, addr):
                     idx += 1
                 else:
-                    raise TracerDesyncError('BUG! Please investigate the claim in the comment above me',
+                    raise TracerDesyncError('Oops! angr did not follow the trace',
                                             deviating_addr=addr,
                                             deviating_trace_idx=idx)
 
             idx -= 1 # use normal code to do the last synchronization
 
-        if sync is not None:
+        if sync == 'entry':
+            trace_addr = self._translate_state_addr(state.addr)
+            # this address should only ever appear once in the trace. we verified this during setup.
+            idx = self._trace.index(trace_addr)
+            state.globals['trace_idx'] = idx
+            state.globals['sync_idx'] = None
+        elif sync is not None:
             timer -= 1
             if self._compare_addr(self._trace[sync], state.addr):
                 state.globals['trace_idx'] = sync
@@ -412,11 +502,14 @@ class Tracer(ExplorationTechnique):
             elif timer > 0:
                 state.globals['sync_timer'] = timer
             else:
-                raise Exception("Trace failed to synchronize! We expected it to hit %#x (untranslated), but it failed to do this within a timeout" % self._trace[sync])
+                raise Exception("Trace failed to synchronize! We expected it to hit %#x (trace addr), but it failed to do this within a timeout" % self._trace[sync])
 
         elif state.history.jumpkind.startswith('Ijk_Exit'):
             # termination! will be handled by filter
             pass
+        elif self.project.is_hooked(state.addr) and not self.project.loader.extern_object.contains_addr(state.addr):
+           # handle simprocedures
+           self._sync_return(state, idx)
         elif self._compare_addr(self._trace[idx + 1], state.addr):
             # normal case
             state.globals['trace_idx'] = idx + 1
@@ -425,9 +518,12 @@ class Tracer(ExplorationTechnique):
             proc = self.project.hooked_by(state.addr)
             if proc is None:
                 raise Exception("Extremely bad news: we're executing an unhooked address in the externs space")
-            if proc.is_continuation:
+            if proc.display_name == 'LinuxLoader':
+                state.globals['sync_idx'] = 'entry'
+            elif proc.is_continuation:
                 orig_addr = self.project.loader.find_symbol(proc.display_name).rebased_addr
-                orig_trace_addr = orig_addr + self._aslr_slides[self.project.loader.find_object_containing(orig_addr)]
+                obj = self.project.loader.find_object_containing(orig_addr)
+                orig_trace_addr = self._translate_state_addr(orig_addr, obj)
                 if 0 <= self._trace[idx + 1] - orig_trace_addr <= 0x10000:
                     # this is fine. we do nothing and then next round it'll get handled by the is_hooked(state.history.addr) case
                     pass
@@ -437,6 +533,9 @@ class Tracer(ExplorationTechnique):
             elif state.addr == getattr(self.project.simos, 'vsyscall_addr', None):
                 if not self._sync_callsite(state, idx, state.history.addr):
                     raise AngrTracerError("Could not synchronize following vsyscall")
+            elif self.project.hooked_by(state.addr).display_name.startswith('IFuncResolver'):
+                if not self._sync_return(state, idx):
+                    raise AngrTracerError("Could not synchronize at ifunc return address")
             else:
                 # see above
                 pass
@@ -451,6 +550,11 @@ class Tracer(ExplorationTechnique):
             # we may have prematurely stopped because of setting stop points. try to resync.
             state.globals['sync_idx'] = idx + 1
             state.globals['sync_timer'] = 1
+        elif self.project.is_hooked(state.addr) and \
+                self.project.loader.find_symbol(self.project.hooked_by(state.addr).display_name) is not None and \
+                self.project.loader.find_symbol(self.project.hooked_by(state.addr).display_name).subtype.value[0] == 10:  # STT_GNU_IFUNC
+            if not self._sync_return(state, idx):
+                raise AngrTracerError("Could not synchronize at ifunc return address")
         elif self._analyze_misfollow(state, idx):
             # misfollow analysis will set a sync point somewhere if it succeeds
             pass
@@ -460,9 +564,9 @@ class Tracer(ExplorationTechnique):
                                     deviating_trace_idx=idx+1)
 
         if state.globals['sync_idx'] is not None:
-            l.debug("Trace: %d-%d/%d synchronizing %d", state.globals['trace_idx'], state.globals['sync_idx'], len(self._trace), state.globals['sync_timer'])
+            l.debug("Trace: %s-%s/%s synchronizing %s", state.globals['trace_idx'], state.globals['sync_idx'], len(self._trace), state.globals['sync_timer'])
         else:
-            l.debug("Trace: %d/%d", state.globals['trace_idx'], len(self._trace))
+            l.debug("Trace: %s/%s", state.globals['trace_idx'], len(self._trace))
 
     def _translate_state_addr(self, state_addr, obj=None):
         if obj is None:
@@ -471,7 +575,13 @@ class Tracer(ExplorationTechnique):
             raise Exception("Internal error: cannot translate address")
         return state_addr + self._aslr_slides[obj]
 
-    def _translate_trace_addr(self, trace_addr, obj):
+    def _translate_trace_addr(self, trace_addr, obj=None):
+        if obj is None:
+            for obj in self._aslr_slides:  # pylint: disable=redefined-argument-from-local
+                if obj.contains_addr(trace_addr - self._aslr_slides[obj]):
+                    break
+            else:
+                raise Exception("Can't figure out which object this address belongs to")
         if obj not in self._aslr_slides:
             raise Exception("Internal error: object is untranslated")
         return trace_addr - self._aslr_slides[obj]
@@ -487,14 +597,100 @@ class Tracer(ExplorationTechnique):
             self._current_slide = self._aslr_slides[current_bin]
             return trace_addr == state_addr + self._current_slide
         elif ((trace_addr - state_addr) & 0xfff) == 0:
-                self._aslr_slides[current_bin] = self._current_slide = trace_addr - state_addr
-                return True
+            self._aslr_slides[current_bin] = self._current_slide = trace_addr - state_addr
+            return True
         # error handling
         elif current_bin:
             raise AngrTracerError("Trace desynced on jumping into %s. Did you load the right version of this library?" % current_bin.provides)
         else:
             raise AngrTracerError("Trace desynced on jumping into %#x, where no library is mapped!" % state_addr)
 
+    def _check_qemu_unicorn_desync(self, state: 'angr.SimState', trace_curr_idx, state_desync_block_addr):
+        """
+        Check if desync occurred because large blocks are split up at different instructions by qemu and unicorn. This
+        is done by reconstructing part of block executed so far from the trace and state history and checking if they
+        the same
+        """
+
+        control_flow_insn_types = [CS_GRP_CALL, CS_GRP_IRET, CS_GRP_JUMP, CS_GRP_RET]
+
+        prev_trace_block = state.project.factory.block(self._translate_trace_addr(self._trace[trace_curr_idx - 1]))
+        for insn_type in control_flow_insn_types:
+            if prev_trace_block.capstone.insns[-1].group(insn_type):
+                # Previous block ends in a control flow instruction. It is a trace desync.
+                return False
+
+        # The previous block did not end in a control flow instruction. Let's find the start of this big block from
+        # trace: it'll be the first block executed after a control flow instruction.
+        big_block_start_addr = None
+        for trace_block_idx in range(trace_curr_idx - 2, -1, -1):
+            trace_block = state.project.factory.block(self._translate_trace_addr(self._trace[trace_block_idx]))
+            trace_block_last_insn = trace_block.capstone.insns[-1]
+            for insn_type in control_flow_insn_types:
+                if trace_block_last_insn.group(insn_type):
+                    big_block_start_addr = self._translate_trace_addr(self._trace[trace_block_idx + 1])
+                    break
+
+            if big_block_start_addr is not None:
+                break
+        else:
+            # Failed to find end of the big basic block in trace. Treat as trace desync.
+            return False
+
+        # Now we check the part of the state history corresponding to this big basic block to ensure there are no
+        # control flow instructions at end of any blocks in the part. This check moves backwards starting from the
+        # desyncing block to the start of the big block we found earlier
+        state_history_desync_block_found = False
+        for state_history_block_addr in reversed(state.history.bbl_addrs):
+            if not state_history_desync_block_found:
+                if state_history_block_addr == state_desync_block_addr:
+                    # Found the desync block in state history
+                    state_history_desync_block_found = True
+
+                # Either we haven't found the desync block in state history or we just found it. Since we want to check
+                # for control flow instructions from only from block before the desync block, we can continue here
+                continue
+
+            state_history_block = state.project.factory.block(state_history_block_addr)
+            state_history_block_last_insn = state_history_block.capstone.insns[-1]
+            for insn_type in control_flow_insn_types:
+                if state_history_block_last_insn.group(insn_type):
+                    # We haven't found the start of big block according to the trace but found a block ending with a
+                    # control flow instruction. It is a trace desync then.
+                    return False
+
+            if state_history_block_addr == big_block_start_addr:
+                if not state_history_desync_block_found:
+                    # We somehow found the start of the big block in the state history but not the block where the
+                    # desync error happened. Something is seriously wrong!
+                    return False
+                else:
+                    # We found start of the big block and no control flow statements in between that and the block where
+                    # desync happend. All good.
+                    break
+
+        # Let's find the address of the last bytes of the big basic block from the trace
+        big_block_end_addr = None
+        for trace_block_idx in range(trace_curr_idx + 1, len(self._trace)):
+            trace_block = state.project.factory.block(self._translate_trace_addr(self._trace[trace_block_idx]))
+            trace_block_last_insn = trace_block.capstone.insns[-1]
+            for insn_type in control_flow_insn_types:
+                if trace_block_last_insn.group(insn_type):
+                    big_block_end_addr = trace_block.addr + trace_block.size
+                    break
+
+        if big_block_end_addr is None:
+            # Failed to find end of the big basic block in trace. Treat as trace desync.
+            return False
+
+        # At this point, we know the following:
+        # - There is no control flow instruction between big_block_start_addr and big_block_end_addr
+        # - There is no control flow instruction between big_block_start_addr and state_desync_block_addr
+        # - state_desync_block_addr is definitely executed after big_block_start_addr
+        # So it's enough to check if state_desync_block_addr is less than big_block_end_addr to ensure that it
+        # is part of the big block
+        return state_desync_block_addr < big_block_end_addr
+
     def _analyze_misfollow(self, state, idx):
         angr_addr = state.addr
         obj = self.project.loader.find_object_containing(angr_addr)
@@ -574,7 +770,7 @@ class Tracer(ExplorationTechnique):
         try:
             sync_idx = self._trace.index(addr_translated, idx)
         except ValueError:
-            l.error("Trying to synchronize at %#x (%#x) but it does not appear in the trace?")
+            l.error("Trying to synchronize at %#x (%#x) but it does not appear in the trace?", addr_translated, addr)
             return False
 
         state.globals['sync_idx'] = sync_idx
