@@ -1,7 +1,7 @@
 # pylint:disable=no-member
 import pickle
 import logging
-from typing import Optional, List, Dict, Tuple
+from typing import Optional, List, Dict, Tuple, TYPE_CHECKING
 from collections import defaultdict
 
 import networkx
@@ -43,9 +43,9 @@ class CFGModel(Serializable):
 
         # Memory references
         # A mapping between address and the actual data in memory
-        self.memory_data = { }
+        self.memory_data: Dict[int,MemoryData] = { }
         # A mapping between address of the instruction that's referencing the memory data and the memory data itself
-        self.insn_addr_to_memory_data = { }
+        self.insn_addr_to_memory_data: Dict[int,MemoryData] = { }
 
         # Lists of CFGNodes indexed by the address of each block. Don't serialize
         self._nodes_by_addr = defaultdict(list)
@@ -109,7 +109,7 @@ class CFGModel(Serializable):
                 if k == 'jumpkind':
                     edge.jumpkind = cfg_jumpkind_to_pb(v)
                 elif k == 'ins_addr':
-                    edge.ins_addr = v if v is not None else -1
+                    edge.ins_addr = v if v is not None else 0xffff_ffff_ffff_ffff
                 elif k == 'stmt_idx':
                     edge.stmt_idx = v if v is not None else -1
                 else:
@@ -153,7 +153,7 @@ class CFGModel(Serializable):
             for k, v in edge_pb2.data.items():
                 data[k] = pickle.loads(v)
             data['jumpkind'] = cfg_jumpkind_from_pb(edge_pb2.jumpkind)
-            data['ins_addr'] = edge_pb2.ins_addr if edge_pb2.ins_addr != -1 else None
+            data['ins_addr'] = edge_pb2.ins_addr if edge_pb2.ins_addr != 0xffff_ffff_ffff_ffff else None
             data['stmt_idx'] = edge_pb2.stmt_idx if edge_pb2.stmt_idx != -1 else None
             model.graph.add_edge(src, dst, **data)
 
@@ -274,11 +274,7 @@ class CFGModel(Serializable):
                                          cfg_node.size is not None and
                                          cfg_node.addr <= addr < (cfg_node.addr + cfg_node.size)
                                          ):
-                if is_syscall and cfg_node.is_syscall:
-                    results.append(cfg_node)
-                elif is_syscall is False and not cfg_node.is_syscall:
-                    results.append(cfg_node)
-                else:
+                if is_syscall is None or is_syscall == cfg_node.is_syscall:
                     results.append(cfg_node)
 
         return results
