@@ -4,7 +4,7 @@ import networkx
 import string
 import itertools
 from collections import defaultdict
-from typing import Union, Optional, Iterable
+from typing import Union, Optional, Iterable, Set, Generator
 from typing import Type # For some reasons the linter doesn't recognize the use in apply_definition but PyCharm needs it imported to correctly recognize it # pylint: disable=unused-import
 
 from cle.backends.symbol import Symbol
@@ -231,7 +231,7 @@ class Function(Serializable):
 
         for block_addr, block in self._local_blocks.items():
             try:
-                yield self._get_block(block_addr, size=block.size,
+                yield self.get_block(block_addr, size=block.size,
                                       byte_string=block.bytestr if isinstance(block, BlockNode) else None)
             except (SimEngineError, SimMemoryError):
                 pass
@@ -257,7 +257,7 @@ class Function(Serializable):
 
         return self._local_block_addrs
 
-    def _get_block(self, addr, size=None, byte_string=None):
+    def get_block(self, addr, size=None, byte_string=None):
         if addr in self._block_cache:
             b = self._block_cache[addr]
             if size is None or b.size == size:
@@ -277,8 +277,11 @@ class Function(Serializable):
         self._block_cache[addr] = block
         return block
 
+    # compatibility
+    _get_block = get_block
+
     @property
-    def nodes(self):
+    def nodes(self) -> Generator[CodeNode,None,None]:
         return self.transition_graph.nodes()
 
     def get_node(self, addr):
@@ -344,7 +347,7 @@ class Function(Serializable):
                 l.warning("The new calling convention for %r does not have a prototype associated. Using the existing "
                           "function prototype to update the new calling convention. The existing function prototype "
                           "will be removed.", self)
-                self._cc.func_ty = self._prototype
+                self._cc.set_func_type_with_arch(self._prototype)
                 self._prototype = None
             elif self._cc.func_ty is not None and self._prototype is not None:
                 l.warning("The new calling convention for %r already has a prototype associated. The existing function "
@@ -374,7 +377,7 @@ class Function(Serializable):
         :return:    None
         """
         if self._cc:
-            self._cc.func_ty = proto.with_arch(self.project.arch) if proto else None
+            self._cc.set_func_type_with_arch(proto)
         else:
             self._prototype = proto
 
@@ -532,24 +535,6 @@ class Function(Serializable):
 
         return constants
 
-    @property
-    def runtime_values(self):
-        """
-        All of the concrete values used by this function at runtime (i.e., including passed-in arguments and global
-        values).
-        """
-        constants = set()
-        for b in self.block_addrs:
-            for sirsb in self._function_manager._cfg.get_all_irsbs(b):
-                for s in sirsb.successors + sirsb.unsat_successors:
-                    for a in s.history.recent_actions:
-                        for ao in a.all_objects:
-                            if not isinstance(ao.ast, claripy.ast.Base):
-                                constants.add(ao.ast)
-                            elif not ao.ast.symbolic:
-                                constants.add(s.solver.eval(ao.ast))
-        return constants
-
     @property
     def num_arguments(self):
         return len(self._argument_registers) + len(self._argument_stack_variables)
@@ -712,7 +697,7 @@ class Function(Serializable):
             if hooker is not None:
                 binary_name = hooker.library_name
 
-        if binary_name is None and self.binary is not None:
+        if binary_name is None and self.binary is not None and self.binary.binary:
             binary_name = os.path.basename(self.binary.binary)
 
         return binary_name
@@ -1139,7 +1124,7 @@ class Function(Serializable):
 
         for b in self._local_blocks.values():
             # TODO: should I call get_blocks?
-            block = self._get_block(b.addr, size=b.size, byte_string=b.bytestr)
+            block = self.get_block(b.addr, size=b.size, byte_string=b.bytestr)
             common_insns = set(block.instruction_addrs).intersection(ins_addrs)
             if common_insns:
                 blocks.append(b)
@@ -1177,7 +1162,7 @@ class Function(Serializable):
         """
 
         for b in self.blocks:
-            block = self._get_block(b.addr, size=b.size, byte_string=b.bytestr)
+            block = self.get_block(b.addr, size=b.size, byte_string=b.bytestr)
             if insn_addr in block.instruction_addrs:
                 index = block.instruction_addrs.index(insn_addr)
                 if index == len(block.instruction_addrs) - 1:
@@ -1432,7 +1417,7 @@ class Function(Serializable):
         self.prototype = proto
         if self.calling_convention is not None:
             self.calling_convention.args = None
-            self.calling_convention.func_ty = proto
+            self.calling_convention.set_func_type_with_arch(proto)
 
     @staticmethod
     def _addr_to_funcloc(addr):
@@ -1487,6 +1472,19 @@ class Function(Serializable):
         else:
             raise TypeError("calling_convention has to be one of: [SimCC, type(SimCC), None]")
 
+    def functions_called(self) -> Set['Function']:
+        """
+        :return: The set of all functions that can be reached from the function represented by self.
+        """
+        called = set()
+        def _find_called(function_address):
+            successors = set(self._function_manager.callgraph.successors(function_address)) - called
+            for s in successors:
+                called.add(s)
+                _find_called(s)
+        _find_called(self.addr)
+        return { self._function_manager.function(a) for a in called }
+
     def copy(self):
         func = Function(self._function_manager, self.addr, name=self.name, syscall=self.is_syscall)
         func.transition_graph = networkx.DiGraph(self.transition_graph)
