@@ -1,7 +1,6 @@
 
 import logging
 
-import pyvex
 import angr
 
 l = logging.getLogger(name=__name__)
@@ -146,16 +145,16 @@ class __libc_start_main(angr.SimProcedure):
 
     def static_exits(self, blocks):
         # Execute those blocks with a blank state, and then dump the arguments
-        blank_state = angr.SimState(project=self.project, mode="fastpath", memory_backer=self.project.loader.memory)
+        blank_state = angr.SimState(project=self.project, mode="fastpath", cle_memory_backer=self.project.loader.memory,
+                                    add_options={angr.options.SYMBOL_FILL_UNCONSTRAINED_MEMORY,
+                                                 angr.options.SYMBOL_FILL_UNCONSTRAINED_REGISTERS})
         # set up the stack pointer
-        blank_state.regs.sp = 0x7fffffff
+        blank_state.regs.sp = 0x7ffffff0
 
         # Execute each block
         state = blank_state
         for b in blocks:
-            # state.regs.ip = next(iter(stmt for stmt in b.statements if isinstance(stmt, pyvex.IRStmt.IMark))).addr
-            irsb = self.project.factory.default_engine.process(state, b,
-                    force_addr=next(iter(stmt for stmt in b.statements if isinstance(stmt, pyvex.IRStmt.IMark))).addr)
+            irsb = self.project.factory.default_engine.process(state, b, force_addr=b.addr)
             if irsb.successors:
                 state = irsb.successors[0]
             else:
