@@ -4,3 +4,5 @@ from .x86_elf_pic_plt import X86ElfPicPltResolver
 from .jumptable import JumpTableResolver
 from .x86_pe_iat import X86PeIatResolver
 from .amd64_elf_got import AMD64ElfGotResolver
+from .arm_elf_fast import ArmElfFastResolver
+from .const_resolver import ConstantResolver
