@@ -5,7 +5,7 @@ import networkx
 import pyvex
 from . import Analysis
 
-from .code_location import CodeLocation
+from ..code_location import CodeLocation
 from ..errors import SimSolverModeError, SimUnsatError, AngrDDGError
 from ..sim_variable import SimRegisterVariable, SimMemoryVariable, SimTemporaryVariable, SimConstantVariable, \
     SimStackVariable
@@ -92,7 +92,7 @@ class DDGJob(object):
         return "<DDGJob %s, call_depth %d>" % (self.cfg_node, self.call_depth)
 
 
-class LiveDefinitions(object):
+class LiveDefinitions:
     """
     A collection of live definitions with some handy interfaces for definition killing and lookups.
     """
@@ -248,7 +248,7 @@ class LiveDefinitions(object):
 
     def lookup_defs(self, variable, size_threshold=32):
         """
-        Find all definitions of the varaible
+        Find all definitions of the variable.
 
         :param SimVariable variable: The variable to lookup for.
         :param int size_threshold: The maximum bytes to consider for the variable. For example, if the variable is 100
@@ -793,7 +793,7 @@ class DDG(Analysis):
                     if (self._call_depth is None) or \
                             (self._call_depth is not None and 0 <= new_call_depth <= self._call_depth):
                         # Put all reachable successors back to our work-list again
-                        for successor in self._cfg.get_all_successors(node):
+                        for successor in self._cfg.model.get_all_successors(node):
                             nw = DDGJob(successor, new_call_depth)
                             self._worklist_append(nw, worklist, worklist_set)
 
