@@ -5,4 +5,7 @@ from .clinic import Clinic
 from .region_simplifier import RegionSimplifier
 from .decompiler import Decompiler
 from .decompilation_options import options, options_by_category
+from .block_simplifier import BlockSimplifier
+from .callsite_maker import CallSiteMaker
+from .ail_simplifier import AILSimplifier
 from . import optimization_passes
