@@ -5,7 +5,7 @@ from ailment import Expr
 
 from ... import AnalysesHub
 from .engine_base import SimplifierAILEngine, SimplifierAILState
-from .optimization_pass import OptimizationPass
+from .optimization_pass import OptimizationPass, OptimizationPassStage
 
 _l = logging.getLogger(name=__name__)
 
@@ -251,10 +251,11 @@ class DivSimplifier(OptimizationPass):
 
     ARCHES = ["X86", "AMD64"]
     PLATFORMS = ["linux", "windows"]
+    STAGE = OptimizationPassStage.AFTER_GLOBAL_SIMPLIFICATION
 
-    def __init__(self, func, blocks, graph):
+    def __init__(self, func, **kwargs):
 
-        super().__init__(func, blocks=blocks, graph=graph)
+        super().__init__(func, **kwargs)
 
         self.state = SimplifierAILState(self.project.arch)
         self.engine = DivSimplifierAILEngine()
@@ -266,7 +267,7 @@ class DivSimplifier(OptimizationPass):
 
     def _analyze(self, cache=None):
 
-        for block in self._graph.nodes():
+        for block in list(self._graph.nodes()):
             new_block = block
             old_block = None
 
