@@ -8,13 +8,14 @@ l = logging.getLogger(name=__name__)
 
 
 class SimEnginePropagatorBase(SimEngineLight):  # pylint:disable=abstract-method
-    def __init__(self, stack_pointer_tracker=None, project=None):
+    def __init__(self, stack_pointer_tracker=None, project=None, propagate_tmps=True):
         super().__init__()
 
         # Used in the VEX engine
         self._project = project
         self.base_state = None
         self._load_callback = None
+        self._propagate_tmps: bool = propagate_tmps
 
         # Used in the AIL engine
         self._stack_pointer_tracker = stack_pointer_tracker
