@@ -1,4 +1,4 @@
-from typing import Set, Optional
+from typing import Set, Optional, Any, Tuple
 from collections import defaultdict
 import logging
 
@@ -160,7 +160,7 @@ class Equivalence:
         self.atom1 = atom1
 
     def __repr__(self):
-        return "<EqRelation: %r==%r@%r>" % (self.atom0, self.atom1, self.codeloc)
+        return "<Eq@%r: %r==%r>" % (self.codeloc, self.atom0, self.atom1)
 
     def __eq__(self, other):
         return type(other) is Equivalence \
@@ -205,11 +205,11 @@ class PropagatorAILState(PropagatorState):
 
         for o in others:
             state._stack_variables.merge_to_top(o._stack_variables, top=Top(1))
-            state._registers.merge_to_top(o._registers, top=Top(1))
+            state._registers.merge_to_top(o._registers, top=(Top(1), None))
 
         return state
 
-    def store_variable(self, old, new):
+    def store_variable(self, old, new, def_at) -> None:
         if old is None or new is None:
             return
         if type(new) is not Top and new.has_atom(old, identity=False):
@@ -218,11 +218,11 @@ class PropagatorAILState(PropagatorState):
         if isinstance(old, ailment.Expr.Tmp):
             self._tmps[old.tmp_idx] = new
         elif isinstance(old, ailment.Expr.Register):
-            self._registers.set_object(old.reg_offset, new, old.size)
+            self._registers.set_object(old.reg_offset, (new, def_at), old.size)
         else:
             _l.warning("Unsupported old variable type %s.", type(old))
 
-    def store_stack_variable(self, addr, size, new, endness=None):  # pylint:disable=unused-argument
+    def store_stack_variable(self, addr, size, new, endness=None) -> None:  # pylint:disable=unused-argument
         if isinstance(addr, ailment.Expr.StackBaseOffset):
             if addr.offset is None:
                 offset = 0
@@ -232,7 +232,7 @@ class PropagatorAILState(PropagatorState):
         else:
             _l.warning("Unsupported addr type %s.", type(addr))
 
-    def get_variable(self, variable):
+    def get_variable(self, variable) -> Any:
         if isinstance(variable, ailment.Expr.Tmp):
             return self._tmps.get(variable.tmp_idx, None)
         elif isinstance(variable, ailment.Expr.Register):
@@ -240,12 +240,15 @@ class PropagatorAILState(PropagatorState):
             if not objs:
                 return None
             # FIXME: Right now we are always getting one item - we should, in fact, work on a multi-value domain
-            first_obj = next(iter(objs))
+            obj: Tuple[Any,Optional[CodeLocation]] = next(iter(objs))
+            first_obj, def_at = obj
+
             if type(first_obj) is Top:
                 # return a Top
                 if first_obj.bits != variable.bits:
                     return Top(variable.bits // 8)
                 return first_obj
+
             if first_obj.bits != variable.bits:
                 # conversion is needed
                 if isinstance(first_obj, ailment.Expr.Convert):
@@ -256,7 +259,10 @@ class PropagatorAILState(PropagatorState):
                                                          first_obj.is_signed, first_obj.operand)
                 else:
                     first_obj = ailment.Expr.Convert(first_obj.idx, first_obj.bits, variable.bits, False, first_obj)
-            return first_obj
+
+            v = first_obj.copy()
+            v.tags['def_at'] = def_at  # put def_at into tags
+            return v
         return None
 
     def get_stack_variable(self, addr, size, endness=None):  # pylint:disable=unused-argument
@@ -269,6 +275,16 @@ class PropagatorAILState(PropagatorState):
 
     def add_replacement(self, codeloc, old, new):
 
+        if isinstance(new, ailment.statement.Call):
+            # do not replace anything with a call expression
+            return
+
+        if type(new) is Top:
+            # eliminate the past propagation of this expression
+            if codeloc in self._replacements and old in self._replacements[codeloc]:
+                del self._replacements[codeloc][old]
+            return
+
         prop_count = 0
         if not isinstance(old, ailment.Expr.Tmp) and isinstance(new, ailment.Expr.Expression) \
                 and not isinstance(new, ailment.Expr.Const):
@@ -350,7 +366,11 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
         self.equivalence: Set[Equivalence] = set()
 
         self._engine_vex = SimEnginePropagatorVEX(project=self.project)
-        self._engine_ail = SimEnginePropagatorAIL(stack_pointer_tracker=self._stack_pointer_tracker)
+        self._engine_ail = SimEnginePropagatorAIL(
+            stack_pointer_tracker=self._stack_pointer_tracker,
+            # We only propagate tmps within the same block. This is because the lifetime of tmps is one block only.
+            propagate_tmps=block is not None,
+        )
 
         self._analyze()
 
@@ -384,7 +404,7 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
 
         if isinstance(node, ailment.Block):
             block = node
-            block_key = node.addr
+            block_key = (node.addr, node.idx)
             engine = self._engine_ail
         else:
             block = self.project.factory.block(node.addr, node.size, opt_level=1, cross_insn_opt=False)
