@@ -1,12 +1,16 @@
-from typing import Optional, Set
+from typing import Optional, Set, List, Tuple, TYPE_CHECKING
 import logging
 
 from ...engines.light import SimEngineLight, SpOffset, ArithmeticExpression
 from ...errors import SimEngineError
-from ...sim_variable import SimVariable, SimStackVariable, SimRegisterVariable
+from ...sim_variable import SimVariable, SimStackVariable, SimRegisterVariable, SimMemoryVariable
 from ...code_location import CodeLocation
 from ..typehoon import typevars, typeconsts
 
+if TYPE_CHECKING:
+    from .variable_recovery_base import VariableRecoveryStateBase
+    from angr.knowledge_plugins.variables.variable_manager import VariableManager
+
 #
 # The base engine used in VariableRecoveryFast
 #
@@ -46,7 +50,7 @@ class SimEngineVRBase(SimEngineLight):
         self.project = project
         self.kb = kb
         self.processor_state = None
-        self.variable_manager = None
+        self.variable_manager: Optional['VariableManager'] = None
 
     @property
     def func_addr(self):
@@ -72,6 +76,47 @@ class SimEngineVRBase(SimEngineLight):
     # Logic
     #
 
+    def _reference(self, richr: RichR, codeloc: CodeLocation, src=None):
+        data = richr.data
+        stack_offset = data.offset
+        existing_vars: List[Tuple[SimVariable,int]] = self.variable_manager[self.func_addr].find_variables_by_stmt(
+            self.block.addr,
+            self.stmt_idx,
+            'memory')
+
+        # find the correct variable
+        variable = None
+        for v, offset in existing_vars:
+            if offset == stack_offset:
+                variable = v
+                break
+
+        if variable is None:
+            # TODO: how to determine the size for a lea?
+            existing_vars = self.state.stack_region.get_variables_by_offset(stack_offset)
+            if not existing_vars:
+                lea_size = 1
+                variable = SimStackVariable(stack_offset, lea_size, base='bp',
+                                            ident=self.variable_manager[self.func_addr].next_variable_ident(
+                                                'stack'),
+                                            region=self.func_addr,
+                                            )
+
+                self.variable_manager[self.func_addr].add_variable('stack', stack_offset, variable)
+                l.debug('Identified a new stack variable %s at %#x.', variable, self.ins_addr)
+            else:
+                variable = next(iter(existing_vars))
+
+        self.state.stack_region.add_variable(stack_offset, variable)
+        typevar = typevars.TypeVariable() if richr.typevar is None else richr.typevar
+        self.state.typevars.add_type_variable(variable, codeloc, typevar)
+        base_offset = self.state.stack_region.get_base_addr(stack_offset)
+        for var in self.state.stack_region.get_variables_by_offset(base_offset):
+            offset_into_var = stack_offset - base_offset
+            if offset_into_var == 0: offset_into_var = None
+            self.variable_manager[self.func_addr].reference_at(var, offset_into_var, codeloc,
+                                                               atom=src)
+
     def _assign_to_register(self, offset, richr, size, src=None, dst=None):
         """
 
@@ -81,7 +126,7 @@ class SimEngineVRBase(SimEngineLight):
         :return:
         """
 
-        codeloc = self._codeloc()  # type: CodeLocation
+        codeloc: CodeLocation = self._codeloc()
         data = richr.data
 
         if offset == self.arch.sp_offset:
@@ -114,47 +159,14 @@ class SimEngineVRBase(SimEngineLight):
 
         if type(data) is SpOffset and isinstance(data.offset, int):
             # lea
-            stack_offset = data.offset
-            existing_vars = self.variable_manager[self.func_addr].find_variables_by_stmt(self.block.addr,
-                                                                                         self.stmt_idx,
-                                                                                         'memory')
-
-            if not existing_vars:
-                # TODO: how to determine the size for a lea?
-                existing_vars = self.state.stack_region.get_variables_by_offset(stack_offset)
-                if not existing_vars:
-                    lea_size = 1
-                    variable = SimStackVariable(stack_offset, lea_size, base='bp',
-                                                ident=self.variable_manager[self.func_addr].next_variable_ident(
-                                                    'stack'),
-                                                region=self.func_addr,
-                                                )
-
-                    self.variable_manager[self.func_addr].add_variable('stack', stack_offset, variable)
-                    l.debug('Identified a new stack variable %s at %#x.', variable, self.ins_addr)
-                else:
-                    variable = next(iter(existing_vars))
-
-            else:
-                variable, _ = existing_vars[0]
-
-            self.state.stack_region.add_variable(stack_offset, variable)
-            typevar = typevars.TypeVariable() if richr.typevar is None else richr.typevar
-            self.state.typevars.add_type_variable(variable, codeloc, typevar)
-            base_offset = self.state.stack_region.get_base_addr(stack_offset)
-            for var in self.state.stack_region.get_variables_by_offset(base_offset):
-                offset_into_var = stack_offset - base_offset
-                if offset_into_var == 0: offset_into_var = None
-                self.variable_manager[self.func_addr].reference_at(var, offset_into_var, codeloc,
-                                                                   atom=src)
-
+            self._reference(richr, codeloc, src=src)
         else:
             pass
 
         # handle register writes
         existing_vars = self.variable_manager[self.func_addr].find_variables_by_atom(self.block.addr, self.stmt_idx,
                                                                                      dst)
-        existing_vars: Set[SimVariable,int]
+        existing_vars: Set[Tuple[SimVariable,int]]
         if not existing_vars:
             variable = SimRegisterVariable(offset, size,
                                            ident=self.variable_manager[self.func_addr].next_variable_ident(
@@ -195,7 +207,7 @@ class SimEngineVRBase(SimEngineLight):
             return
 
         if type(addr) is int:
-            # TODO: Handle storing to global
+            self._store_to_global(addr, data, size, stmt=stmt)
             return
 
         if addr is None:
@@ -253,6 +265,39 @@ class SimEngineVRBase(SimEngineLight):
                     )
         # TODO: Create a tv_sp.store.<bits>@N <: typevar type constraint for the stack pointer
 
+    def _store_to_global(self, addr: int, data, size, stmt=None):
+        variable_manager = self.variable_manager['global']
+        if stmt is None:
+            existing_vars = variable_manager.find_variables_by_stmt(self.block.addr, self.stmt_idx, 'memory')
+        else:
+            existing_vars = variable_manager.find_variables_by_atom(self.block.addr, self.stmt_idx, stmt)
+        if not existing_vars:
+            variable = SimMemoryVariable(addr, size,
+                                        ident=variable_manager.next_variable_ident('global'),
+                                        )
+            variable_manager.set_variable('global', addr, variable)
+            l.debug('Identified a new global variable %s at %#x.', variable, self.ins_addr)
+
+        else:
+            variable, _ = next(iter(existing_vars))
+
+        self.state.global_region.set_variable(addr, variable)
+        codeloc = CodeLocation(self.block.addr, self.stmt_idx, ins_addr=self.ins_addr)
+        for var in self.state.global_region.get_variables_by_offset(addr):
+            variable_manager.write_to(var, 0, codeloc, atom=stmt)
+
+        # create type constraints
+        if data.typevar is not None:
+            if not self.state.typevars.has_type_variable_for(variable, codeloc):
+                typevar = typevars.TypeVariable()
+                self.state.typevars.add_type_variable(variable, codeloc, typevar)
+            else:
+                typevar = self.state.typevars.get_type_variable(variable, codeloc)
+            if typevar is not None:
+                self.state.add_type_constraint(
+                    typevars.Subtype(data.typevar, typevar)
+                )
+
     def _store_to_variable(self, richr_addr: RichR, size, stmt=None):  # pylint:disable=unused-argument
 
         addr_variable = richr_addr.variable
@@ -295,7 +340,10 @@ class SimEngineVRBase(SimEngineLight):
         :return:
         """
 
+        self.state: 'VariableRecoveryStateBase'
+
         addr = richr_addr.data
+        codeloc = CodeLocation(self.block.addr, self.stmt_idx, ins_addr=self.ins_addr)
 
         if type(addr) is SpOffset:
             # Loading data from stack
@@ -334,7 +382,6 @@ class SimEngineVRBase(SimEngineLight):
                 l.debug('Identified a new stack variable %s at %#x.', variable, self.ins_addr)
 
             base_offset = self.state.stack_region.get_base_addr(concrete_offset)
-            codeloc = CodeLocation(self.block.addr, self.stmt_idx, ins_addr=self.ins_addr)
 
             all_vars = self.state.stack_region.get_variables_by_offset(base_offset)
             if len(all_vars) > 1:
@@ -381,6 +428,17 @@ class SimEngineVRBase(SimEngineLight):
 
             return RichR(data, variable=var, typevar=typevar)
 
+        elif isinstance(addr, int):
+            # Loading data from memory
+            global_variables = self.variable_manager['global']
+            variables = global_variables.get_global_variables(addr)
+            if not variables:
+                var = SimMemoryVariable(addr, size)
+                global_variables.add_variable('global', addr, var)
+                variables = [ var ]
+            for var in variables:
+                global_variables.read_from(var, 0, codeloc, atom=expr)
+
         # Loading data from a pointer
         if richr_addr.type_constraints:
             for tc in richr_addr.type_constraints:
