@@ -16,9 +16,9 @@ class SimEngineXRefsVEX(
     SimEngineLightVEXMixin,
     SimEngineLight,
 ):
-    def __init__(self, xref_manager, replacements=None):
+    def __init__(self, xref_manager, project=None, replacements=None):
         super().__init__()
-
+        self.project = project
         self.xref_manager = xref_manager
         self.replacements = replacements if replacements is not None else { }
 
@@ -34,19 +34,24 @@ class SimEngineXRefsVEX(
     def _handle_WrTmp(self, stmt):
         # Don't execute the tmp write since it has been done during constant propagation
         self._expr(stmt.data)
+        if type(stmt.data) is pyvex.IRExpr.Load:
+            self._handle_data_offset_refs(stmt.tmp)
 
     def _handle_Put(self, stmt):
         # if there is a Load, get it executed
         self._expr(stmt.data)
 
     def _handle_Store(self, stmt):
-        blockloc = self._codeloc(block_only=True)
-        # TODO: Handle constant stores
-        if type(stmt.addr) is pyvex.IRExpr.RdTmp:
+        if isinstance(stmt.addr, pyvex.IRExpr.RdTmp):
             addr_tmp = VEXTmp(stmt.addr.tmp)
+            blockloc = self._codeloc(block_only=True)
             if addr_tmp in self.replacements[blockloc] and not isinstance(self.replacements[blockloc][addr_tmp], Top):
                 addr = self.replacements[blockloc][addr_tmp]
-                self.add_xref(XRefType.Write, self._codeloc(), addr)
+                if isinstance(addr, int):
+                    self.add_xref(XRefType.Write, self._codeloc(), addr)
+        elif isinstance(stmt.addr, pyvex.IRExpr.Const):
+            addr = stmt.addr.con.value
+            self.add_xref(XRefType.Write, self._codeloc(), addr)
 
     def _handle_StoreG(self, stmt):
         blockloc = self._codeloc(block_only=True)
@@ -54,7 +59,8 @@ class SimEngineXRefsVEX(
             addr_tmp = VEXTmp(stmt.addr.tmp)
             if addr_tmp in self.replacements[blockloc] and not isinstance(self.replacements[blockloc][addr_tmp], Top):
                 addr = self.replacements[blockloc][addr_tmp]
-                self.add_xref(XRefType.Write, self._codeloc(), addr)
+                if isinstance(addr, int):
+                    self.add_xref(XRefType.Write, self._codeloc(), addr)
 
     def _handle_LoadG(self, stmt):
         # What are we reading?
@@ -63,7 +69,41 @@ class SimEngineXRefsVEX(
             addr_tmp = VEXTmp(stmt.addr.tmp)
             if addr_tmp in self.replacements[blockloc] and not isinstance(self.replacements[blockloc][addr_tmp], Top):
                 addr = self.replacements[blockloc][addr_tmp]
-                self.add_xref(XRefType.Read, self._codeloc(), addr)
+                if isinstance(addr, int):
+                    self.add_xref(XRefType.Read, self._codeloc(), addr)
+        self._handle_data_offset_refs(stmt.dst)
+
+    def _handle_LLSC(self, stmt: pyvex.IRStmt.LLSC):
+        blockloc = self._codeloc(block_only=True)
+        if isinstance(stmt.addr, pyvex.IRExpr.RdTmp):
+            addr_tmp = VEXTmp(stmt.addr.tmp)
+            if addr_tmp in self.replacements[blockloc]:
+                addr = self.replacements[blockloc][addr_tmp]
+                if isinstance(addr, int):
+                    if stmt.storedata is None:
+                        # load-link
+                        xref_type = XRefType.Read
+                    else:
+                        xref_type = XRefType.Write
+                    self.add_xref(xref_type, self._codeloc(), addr)
+
+    def _handle_data_offset_refs(self, data_tmp):
+        # is this thing a pointer?
+        # If so, produce the ida-style "Offset" XRefs.
+        blockloc = self._codeloc(block_only=True)
+        tmp = VEXTmp(data_tmp)
+        if tmp in self.replacements[blockloc] and not isinstance(self.replacements[blockloc][tmp], Top):
+            data = self.replacements[blockloc][tmp]
+            # Is this thing not an integer? If so, get out of here
+            # e.g., you can't find_object_containing on an SPOffset
+            if not isinstance(data, int):
+                return
+            if data is not None and self.project.loader.find_object_containing(data) is not None:
+                # HACK: Avoid spamming Xrefs if the binary is loaded at 0
+                # e.g., firmware!
+                # (magic value chosen due to length of CM EVT)
+                if data > 0x200:
+                    self.add_xref(XRefType.Offset, self._codeloc(), data)
 
     #
     # Expression handlers
@@ -74,12 +114,15 @@ class SimEngineXRefsVEX(
 
     def _handle_Load(self, expr):
         blockloc = self._codeloc(block_only=True)
-        # TODO: Handle constant reads
         if type(expr.addr) is pyvex.IRExpr.RdTmp:
             addr_tmp = VEXTmp(expr.addr.tmp)
             if addr_tmp in self.replacements[blockloc] and not isinstance(self.replacements[blockloc][addr_tmp], Top):
                 addr = self.replacements[blockloc][addr_tmp]
-                self.add_xref(XRefType.Read, self._codeloc(), addr)
+                if isinstance(addr, int):
+                    self.add_xref(XRefType.Read, self._codeloc(), addr)
+        elif type(expr.addr) is pyvex.IRExpr.Const:
+            addr = expr.addr.con.value
+            self.add_xref(XRefType.Read, self._codeloc(), addr)
 
     def _handle_CCall(self, expr):
         return None
@@ -138,14 +181,14 @@ class XRefsAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-metho
 
         self._node_iterations = defaultdict(int)
 
-        self._engine_vex = SimEngineXRefsVEX(self.kb.xrefs, replacements=replacements)
+        self._engine_vex = SimEngineXRefsVEX(self.kb.xrefs, project=self.project, replacements=replacements)
         self._engine_ail = None
 
         self._analyze()
 
-        #
-        # Main analysis routines
-        #
+    #
+    # Main analysis routines
+    #
 
     def _pre_analysis(self):
         pass
@@ -161,7 +204,10 @@ class XRefsAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-metho
 
     def _run_on_node(self, node, state):
 
-        block = self.project.factory.block(node.addr, node.size, opt_level=0)
+        block = self.project.factory.block(node.addr, node.size, opt_level=1, cross_insn_opt=False)
+        if block.size == 0:
+            # VEX couldn't decode it
+            return False, None
         block_key = node.addr
         engine = self._engine_vex
 
