@@ -3,7 +3,7 @@ import logging
 import claripy
 import archinfo
 from archinfo import RegisterName
-from typing import Union, Optional, List, Dict
+from typing import Union, Optional, List, Dict, Type
 
 from .sim_type import SimType
 from .sim_type import SimTypeChar
@@ -77,6 +77,7 @@ class SimFunctionArgument:
     def check_value(self, value):
         if not isinstance(value, claripy.ast.Base) and self.size is None:
             raise TypeError("Only claripy objects may be stored through SimFunctionArgument when size is not provided")
+        # TODO: this also looks byte related, change to ARCH.byte_width
         if self.size is not None and isinstance(value, claripy.ast.Base) and self.size*8 < value.length:
             raise TypeError("%s doesn't fit in an argument of size %d" % (value, self.size))
 
@@ -128,8 +129,8 @@ class SimRegArg(SimFunctionArgument):
     def set_value(self, state, value, endness=None, size=None, **kwargs):  # pylint: disable=unused-argument,arguments-differ
         self.check_value(value)
         if endness is None: endness = state.arch.register_endness
-        if isinstance(value, int): value = claripy.BVV(value, self.size*8)
-        if size is None: size = min(self.size, value.length // 8)
+        if isinstance(value, int): value = claripy.BVV(value, self.size*state.arch.byte_width)
+        if size is None: size = min(self.size, value.length // state.arch.byte_width)
         offset = self._fix_offset(state, size)
         state.registers.store(offset, value, endness=endness, size=size)
 
@@ -164,8 +165,8 @@ class SimStackArg(SimFunctionArgument):
         self.check_value(value)
         if endness is None: endness = state.arch.memory_endness
         if stack_base is None: stack_base = state.regs.sp
-        if isinstance(value, int): value = claripy.BVV(value, self.size*8)
-        state.memory.store(stack_base + self.stack_offset, value, endness=endness, size=value.length//8)
+        if isinstance(value, int): value = claripy.BVV(value, self.size*state.arch.byte_width)
+        state.memory.store(stack_base + self.stack_offset, value, endness=endness, size=value.length//state.arch.byte_width)
 
     def get_value(self, state, endness=None, stack_base=None, size=None):  # pylint: disable=arguments-differ
         if endness is None: endness = state.arch.memory_endness
@@ -175,7 +176,7 @@ class SimStackArg(SimFunctionArgument):
 
 class SimComboArg(SimFunctionArgument):
     def __init__(self, locations):
-        super(SimComboArg, self).__init__(sum(x.size for x in locations))
+        super().__init__(sum(x.size for x in locations))
         self.locations = locations
 
     def __repr__(self):
@@ -238,7 +239,7 @@ class ArgSession:
                 arg = next(self.real_args)
                 if is_fp and self.cc.is_fp_arg(arg) is False:
                     raise TypeError("Can't put a float here - concrete arg positions are specified")
-                elif not is_fp and self.cc.is_fp_arg(arg) is True:
+                if not is_fp and self.cc.is_fp_arg(arg) is True:
                     raise TypeError("Can't put an int here - concrete arg positions are specified")
             except StopIteration:
                 raise TypeError("Accessed too many arguments - concrete number are specified")
@@ -505,8 +506,7 @@ class SimCC:
                 args = [ a.with_arch(self.arch) for a in self.func_ty.args ]
             else:
                 args = self.args
-            is_fp = [ True if isinstance(arg, (SimTypeFloat, SimTypeDouble)) or self.is_fp_arg(arg) else False
-                      for arg in args ]
+            is_fp = [ isinstance(arg, (SimTypeFloat, SimTypeDouble)) or self.is_fp_arg(arg) for arg in args ]
             if sizes is None:
                 # initialize sizes from args
                 sizes = [ ]
@@ -515,7 +515,7 @@ class SimCC:
                         if isinstance(a, SimTypeFixedSizeArray) or a.size is NotImplemented:
                             sizes.append(self.arch.bytes)
                         else:
-                            sizes.append(a.size // 8)  # SimType.size is in bits
+                            sizes.append(a.size // self.arch.byte_width)  # SimType.size is in bits
                     elif isinstance(a, SimFunctionArgument):
                         sizes.append(a.size)  # SimFunctionArgument.size is in bytes
                     else:
@@ -568,8 +568,7 @@ class SimCC:
             if self.args is None:
                 if self.func_ty is None:
                     raise ValueError("You must either customize this CC or pass a value to is_fp!")
-                else:
-                    arg_locs = self.arg_locs([False]*len(self.func_ty.args))
+                arg_locs = self.arg_locs([False]*len(self.func_ty.args))
             else:
                 arg_locs = self.args
 
@@ -686,7 +685,7 @@ class SimCC:
         allocator.apply(state, alloc_base)
 
         for loc, val in zip(arg_locs, vals):
-            if val.length > loc.size * 8:
+            if val.length > loc.size * state.arch.byte_width:
                 raise ValueError("Can't fit value {} into location {}".format(repr(val), repr(loc)))
             loc.set_value(state, val, endness='Iend_BE', stack_base=stack_base)
         self.return_addr.set_value(state, ret_addr, stack_base=stack_base)
@@ -725,6 +724,17 @@ class SimCC:
 
         return ret_addr
 
+    def set_func_type_with_arch(self, func_ty: Optional[SimTypeFunction]) -> None:
+        """
+        Set self.func_ty to another function type and set its arch to self.arch.
+
+        :param func_ty: The SimTypeFunction type to set.
+        :return:        None
+        """
+        if func_ty is None or self.arch is None:
+            self.func_ty = func_ty
+        else:
+            self.func_ty = func_ty.with_arch(self.arch)
 
     # pylint: disable=unused-argument
     def get_return_val(self, state, is_fp=None, size=None, stack_base=None):
@@ -756,8 +766,8 @@ class SimCC:
         ty = self.func_ty.returnty if self.func_ty is not None else None
         try:
             betterval = self._standardize_value(val, ty, state, None)
-        except AttributeError:
-            raise ValueError("Can't fit value %s into a return value" % repr(val))
+        except AttributeError as ex:
+            raise ValueError("Can't fit value %s into a return value" % repr(val)) from ex
 
         if self.ret_val is not None:
             loc = self.ret_val
@@ -1014,7 +1024,7 @@ class SimLyingRegArg(SimRegArg):
     """
     def __init__(self, name):
         # TODO: This looks byte-related.  Make sure to use Arch.byte_width
-        super(SimLyingRegArg, self).__init__(name, 8)
+        super().__init__(name, 8)
 
     def get_value(self, state, size=None, endness=None, **kwargs):  # pylint:disable=arguments-differ
         #val = super(SimLyingRegArg, self).get_value(state, **kwargs)
@@ -1063,7 +1073,54 @@ class SimCCMicrosoftAMD64(SimCC):
     ARCH = archinfo.ArchAMD64
     STACK_ALIGNMENT = 16
 
-class SimCCX86LinuxSyscall(SimCC):
+
+class SimCCSyscall(SimCC):
+    """
+    The base class of all syscall CCs.
+    """
+
+    ERROR_REG : SimRegArg = None
+    SYSCALL_ERRNO_START = None
+
+    @staticmethod
+    def syscall_num(state) -> int:
+        raise NotImplementedError()
+
+    def linux_syscall_update_error_reg(self, state, expr):
+        # special handling for Linux syscalls: on some architectures (mips/a3, powerpc/cr0_0) a bool indicating success
+        # or failure of a system call is used as an error flag (0 for success, 1 for error). we have to set this
+        if state.project is None or state.project.simos is None or state.project.simos.name != 'Linux':
+            return expr
+        if type(expr) is int:
+            expr = claripy.BVV(expr, state.arch.bits)
+        try:
+            expr = expr.ast
+        except AttributeError:
+            pass
+        nbits = self.ERROR_REG.size * state.arch.byte_width
+        error_cond = claripy.UGE(expr, self.SYSCALL_ERRNO_START)
+        if state.solver.is_false(error_cond):
+            # guaranteed no error
+            error_reg_val = claripy.BVV(0, nbits)
+        elif state.solver.is_true(error_cond):
+            # guaranteed error
+            error_reg_val = claripy.BVV(-1, nbits)
+            expr = -expr
+        else:
+            # both are satisfied, handle gracefully
+            error_reg_val = claripy.If(error_cond, claripy.BVV(-1, nbits), 0)
+            expr = claripy.If(error_cond, -expr, expr)
+
+        self.ERROR_REG.set_value(state, error_reg_val)
+        return expr
+
+    def set_return_val(self, state, val, is_fp=None, size=None, stack_base=None):
+        if self.ERROR_REG is not None:
+            val = self.linux_syscall_update_error_reg(state, val)
+        super().set_return_val(state, val, is_fp=is_fp, size=size, stack_base=stack_base)
+
+
+class SimCCX86LinuxSyscall(SimCCSyscall):
     ARG_REGS = ['ebx', 'ecx', 'edx', 'esi', 'edi', 'ebp']
     FP_ARG_REGS = []
     RETURN_VAL = SimRegArg('eax', 4)
@@ -1079,7 +1136,8 @@ class SimCCX86LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.eax
 
-class SimCCX86WindowsSyscall(SimCC):
+
+class SimCCX86WindowsSyscall(SimCCSyscall):
     # TODO: Make sure the information is correct
     ARG_REGS = [ ]
     FP_ARG_REGS = [ ]
@@ -1096,6 +1154,7 @@ class SimCCX86WindowsSyscall(SimCC):
     def syscall_num(state):
         return state.regs.eax
 
+
 class SimCCSystemVAMD64(SimCC):
     ARG_REGS = ['rdi', 'rsi', 'rdx', 'rcx', 'r8', 'r9']
     FP_ARG_REGS = ['xmm0', 'xmm1', 'xmm2', 'xmm3', 'xmm4', 'xmm5', 'xmm6', 'xmm7']
@@ -1109,7 +1168,7 @@ class SimCCSystemVAMD64(SimCC):
     STACK_ALIGNMENT = 16
 
     def __init__(self, arch, args=None, ret_val=None, sp_delta=None, func_ty=None):
-        super(SimCCSystemVAMD64, self).__init__(arch, args, ret_val, sp_delta, func_ty)
+        super().__init__(arch, args, ret_val, sp_delta, func_ty)
 
         # Remove the ret address on stack
         if self.args is not None:
@@ -1136,7 +1195,8 @@ class SimCCSystemVAMD64(SimCC):
 
         return True
 
-class SimCCAMD64LinuxSyscall(SimCC):
+
+class SimCCAMD64LinuxSyscall(SimCCSyscall):
     ARG_REGS = ['rdi', 'rsi', 'rdx', 'r10', 'r8', 'r9']
     RETURN_VAL = SimRegArg('rax', 8)
     RETURN_ADDR = SimRegArg('ip_at_syscall', 8)
@@ -1151,7 +1211,8 @@ class SimCCAMD64LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.rax
 
-class SimCCAMD64WindowsSyscall(SimCC):
+
+class SimCCAMD64WindowsSyscall(SimCCSyscall):
     # TODO: Make sure the information is correct
     ARG_REGS = [ ]
     FP_ARG_REGS = [ ]
@@ -1168,6 +1229,7 @@ class SimCCAMD64WindowsSyscall(SimCC):
     def syscall_num(state):
         return state.regs.rax
 
+
 class SimCCARM(SimCC):
     ARG_REGS = [ 'r0', 'r1', 'r2', 'r3' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1176,7 +1238,8 @@ class SimCCARM(SimCC):
     RETURN_VAL = SimRegArg('r0', 4)
     ARCH = archinfo.ArchARM
 
-class SimCCARMLinuxSyscall(SimCC):
+
+class SimCCARMLinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
     ARG_REGS = [ 'r0', 'r1', 'r2', 'r3' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1193,6 +1256,7 @@ class SimCCARMLinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.r7
 
+
 class SimCCAArch64(SimCC):
     ARG_REGS = [ 'x0', 'x1', 'x2', 'x3', 'x4', 'x5', 'x6', 'x7' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1200,7 +1264,8 @@ class SimCCAArch64(SimCC):
     RETURN_VAL = SimRegArg('x0', 8)
     ARCH = archinfo.ArchAArch64
 
-class SimCCAArch64LinuxSyscall(SimCC):
+
+class SimCCAArch64LinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
     ARG_REGS = [ 'x0', 'x1', 'x2', 'x3', 'x4', 'x5', 'x6', 'x7' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1217,6 +1282,7 @@ class SimCCAArch64LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.x8
 
+
 class SimCCO32(SimCC):
     ARG_REGS = [ 'a0', 'a1', 'a2', 'a3' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1226,7 +1292,8 @@ class SimCCO32(SimCC):
     RETURN_VAL = SimRegArg('v0', 4)
     ARCH = archinfo.ArchMIPS32
 
-class SimCCO32LinuxSyscall(SimCC):
+
+class SimCCO32LinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
     ARG_REGS = [ 'a0', 'a1', 'a2', 'a3' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1234,6 +1301,9 @@ class SimCCO32LinuxSyscall(SimCC):
     RETURN_ADDR = SimRegArg('ip_at_syscall', 4)
     ARCH = archinfo.ArchMIPS32
 
+    ERROR_REG = SimRegArg('a3', 4)
+    SYSCALL_ERRNO_START = -1133
+
     @classmethod
     def _match(cls, arch, args, sp_delta):  # pylint: disable=unused-argument
         # never appears anywhere except syscalls
@@ -1243,6 +1313,7 @@ class SimCCO32LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.v0
 
+
 class SimCCO64(SimCC):      # TODO: add n32 and n64
     ARG_REGS = [ 'a0', 'a1', 'a2', 'a3' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1251,14 +1322,17 @@ class SimCCO64(SimCC):      # TODO: add n32 and n64
     RETURN_VAL = SimRegArg('v0', 8)
     ARCH = archinfo.ArchMIPS64
 
-class SimCCO64LinuxSyscall(SimCC):
-    # TODO: Make sure all the information is correct
-    ARG_REGS = [ 'a0', 'a1', 'a2', 'a3' ]
+
+class SimCCO64LinuxSyscall(SimCCSyscall):
+    ARG_REGS = [ 'a0', 'a1', 'a2', 'a3', 'a4', 'a5', 'a6', 'a7' ]
     FP_ARG_REGS = []    # TODO: ???
     RETURN_VAL = SimRegArg('v0', 8)
     RETURN_ADDR = SimRegArg('ip_at_syscall', 8)
     ARCH = archinfo.ArchMIPS64
 
+    ERROR_REG = SimRegArg('a3', 8)
+    SYSCALL_ERRNO_START = -1133
+
     @classmethod
     def _match(cls, arch, args, sp_delta):  # pylint: disable=unused-argument
         # never appears anywhere except syscalls
@@ -1268,6 +1342,7 @@ class SimCCO64LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.v0
 
+
 class SimCCPowerPC(SimCC):
     ARG_REGS = [ 'r3', 'r4', 'r5', 'r6', 'r7', 'r8', 'r9', 'r10' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1276,7 +1351,8 @@ class SimCCPowerPC(SimCC):
     RETURN_VAL = SimRegArg('r3', 4)
     ARCH = archinfo.ArchPPC32
 
-class SimCCPowerPCLinuxSyscall(SimCC):
+
+class SimCCPowerPCLinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
     ARG_REGS = ['r3', 'r4', 'r5', 'r6', 'r7', 'r8', 'r9', 'r10']
     FP_ARG_REGS = [ ]
@@ -1284,6 +1360,9 @@ class SimCCPowerPCLinuxSyscall(SimCC):
     RETURN_ADDR = SimRegArg('ip_at_syscall', 4)
     ARCH = archinfo.ArchPPC32
 
+    ERROR_REG = SimRegArg('cr0_0', 1)
+    SYSCALL_ERRNO_START = -515
+
     @classmethod
     def _match(cls, arch, args, sp_delta):  # pylint: disable=unused-argument
         # never appears anywhere except syscalls
@@ -1293,6 +1372,7 @@ class SimCCPowerPCLinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.r0
 
+
 class SimCCPowerPC64(SimCC):
     ARG_REGS = [ 'r3', 'r4', 'r5', 'r6', 'r7', 'r8', 'r9', 'r10' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1301,14 +1381,18 @@ class SimCCPowerPC64(SimCC):
     RETURN_VAL = SimRegArg('r3', 8)
     ARCH = archinfo.ArchPPC64
 
-class SimCCPowerPC64LinuxSyscall(SimCC):
+
+class SimCCPowerPC64LinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
-    ARG_REGS = [ ]
+    ARG_REGS = ['r3', 'r4', 'r5', 'r6', 'r7', 'r8', 'r9', 'r10']
     FP_ARG_REGS = [ ]
     RETURN_VAL = SimRegArg('r3', 8)
     RETURN_ADDR = SimRegArg('ip_at_syscall', 8)
     ARCH = archinfo.ArchPPC64
 
+    ERROR_REG = SimRegArg('cr0_0', 1)
+    SYSCALL_ERRNO_START = -515
+
     @classmethod
     def _match(cls, arch, args, sp_delta):  # pylint: disable=unused-argument
         # never appears anywhere except syscalls
@@ -1318,10 +1402,12 @@ class SimCCPowerPC64LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.r0
 
+
 class SimCCSoot(SimCC):
     ARCH = archinfo.ArchSoot
     ARG_REGS = []
 
+
 class SimCCUnknown(SimCC):
     """
     Represent an unknown calling convention.
@@ -1334,6 +1420,7 @@ class SimCCUnknown(SimCC):
     def __repr__(self):
         return "<SimCCUnknown - %s %s sp_delta=%d>" % (self.arch.name, self.args, self.sp_delta)
 
+
 class SimCCS390X(SimCC):
     ARG_REGS = ['r2', 'r3', 'r4', 'r5', 'r6']
     FP_ARG_REGS = ['f0', 'f2', 'f4', 'f6']
@@ -1342,7 +1429,8 @@ class SimCCS390X(SimCC):
     RETURN_VAL = SimRegArg('r2', 8)
     ARCH = archinfo.ArchS390X
 
-class SimCCS390XLinuxSyscall(SimCC):
+
+class SimCCS390XLinuxSyscall(SimCCSyscall):
     ARG_REGS = ['r2', 'r3', 'r4', 'r5', 'r6', 'r7']
     FP_ARG_REGS = []
     RETURN_VAL = SimRegArg('r2', 8)
@@ -1358,6 +1446,7 @@ class SimCCS390XLinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.r1
 
+
 CC = {
     'AMD64': [
         SimCCSystemVAMD64,
@@ -1395,7 +1484,7 @@ CC = {
 }
 
 
-DEFAULT_CC: Dict[str,type] = {
+DEFAULT_CC: Dict[str,Type[SimCC]] = {
     'AMD64': SimCCSystemVAMD64,
     'X86': SimCCCdecl,
     'ARMEL': SimCCARM,
@@ -1407,7 +1496,7 @@ DEFAULT_CC: Dict[str,type] = {
     'PPC64': SimCCPowerPC64,
     'AARCH64': SimCCAArch64,
     'Soot': SimCCSoot,
-    'AVR': SimCCUnknown,
+    'AVR8': SimCCUnknown,
     'MSP': SimCCUnknown,
     'S390X': SimCCS390X,
 }
@@ -1416,7 +1505,8 @@ DEFAULT_CC: Dict[str,type] = {
 def register_default_cc(arch, cc):
     DEFAULT_CC[arch] = cc
 
-SYSCALL_CC = {
+
+SYSCALL_CC: Dict[str,Dict[str,Type[SimCCSyscall]]] = {
     'X86': {
         'default': SimCCX86LinuxSyscall,
         'Linux': SimCCX86LinuxSyscall,
