@@ -1,6 +1,6 @@
 
 from .memory_data import MemoryDataSort, MemoryData
 from .cfg_node import CFGNode, CFGENode
-from .indirect_jump import IndirectJump
+from .indirect_jump import IndirectJump, IndirectJumpType
 from .cfg_model import CFGModel
 from .cfg_manager import CFGManager
