@@ -1,14 +1,24 @@
+from typing import List, Optional
 
 from ...serializable import Serializable
 
 
+class IndirectJumpType:
+    Jumptable_AddressLoadedFromMemory = 0
+    Jumptable_AddressComputed = 1
+    Unknown = 2
+
+
 class IndirectJump(Serializable):
 
     __slots__ = ("addr", "ins_addr", "func_addr", "jumpkind", "stmt_idx", "resolved_targets", "jumptable",
-                 "jumptable_addr", "jumptable_size", "jumptable_entry_size", "jumptable_entries", )
+                 "jumptable_addr", "jumptable_size", "jumptable_entry_size", "jumptable_entries", "type", )
 
-    def __init__(self, addr, ins_addr, func_addr, jumpkind, stmt_idx, resolved_targets=None, jumptable=False,
-                 jumptable_addr=None, jumptable_size=None, jumptable_entry_size=None, jumptable_entries=None):
+    def __init__(self, addr: int, ins_addr: int, func_addr: int, jumpkind: str, stmt_idx: int,
+                 resolved_targets: Optional[List[int]]=None, jumptable: bool=False,
+                 jumptable_addr: Optional[int]=None, jumptable_size: Optional[int]=None,
+                 jumptable_entry_size: Optional[int]=None, jumptable_entries: Optional[List[int]]=None,
+                 type_: Optional[int]=IndirectJumpType.Unknown):
         self.addr = addr
         self.ins_addr = ins_addr
         self.func_addr = func_addr
@@ -20,6 +30,7 @@ class IndirectJump(Serializable):
         self.jumptable_size = jumptable_size
         self.jumptable_entry_size = jumptable_entry_size
         self.jumptable_entries = jumptable_entries
+        self.type = type_
 
     def __repr__(self):
 
