@@ -4,7 +4,7 @@ import archinfo
 from collections import defaultdict
 import logging
 import inspect
-from typing import Optional, Dict
+from typing import Optional, Dict, Type, TYPE_CHECKING
 
 
 from ...sim_type import parse_cpp_file, SimTypeFunction
@@ -14,8 +14,12 @@ from ...sim_type import parse_file
 from ..stubs.ReturnUnconstrained import ReturnUnconstrained
 from ..stubs.syscall_stub import syscall as stub_syscall
 
+if TYPE_CHECKING:
+    from angr.calling_conventions import SimCCSyscall
+
+
 l = logging.getLogger(name=__name__)
-SIM_LIBRARIES = {}
+SIM_LIBRARIES: Dict[str,'SimLibrary'] = {}
 
 
 class SimLibrary:
@@ -336,7 +340,7 @@ class SimCppLibrary(SimLibrary):
         # try to determine a prototype from the function name if possible
         if demangled_name != name:
             # itanium-mangled function name
-            stub.cc.func_ty = self._proto_from_demangled_name(demangled_name)
+            stub.cc.set_func_type_with_arch(self._proto_from_demangled_name(demangled_name))
             if stub.cc.func_ty is not None and not stub.ARGS_MISMATCH:
                 stub.cc.num_args = len(stub.cc.func_ty.args)
                 stub.num_args = len(stub.cc.func_ty.args)
@@ -388,7 +392,7 @@ class SimCppLibrary(SimLibrary):
 class SimSyscallLibrary(SimLibrary):
     """
     SimSyscallLibrary is a specialized version of SimLibrary for dealing not with a dynamic library's API but rather
-    an operating system's syscall API. Because this interface is inherantly lower-level than a dynamic library, many
+    an operating system's syscall API. Because this interface is inherently lower-level than a dynamic library, many
     parts of this class has been changed to store data based on an "ABI name" (ABI = application binary interface,
     like an API but for when there's no programming language) instead of an architecture. An ABI name is just an
     arbitrary string with which a calling convention and a syscall numbering is associated.
@@ -397,10 +401,11 @@ class SimSyscallLibrary(SimLibrary):
     associates them with numbers.
     """
     def __init__(self):
-        super(SimSyscallLibrary, self).__init__()
-        self.syscall_number_mapping = defaultdict(dict)
-        self.syscall_name_mapping = defaultdict(dict)
-        self.default_cc_mapping = {}
+        super().__init__()
+        self.syscall_number_mapping: Dict[str,Dict[int,str]] = defaultdict(dict)  # keyed by abi
+        self.syscall_name_mapping: Dict[str,Dict[str,int]] = defaultdict(dict)  # keyed by abi
+        self.default_cc_mapping: Dict[str,Type['SimCCSyscall']] = {}  # keyed by abi
+        self.syscall_prototypes: Dict[str,Dict[str,SimTypeFunction]] = defaultdict(dict)  # keyed by abi
         self.fallback_proc = stub_syscall
 
     def copy(self):
@@ -412,11 +417,12 @@ class SimSyscallLibrary(SimLibrary):
         o.names = list(self.names)
         o.syscall_number_mapping = defaultdict(dict, self.syscall_number_mapping) # {abi: {number: name}}
         o.syscall_name_mapping = defaultdict(dict, self.syscall_name_mapping) # {abi: {name: number}}
+        o.syscall_prototypes = defaultdict(dict, self.syscall_prototypes) # as above
         o.default_cc_mapping = dict(self.default_cc_mapping) # {abi: cc}
         return o
 
     def update(self, other):
-        super(SimSyscallLibrary, self).update(other)
+        super().update(other)
         self.syscall_number_mapping.update(other.syscall_number_mapping)
         self.syscall_name_mapping.update(other.syscall_name_mapping)
         self.default_cc_mapping.update(other.default_cc_mapping)
@@ -471,6 +477,25 @@ class SimSyscallLibrary(SimLibrary):
         """
         self.default_cc_mapping[abi] = cc_cls
 
+    def set_prototype(self, abi: str, name: str, proto: SimTypeFunction) -> None:  # pylint: disable=arguments-differ
+        """
+        Set the prototype of a function in the form of a SimTypeFunction containing argument and return types
+
+        :param abi:     ABI of the syscall.
+        :param name:    The name of the syscall as a string
+        :param proto:   The prototype of the syscall as a SimTypeFunction
+        """
+        self.syscall_prototypes[abi][name] = proto
+
+    def set_prototypes(self, abi: str, protos: Dict[str,SimTypeFunction]) -> None:  # pylint: disable=arguments-differ
+        """
+        Set the prototypes of many syscalls.
+
+        :param abi:     ABI of the syscalls.
+        :param protos:  Dictionary mapping syscall names to SimTypeFunction objects
+        """
+        self.syscall_prototypes[abi].update(protos)
+
     def _canonicalize(self, number, arch, abi_list):
         if type(arch) is str:
             arch = archinfo.arch_from_id(arch)
@@ -488,8 +513,12 @@ class SimSyscallLibrary(SimLibrary):
         if abi in self.default_cc_mapping:
             cc = self.default_cc_mapping[abi](arch)
             if proc.cc is not None:
-                cc.func_ty = proc.cc.func_ty
+                cc.set_func_type_with_arch(proc.cc.func_ty)
             proc.cc = cc
+        # a bit of a hack.
+        name = proc.display_name
+        if self.syscall_prototypes[abi].get(name, None) is not None and proc.cc is not None:
+            proc.cc.func_ty = self.syscall_prototypes[abi][name].with_arch(arch)
 
     # pylint: disable=arguments-differ
     def get(self, number, arch, abi_list=()):
@@ -509,7 +538,7 @@ class SimSyscallLibrary(SimLibrary):
                             implementation is available
         """
         name, arch, abi = self._canonicalize(number, arch, abi_list)
-        proc = super(SimSyscallLibrary, self).get(name, arch)
+        proc = super().get(name, arch)
         proc.is_syscall = True
         self._apply_numerical_metadata(proc, number, arch, abi)
         return proc
@@ -524,11 +553,28 @@ class SimSyscallLibrary(SimLibrary):
         :return:            A SimProcedure representing a plausable stub that could model the syscall
         """
         name, arch, abi = self._canonicalize(number, arch, abi_list)
-        proc = super(SimSyscallLibrary, self).get_stub(name, arch)
+        proc = super().get_stub(name, arch)
         self._apply_numerical_metadata(proc, number, arch, abi)
         l.debug("unsupported syscall: %s", number)
         return proc
 
+    def get_prototype(self, abi: str, name: str, arch=None) -> Optional[SimTypeFunction]:
+        """
+        Get a prototype of the given syscall name and its ABI, optionally specialize the prototype to a given
+        architecture.
+
+        :param abi:     ABI of the prototype to get.
+        :param name:    Name of the syscall.
+        :param arch:    The architecture to specialize to.
+        :return:        Prototype of the syscall, or None if the prototype does not exist.
+        """
+        if abi not in self.syscall_prototypes:
+            return None
+        proto = self.syscall_prototypes[abi].get(name, None)
+        if proto is None:
+            return None
+        return proto.with_arch(arch=arch)
+
     def has_metadata(self, number, arch, abi_list=()):
         """
         Pretty much the intersection of SimLibrary.has_metadata() and SimSyscallLibrary.get().
@@ -539,7 +585,7 @@ class SimSyscallLibrary(SimLibrary):
         :return:            A bool of whether or not any implementation or metadata is known about the given syscall
         """
         name, _, _ = self._canonicalize(number, arch, abi_list)
-        return super(SimSyscallLibrary, self).has_metadata(name)
+        return super().has_metadata(name)
 
     def has_implementation(self, number, arch, abi_list=()):
         """
@@ -551,7 +597,20 @@ class SimSyscallLibrary(SimLibrary):
         :return:            A bool of whether or not an implementation of the syscall is available
         """
         name, _, _ = self._canonicalize(number, arch, abi_list)
-        return super(SimSyscallLibrary, self).has_implementation(name)
+        return super().has_implementation(name)
+
+    def has_prototype(self, abi: str, name: str) -> bool:
+        """
+        Check if a function has a prototype associated with it. Demangle the function name if it is a mangled C++ name.
+
+        :param abi:         Name of the ABI.
+        :param name:        The syscall name.
+        :return:            bool
+        """
+        if abi not in self.syscall_prototypes:
+            return False
+        return name in self.syscall_prototypes[abi]
+
 
 for _ in autoimport.auto_import_modules('angr.procedures.definitions', os.path.dirname(os.path.realpath(__file__))):
     pass
