@@ -1,5 +1,4 @@
 import angr
-from angr.sim_type import SimTypeString, SimTypeInt
 
 ######################################
 # puts
@@ -9,8 +8,6 @@ class puts(angr.SimProcedure):
     #pylint:disable=arguments-differ
 
     def run(self, string):
-        self.argument_types = {0: self.ty_ptr(SimTypeString())}
-        self.return_type = SimTypeInt(32, True)
 
         stdout = self.state.posix.get_fd(1)
         if stdout is None:
