@@ -2,9 +2,13 @@ import inspect
 import copy
 import itertools
 import logging
+from typing import TYPE_CHECKING
 from cle import SymbolType
 from archinfo.arch_soot import SootAddressDescriptor
 
+if TYPE_CHECKING:
+    import angr
+
 l = logging.getLogger(name=__name__)
 symbolic_count = itertools.count()
 
@@ -91,10 +95,10 @@ class SimProcedure:
         is_function=None, **kwargs
     ):
         # WE'LL FIGURE IT OUT
-        self.project = project
+        self.project = project # type: angr.Project
         self.arch = project.arch if project is not None else None
         self.addr = None
-        self.cc = cc
+        self.cc = cc # type: angr.SimCC
         self.canonical = self
 
         self.kwargs = kwargs
