@@ -48,6 +48,18 @@ class SimFilesystem(SimStatePlugin): # pretends links don't exist
         for fname in files:
             self.insert(fname, files[fname])
 
+    @SimStatePlugin.memo
+    def copy(self, memo):
+        o = super().copy(memo)
+
+        o.pathsep=self.pathsep
+        o.cwd=self.cwd
+        o._unlinks = list(self._unlinks)
+        o._files={k: v.copy(memo) for k, v in self._files.items()}
+        o._mountpoints={k: v.copy(memo) for k, v in self._mountpoints.items()}
+
+        return o
+
     @property
     def unlinks(self):
         for _, f in self._unlinks:
@@ -61,20 +73,7 @@ class SimFilesystem(SimStatePlugin): # pretends links don't exist
         for fname in self._mountpoints:
             self._mountpoints[fname].set_state(state)
 
-    @SimStatePlugin.memo
-    def copy(self, memo):
-        o = SimFilesystem(
-                files={k: v.copy(memo) for k, v in self._files.items()},
-                pathsep=self.pathsep,
-                cwd=self.cwd,
-                mountpoints={k: v.copy(memo) for k, v in self._mountpoints.items()}
-            )
-        o._unlinks = list(self._unlinks)
-        return o
-
     def merge(self, others, merge_conditions, common_ancestor=None):
-        merging_occured = False
-
         for o in others:
             if o.cwd != self.cwd:
                 raise SimMergeError("Can't merge filesystems with disparate cwds")
@@ -87,14 +86,14 @@ class SimFilesystem(SimStatePlugin): # pretends links don't exist
             try:
                 subdeck = [o._mountpoints[fname] for o in others]
             except KeyError:
-                raise SimMergeError("Can't merge filesystems with disparate file sets")
+                raise SimMergeError("Can't merge filesystems with disparate file sets") # pylint: disable=raise-missing-from
 
             if common_ancestor is not None and fname in common_ancestor._mountpoints:
                 common_mp = common_ancestor._mountpoints[fname]
             else:
                 common_mp = None
 
-            merging_occured |= self._mountpoints[fname].merge(subdeck, merge_conditions, common_ancestor=common_mp)
+            self._mountpoints[fname].merge(subdeck, merge_conditions, common_ancestor=common_mp)
 
         # this is a little messy
         deck = [self] + others
@@ -113,9 +112,9 @@ class SimFilesystem(SimStatePlugin): # pretends links don't exist
             else:
                 common_simfile = None
 
-            merging_occured |= subdeck[0].merge(subdeck[1:], merge_conditions, common_ancestor=common_simfile)
+            subdeck[0].merge(subdeck[1:], merge_conditions, common_ancestor=common_simfile)
 
-        return merging_occured
+        return True
 
     def widen(self, others): # pylint: disable=unused-argument
         if once('fs_widen_warning'):
@@ -270,6 +269,16 @@ class SimMount(SimStatePlugin):
         """
         raise NotImplementedError
 
+    def lookup(self, sim_file):
+        """
+        Look up the path of a SimFile in the mountpoint
+
+        :param sim_file:        A SimFile object needs to be looked up
+        :return:                A string representing the path of the file in the mountpoint
+                                Or None if the SimFile does not exist in the mountpoint
+        """
+        raise NotImplementedError
+
 class SimConcreteFilesystem(SimMount):
     """
     Abstract SimMount allowing the user to import files from some external source into the guest
@@ -297,7 +306,7 @@ class SimConcreteFilesystem(SimMount):
     def _load_file(self, guest_path):
         raise NotImplementedError
 
-    def _get_stat(self, guest_path):
+    def _get_stat(self, guest_path, dereference=False):
         raise NotImplementedError
 
     def insert(self, path_elements, simfile):
@@ -312,6 +321,12 @@ class SimConcreteFilesystem(SimMount):
         self.deleted_list.add(path)
         return self.cache.pop(path, None) is not None
 
+    def lookup(self, sim_file):
+        for key, val in self.cache.items():
+            if sim_file == val:
+                return key
+        return None
+
     @SimStatePlugin.memo
     def copy(self, memo):
         x = type(self)(pathsep=self.pathsep)
@@ -364,6 +379,7 @@ class SimConcreteFilesystem(SimMount):
         """
         return self.pathsep + self.pathsep.join(keys)
 
+
 class SimHostFilesystem(SimConcreteFilesystem):
     """
     Simulated mount that makes some piece from the host filesystem available to the guest.
@@ -390,12 +406,14 @@ class SimHostFilesystem(SimConcreteFilesystem):
         except OSError:
             return None
         else:
-            return SimFile(name='file://' + path, content=content, size=len(content))
+            return SimFile(name='file://' + os.path.realpath(path), content=content, size=len(content))
 
-    def _get_stat(self, guest_path):
+    def _get_stat(self, guest_path, dereference=False):
         guest_path = guest_path.lstrip(self.pathsep)
         path = os.path.join(self.host_path, guest_path)
         try:
+            if dereference:
+                path = os.path.realpath(path)
             s = os.stat(path)
             stat = Stat(s.st_dev, s.st_ino, s.st_nlink, s.st_mode, s.st_uid,
                         s.st_gid, s.st_rdev, s.st_size, s.st_blksize, s.st_blocks,
