@@ -10,7 +10,7 @@ l = logging.getLogger(name=__name__)
 max_fds = 8192
 
 class PosixDevFS(SimMount): # this'll be mounted at /dev
-    def get(self, path):
+    def get(self, path): # pylint: disable=arguments-differ
         if path == ['stdin']:
             return self.state.posix.fd.get(0, None)
         elif path == ['stdout']:
@@ -20,13 +20,16 @@ class PosixDevFS(SimMount): # this'll be mounted at /dev
         else:
             return None
 
-    def insert(self, path, simfile): # pylint: disable=unused-argument
+    def insert(self, path, simfile): # pylint: disable=unused-argument, arguments-differ
         return False
 
-    def delete(self, path): # pylint: disable=unused-argument
+    def delete(self, path): # pylint: disable=unused-argument, arguments-differ
         return False
 
-    def merge(self, others, conditions, common_ancestor=None): # pylint: disable=unused-argument
+    def lookup(self, _): # disable=unused-argument
+        return False
+
+    def merge(self, others, conditions, common_ancestor=None): # pylint: disable=unused-argument, arguments-differ
         return False
 
     def widen(self, others): # pylint: disable=unused-argument
@@ -40,19 +43,22 @@ class PosixProcFS(SimMount):
     """
     The virtual file system mounted at /proc (as of now, on Linux).
     """
-    def get(self, path):
+    def get(self, path): # pylint: disable=arguments-differ
         if path == [b"uptime"]:
             return SimFile(b"uptime", content=b"0 0")
         else:
             return None
 
-    def insert(self, path, simfile): # pylint: disable=unused-argument
+    def insert(self, path, simfile): # pylint: disable=unused-argument, arguments-differ
         return False
 
-    def delete(self, path): # pylint: disable=unused-argument
+    def delete(self, path): # pylint: disable=unused-argument, arguments-differ
         return False
 
-    def merge(self, others, conditions, common_ancestor=None): # pylint: disable=unused-argument
+    def lookup(self, _): # disable=unused-argument
+        return False
+
+    def merge(self, others, conditions, common_ancestor=None): # pylint: disable=unused-argument, arguments-differ
         return False
 
     def widen(self, others): # pylint: disable=unused-argument
@@ -128,7 +134,7 @@ class SimSystemPosix(SimStatePlugin):
             uid=None,
             gid=None,
             brk=None):
-        super(SimSystemPosix, self).__init__()
+        super().__init__()
 
         # some limits and constants
         self.sigmask_bits = 1024
@@ -181,6 +187,37 @@ class SimSystemPosix(SimStatePlugin):
         self.stdout = stdout
         self.stderr = stderr
 
+    @SimStatePlugin.memo
+    def copy(self, memo):
+        o = super().copy(memo)
+
+        o.sigmask_bits = self.sigmask_bits
+        o.maximum_symbolic_syscalls = self.maximum_symbolic_syscalls
+        o.max_length = self.max_length
+        o.stdin = self.stdin.copy(memo)
+        o.stdout = self.stdout.copy(memo)
+        o.stderr = self.stderr.copy(memo)
+        o.fd = {k: self.fd[k].copy(memo) for k in self.fd}
+        o.sockets = {ident: tuple(x.copy(memo) for x in self.sockets[ident]) for ident in self.sockets}
+        o.socket_queue = self.socket_queue # shouldn't need to copy this - should be copied before use.
+        o.argv = self.argv
+        o.argc = self.argc
+        o.environ = self.environ
+        o.auxv = self.auxv
+        o.tls_modules = self.tls_modules
+        o._sigmask = self._sigmask
+        o.pid = self.pid
+        o.ppid = self.ppid
+        o.uid = self.uid
+        o.gid = self.gid
+        o.brk = self.brk
+        o.autotmp_counter = self.autotmp_counter
+        o.dev_fs = self.dev_fs.copy(memo)
+        o.proc_fs = self.proc_fs.copy(memo)
+        o._closed_fds = list(self._closed_fds)
+
+        return o
+
     @property
     def closed_fds(self):
         for _, f in self._closed_fds:
@@ -229,7 +266,7 @@ class SimSystemPosix(SimStatePlugin):
         return self.brk
 
     def set_state(self, state):
-        super(SimSystemPosix, self).set_state(state)
+        super().set_state(state)
 
         for fd in self.fd:
             self.fd[fd].set_state(state)
@@ -350,19 +387,23 @@ class SimSystemPosix(SimStatePlugin):
         If the number is concrete and does not map to anything, return None.
         If the number is symbolic, constrain it to an open fd and create a new file for it.
         """
+
         try:
             fd = self.state.solver.eval_one(fd)
+            return self.fd.get(fd)
         except SimSolverError:
-            ideal = self._pick_fd()
-            self.state.solver.add(fd == ideal)
-            if not self.state.solver.satisfiable():
-                raise SimPosixError("Tried to do operation on symbolic but partially constrained file descriptor")
-            fd = ideal
-            new_filename = b'/tmp/angr_implicit_%d' % self.autotmp_counter
-            l.warning("Tried to look up a symbolic fd - constrained to %d and opened %s", ideal, new_filename)
-            self.autotmp_counter += 1
-            if self.open(new_filename, Flags.O_RDWR, preferred_fd=fd) != fd:
-                raise SimPosixError("Something went wrong trying to open implicit temp")
+            pass
+
+        ideal = self._pick_fd()
+        self.state.solver.add(fd == ideal)
+        if not self.state.solver.satisfiable():
+            raise SimPosixError("Tried to do operation on symbolic but partially constrained file descriptor")
+        fd = ideal
+        new_filename = b'/tmp/angr_implicit_%d' % self.autotmp_counter
+        l.warning("Tried to look up a symbolic fd - constrained to %d and opened %s", ideal, new_filename)
+        self.autotmp_counter += 1
+        if self.open(new_filename, Flags.O_RDWR, preferred_fd=fd) != fd:
+            raise SimPosixError("Something went wrong trying to open implicit temp")
 
         return self.fd.get(fd)
 
@@ -390,9 +431,9 @@ class SimSystemPosix(SimStatePlugin):
     def fstat(self, sim_fd): #pylint:disable=unused-argument
         # sizes are AMD64-specific for symbolic files for now
         fd = None
-        sim_file = None
         mount = None
         mode = None
+        guest_path = None
 
         if not self.state.solver.symbolic(sim_fd):
             fd = self.state.solver.eval(sim_fd)
@@ -402,13 +443,15 @@ class SimSystemPosix(SimStatePlugin):
             # a fd can be SimFileDescriptorDuplex which is not backed by a file
             if isinstance(fd_desc, SimFileDescriptor):
                 sim_file = fd_desc.file
-                mount = self.state.fs.get_mountpoint(sim_file.name)[0]
+                mount = self.state.fs.get_mountpoint(sim_file.name)[0]  # TODO this is wrong. .name starts with file://
+                if mount:
+                    guest_path = mount.lookup(sim_file)
 
         # if it is mounted, let the filesystem figure out the stat
-        if sim_file is not None and mount is not None:
-            stat = mount._get_stat(sim_file.name)
+        if guest_path is not None and mount is not None:
+            stat = mount._get_stat(guest_path, dereference=True)
             if stat is None:
-                raise SimPosixError("file %s does not exist on mount %s" % (sim_file.name, mount))
+                raise SimPosixError("file %s does not exist on mount %s" % (guest_path, mount))
             size = stat.st_size
             mode = stat.st_mode
         else:
@@ -428,7 +471,7 @@ class SimSystemPosix(SimStatePlugin):
                     self.state.solver.BVV(0, 32), # st_gid
                     self.state.solver.BVV(0, 64), # st_rdev
                     size, # st_size
-                    self.state.solver.BVV(0, 64), # st_blksize
+                    self.state.solver.BVV(0x400, 64), # st_blksize
                     self.state.solver.BVV(0, 64), # st_blocks
                     self.state.solver.BVV(0, 64), # st_atime
                     self.state.solver.BVV(0, 64), # st_atimensec
@@ -477,33 +520,6 @@ class SimSystemPosix(SimStatePlugin):
             oldmask
         )
 
-    @SimStatePlugin.memo
-    def copy(self, memo):
-        o = SimSystemPosix(
-                stdin=self.stdin.copy(memo),
-                stdout=self.stdout.copy(memo),
-                stderr=self.stderr.copy(memo),
-                fd={k: self.fd[k].copy(memo) for k in self.fd},
-                sockets={ident: tuple(x.copy(memo) for x in self.sockets[ident]) for ident in self.sockets},
-                socket_queue=self.socket_queue, # shouldn't need to copy this - should be copied before use.
-                                                # as a result, we must update the state of each socket before making
-                                                # copies.
-                argv=self.argv,
-                argc=self.argc,
-                environ=self.environ,
-                auxv=self.auxv,
-                tls_modules=self.tls_modules,
-                sigmask=self._sigmask,
-                pid=self.pid,
-                ppid=self.ppid,
-                uid=self.uid,
-                gid=self.gid,
-                brk=self.brk)
-        o.dev_fs = self.dev_fs.copy(memo)
-        o.proc_fs = self.proc_fs.copy(memo)
-        o._closed_fds = list(self._closed_fds)
-        return o
-
     def merge(self, others, merge_conditions, common_ancestor=None):
         for o in others:
             if len(self.fd) != len(o.fd):
@@ -534,10 +550,14 @@ class SimSystemPosix(SimStatePlugin):
                 common_sock = common_ancestor.sockets[ident]
             except (AttributeError, KeyError):
                 common_sock = None
-            merging_occurred |= self.sockets[ident].merge(
-                    [o.sockets[ident] for o in others],
+            merging_occurred |= self.sockets[ident][0].merge(
+                    [o.sockets[ident][0] for o in others],
                     merge_conditions,
-                    common_ancestor=common_sock)
+                    common_ancestor=common_sock[0])
+            merging_occurred |= self.sockets[ident][1].merge(
+                [o.sockets[ident][1] for o in others],
+                merge_conditions,
+                common_ancestor=common_sock[1])
 
         # pylint: disable=no-member
         # pylint seems to be seriously flipping out here for reasons I'm unsure of
