@@ -83,7 +83,7 @@ class CongruencyCheck(Analysis):
         elif simgr.left[0].history.block_count < simgr.right[0].history.block_count:
             l.debug("... right is ahead; stepping left %s times",
                     simgr.right[0].history.block_count - simgr.left[0].history.block_count)
-            npg = simgr.step(
+            npg = simgr.run(
                 stash='left',
                 until=lambda lpg: lpg.left[0].history.block_count >= simgr.right[0].history.block_count,
                 n=max_steps
@@ -91,7 +91,7 @@ class CongruencyCheck(Analysis):
         elif simgr.right[0].history.block_count < simgr.left[0].history.block_count:
             l.debug("... left is ahead; stepping right %s times",
                     simgr.left[0].history.block_count - simgr.right[0].history.block_count)
-            npg = simgr.step(
+            npg = simgr.run(
                 stash='right',
                 until=lambda lpg: lpg.right[0].history.block_count >= simgr.left[0].history.block_count,
                 n=max_steps
@@ -296,8 +296,14 @@ class CongruencyCheck(Analysis):
             n_canoner_constraint = u_canoner_constraint = n_canon_constraint
         joint_solver.add((n_canoner_constraint, u_canoner_constraint))
         if n_canoner_constraint is not u_canoner_constraint:
-            self._report_incongruency("Different constraints!")
-            return False
+            # extra check: are these two constraints equivalent?
+            tmp_solver = claripy.Solver()
+            a = tmp_solver.satisfiable(extra_constraints=(n_canoner_constraint == u_canoner_constraint,))
+            b = tmp_solver.satisfiable(extra_constraints=(n_canoner_constraint != u_canoner_constraint,))
+
+            if not (a is True and b is False):
+                self._report_incongruency("Different constraints!")
+                return False
 
         # get the differences in registers and memory
         mem_diff = sr.memory.changed_bytes(sl.memory)
