@@ -1,5 +1,7 @@
+from typing import Iterable
 import operator
 import logging
+from typing import Dict, Any
 
 import networkx
 
@@ -16,12 +18,29 @@ from .utils import extract_jump_targets, switch_extract_cmp_bounds
 l = logging.getLogger(__name__)
 
 
+class TagsAnnotation(claripy.Annotation):
+    def __init__(self, **tags):
+        self.tags = tags
+        super().__init__()
+
+    @property
+    def eliminatable(self):
+        return False
+
+    @property
+    def relocatable(self):
+        return True
+
+    def relocate(self, src, dst):
+        return self
+
+
 class ConditionProcessor:
     """
     Convert between claripy AST and AIL expressions. Also calculates reaching conditions of all nodes on a graph.
     """
     def __init__(self, condition_mapping=None):
-        self._condition_mapping = {} if condition_mapping is None else condition_mapping
+        self._condition_mapping: Dict[str,Any] = {} if condition_mapping is None else condition_mapping
         self.reaching_conditions = {}
 
     def clear(self):
@@ -80,10 +99,15 @@ class ConditionProcessor:
         reaching_conditions = {}
         # recover the reaching condition for each node
         sorted_nodes = CFGUtils.quasi_topological_sort_nodes(_g)
+        terminating_nodes = [ ]
         for node in sorted_nodes:
             preds = _g.predecessors(node)
             reaching_condition = None
 
+            out_degree = _g.out_degree(node)
+            if out_degree == 0:
+                terminating_nodes.append(node)
+
             if node is region.head:
                 # the head is always reachable
                 reaching_condition = claripy.true
@@ -104,6 +128,22 @@ class ConditionProcessor:
             if reaching_condition is not None:
                 reaching_conditions[node] = self.simplify_condition(reaching_condition)
 
+        # My hypothesis to be proved: in any regioned graph, there must be a node with a 0 out-degree whose reaching
+        # conditions can be marked as True. In other words, if all 0 out-degree nodes have non-trivial reaching
+        # conditions, we can always pick one of them and change its reaching condition to True, without changing the
+        # semantics of the regioned graph.
+        if terminating_nodes and all(not reaching_conditions[node].is_true() for node in terminating_nodes
+                                     if node in reaching_conditions):
+            # pick the node with the greatest in-degree
+            terminating_nodes = sorted(terminating_nodes, key=_g.in_degree)
+            node_with_greatest_indegree = terminating_nodes[-1]
+            if _g.in_degree(node_with_greatest_indegree) > 1:
+                # forcing the in-degree to be greater than 1 allows us to skip the case blocks in switch-cases
+                # otherwise structurer will fail to structure the control flow
+                reaching_conditions[node_with_greatest_indegree] = claripy.true
+                l.warning("Marking node %r as trivially reachable. Disable this optimization in condition_processor.py "
+                          "if it leads to incorrect decompilation result.", node_with_greatest_indegree)
+
         self.reaching_conditions = reaching_conditions
 
     def recover_reaching_conditions_for_jumptables(self, region, jump_tables, edge_conditions):
@@ -378,6 +418,21 @@ class ConditionProcessor:
     # Expression conversion
     #
 
+    def _convert_extract(self, hi, lo, expr, annotations, memo=None):
+        # ailment does not support Extract. We translate Extract to Convert and shift.
+        if lo == 0:
+            try:
+                tag_annotation = next(iter(anno for anno in annotations if isinstance(anno, TagsAnnotation)))
+                tags = tag_annotation.tags
+            except StopIteration:
+                tags = {}
+            return ailment.Expr.Convert(None, expr.size(), hi + 1, False,
+                                        self.convert_claripy_bool_ast(expr, memo=memo),
+                                        **tags,
+                                        )
+
+        raise NotImplementedError("This case will be implemented once encountered.")
+
     def convert_claripy_bool_ast(self, cond, memo=None):
         """
         Convert recovered reaching conditions from claripy ASTs to ailment Expressions
@@ -399,48 +454,65 @@ class ConditionProcessor:
 
         if cond.op == "BoolS" and claripy.is_true(cond):
             return cond
-        if cond in self._condition_mapping:
-            return self._condition_mapping[cond]
+        if cond.args[0] in self._condition_mapping:
+            return self._condition_mapping[cond.args[0]]
 
-        def _binary_op_reduce(op, args, signed=False):
+        def _binary_op_reduce(op, args, annotations: Iterable[claripy.Annotation], signed=False):
             r = None
             for arg in args:
                 if r is None:
                     r = self.convert_claripy_bool_ast(arg, memo=memo)
                 else:
-                    r = ailment.Expr.BinaryOp(None, op, (r, self.convert_claripy_bool_ast(arg, memo=memo)), signed)
+                    try:
+                        tag_annotation = next(iter(anno for anno in annotations if isinstance(anno, TagsAnnotation)))
+                        tags = tag_annotation.tags
+                    except StopIteration:
+                        tags = {}
+                    r = ailment.Expr.BinaryOp(None, op, (r, self.convert_claripy_bool_ast(arg, memo=memo)), signed,
+                                              **tags
+                                              )
             return r
 
+        def _unary_op_reduce(op, arg, annotations: Iterable[claripy.Annotation]):
+            r = self.convert_claripy_bool_ast(arg, memo=memo)
+            try:
+                tag_annotation = next(iter(anno for anno in annotations if isinstance(anno, TagsAnnotation)))
+                tags = tag_annotation.tags
+            except StopIteration:
+                tags = {}
+            return ailment.Expr.UnaryOp(None, op, r, **tags)
+
         _mapping = {
-            'Not': lambda cond_: _binary_op_reduce('Not', cond_.args),
-            'And': lambda cond_: _binary_op_reduce('LogicalAnd', cond_.args),
-            'Or': lambda cond_: _binary_op_reduce('LogicalOr', cond_.args),
-            '__le__': lambda cond_: _binary_op_reduce('CmpLE', cond_.args, signed=True),
-            'SLE': lambda cond_: _binary_op_reduce('CmpLE', cond_.args, signed=True),
-            '__lt__': lambda cond_: _binary_op_reduce('CmpLT', cond_.args, signed=True),
-            'SLT': lambda cond_: _binary_op_reduce('CmpLT', cond_.args, signed=True),
-            'UGT': lambda cond_: _binary_op_reduce('CmpGT', cond_.args),
-            'UGE': lambda cond_: _binary_op_reduce('CmpGE', cond_.args),
-            '__gt__': lambda cond_: _binary_op_reduce('CmpGT', cond_.args, signed=True),
-            '__ge__': lambda cond_: _binary_op_reduce('CmpGE', cond_.args, signed=True),
-            'SGT': lambda cond_: _binary_op_reduce('CmpGT', cond_.args, signed=True),
-            'SGE': lambda cond_: _binary_op_reduce('CmpGE', cond_.args, signed=True),
-            'ULT': lambda cond_: _binary_op_reduce('CmpLT', cond_.args),
-            'ULE': lambda cond_: _binary_op_reduce('CmpLE', cond_.args),
-            '__eq__': lambda cond_: _binary_op_reduce('CmpEQ', cond_.args),
-            '__ne__': lambda cond_: _binary_op_reduce('CmpNE', cond_.args),
-            '__add__': lambda cond_: _binary_op_reduce('Add', cond_.args, signed=False),
-            '__sub__': lambda cond_: _binary_op_reduce('Sub', cond_.args),
-            '__mul__': lambda cond_: _binary_op_reduce('Mul', cond_.args),
-            '__xor__': lambda cond_: _binary_op_reduce('Xor', cond_.args),
-            '__or__': lambda cond_: _binary_op_reduce('Or', cond_.args, signed=False),
-            '__and__': lambda cond_: _binary_op_reduce('And', cond_.args),
-            '__lshift__': lambda cond_: _binary_op_reduce('Shl', cond_.args),
-            '__rshift__': lambda cond_: _binary_op_reduce('Sar', cond_.args),
-            'LShR': lambda cond_: _binary_op_reduce('Shr', cond_.args),
+            'Not': lambda cond_: _unary_op_reduce('Not', cond_.args[0], cond_.annotations),
+            'And': lambda cond_: _binary_op_reduce('LogicalAnd', cond_.args, cond_.annotations),
+            'Or': lambda cond_: _binary_op_reduce('LogicalOr', cond_.args, cond_.annotations),
+            '__le__': lambda cond_: _binary_op_reduce('CmpLE', cond_.args, cond_.annotations, signed=True),
+            'SLE': lambda cond_: _binary_op_reduce('CmpLE', cond_.args, cond_.annotations, signed=True),
+            '__lt__': lambda cond_: _binary_op_reduce('CmpLT', cond_.args, cond_.annotations, signed=True),
+            'SLT': lambda cond_: _binary_op_reduce('CmpLT', cond_.args, cond_.annotations, signed=True),
+            'UGT': lambda cond_: _binary_op_reduce('CmpGT', cond_.args, cond_.annotations),
+            'UGE': lambda cond_: _binary_op_reduce('CmpGE', cond_.args, cond_.annotations),
+            '__gt__': lambda cond_: _binary_op_reduce('CmpGT', cond_.args, cond_.annotations, signed=True),
+            '__ge__': lambda cond_: _binary_op_reduce('CmpGE', cond_.args, cond_.annotations, signed=True),
+            'SGT': lambda cond_: _binary_op_reduce('CmpGT', cond_.args, cond_.annotations, signed=True),
+            'SGE': lambda cond_: _binary_op_reduce('CmpGE', cond_.args, cond_.annotations, signed=True),
+            'ULT': lambda cond_: _binary_op_reduce('CmpLT', cond_.args, cond_.annotations),
+            'ULE': lambda cond_: _binary_op_reduce('CmpLE', cond_.args, cond_.annotations),
+            '__eq__': lambda cond_: _binary_op_reduce('CmpEQ', cond_.args, cond_.annotations),
+            '__ne__': lambda cond_: _binary_op_reduce('CmpNE', cond_.args, cond_.annotations),
+            '__add__': lambda cond_: _binary_op_reduce('Add', cond_.args, cond_.annotations, signed=False),
+            '__sub__': lambda cond_: _binary_op_reduce('Sub', cond_.args, cond_.annotations),
+            '__mul__': lambda cond_: _binary_op_reduce('Mul', cond_.args, cond_.annotations),
+            '__xor__': lambda cond_: _binary_op_reduce('Xor', cond_.args, cond_.annotations),
+            '__or__': lambda cond_: _binary_op_reduce('Or', cond_.args, cond_.annotations, signed=False),
+            '__and__': lambda cond_: _binary_op_reduce('And', cond_.args, cond_.annotations),
+            '__lshift__': lambda cond_: _binary_op_reduce('Shl', cond_.args, cond_.annotations),
+            '__rshift__': lambda cond_: _binary_op_reduce('Sar', cond_.args, cond_.annotations),
+            'LShR': lambda cond_: _binary_op_reduce('Shr', cond_.args, cond_.annotations),
             'BVV': lambda cond_: ailment.Expr.Const(None, None, cond_.args[0], cond_.size()),
             'BoolV': lambda cond_: ailment.Expr.Const(None, None, True, 1) if cond_.args[0] is True
                                                                         else ailment.Expr.Const(None, None, False, 1),
+            'Extract': lambda cond_: self._convert_extract(*cond_.args, cond_.annotations, memo=memo),
         }
 
         if cond.op in _mapping:
@@ -448,10 +520,10 @@ class ConditionProcessor:
         raise NotImplementedError(("Condition variable %s has an unsupported operator %s. "
                                    "Consider implementing.") % (cond, cond.op))
 
-    def claripy_ast_from_ail_condition(self, condition):
+    def claripy_ast_from_ail_condition(self, condition) -> claripy.ast.Base:
 
         # Unpack a condition all the way to the leaves
-        if isinstance(condition, claripy.ast.Base):
+        if isinstance(condition, claripy.ast.Base):  # pylint:disable=isinstance-second-argument-not-valid-type
             return condition
 
         def _op_with_unified_size(op, conv, operand0, operand1):
@@ -492,13 +564,13 @@ class ConditionProcessor:
         }
 
         if isinstance(condition, (ailment.Expr.Load, ailment.Expr.DirtyExpression, ailment.Expr.BasePointerOffset,
-                                  ailment.Expr.ITE)):
+                                  ailment.Expr.ITE, ailment.Stmt.Call)):
             var = claripy.BVS('ailexpr_%s' % repr(condition), condition.bits, explicit_name=True)
-            self._condition_mapping[var] = condition
+            self._condition_mapping[var.args[0]] = condition
             return var
         elif isinstance(condition, ailment.Expr.Register):
             var = claripy.BVS('ailexpr_%s-%d' % (repr(condition), condition.idx), condition.bits, explicit_name=True)
-            self._condition_mapping[var] = condition
+            self._condition_mapping[var.args[0]] = condition
             return var
         elif isinstance(condition, ailment.Expr.Convert):
             # convert is special. if it generates a 1-bit variable, it should be treated as a BVS
@@ -510,7 +582,7 @@ class ConditionProcessor:
                 var_ = self.claripy_ast_from_ail_condition(condition.operands[0])
                 name = 'ailexpr_Conv(%d->%d, %s)' % (condition.from_bits, condition.to_bits, repr(var_))
                 var = claripy.BVS(name, condition.to_bits, explicit_name=True)
-            self._condition_mapping[var] = condition
+            self._condition_mapping[var.args[0]] = condition
             return var
         elif isinstance(condition, ailment.Expr.Const):
             var = claripy.BVV(condition.value, condition.bits)
@@ -520,8 +592,8 @@ class ConditionProcessor:
             if condition.bits == 1:
                 var = claripy.BoolV('ailtmp_%d' % condition.tmp_idx)
             else:
-                var = claripy.BVS('ailtmp_%d' % condition.tmp_idx, condition.bits)
-            self._condition_mapping[var] = condition
+                var = claripy.BVS('ailtmp_%d' % condition.tmp_idx, condition.bits, explicit_name=True)
+            self._condition_mapping[var.args[0]] = condition
             return var
 
         lambda_expr = _mapping.get(condition.verbose_op, None)
@@ -530,7 +602,10 @@ class ConditionProcessor:
         r = lambda_expr(condition, self.claripy_ast_from_ail_condition)
         if r is NotImplemented:
             r = claripy.BVS("ailexpr_%r" % condition, condition.bits, explicit_name=True)
-            self._condition_mapping[r] = condition
+            self._condition_mapping[r.args[0]] = condition
+        else:
+            # don't lose tags
+            r = r.annotate(TagsAnnotation(**condition.tags))
         return r
 
     #
