@@ -1,6 +1,4 @@
 from .reaching_definitions import ReachingDefinitionsAnalysis
-from .live_definitions import LiveDefinitions
-from .constants import OP_AFTER, OP_BEFORE
 from .. import register_analysis
 
 
