@@ -6,19 +6,20 @@ import pyvex
 from ...engines.light import SimEngineLight, SimEngineLightVEXMixin, SpOffset, RegisterOffset
 from ...engines.vex.claripy.irop import operations as vex_operations
 from ...errors import SimEngineError
-from ...calling_conventions import DEFAULT_CC, SimRegArg, SimStackArg
+from ...calling_conventions import DEFAULT_CC, SimRegArg, SimStackArg, SimCC
 from ...utils.constants import DEFAULT_STATEMENT
-from ...knowledge_plugins.key_definitions.definition import Definition, RetValueTag
-from ...knowledge_plugins.key_definitions.atoms import Register, MemoryLocation, Parameter, Tmp
+from ...knowledge_plugins.key_definitions.definition import Definition
+from ...knowledge_plugins.key_definitions.tag import LocalVariableTag, ParameterTag, ReturnValueTag, Tag
+from ...knowledge_plugins.key_definitions.atoms import Atom, Register, MemoryLocation, Parameter, Tmp
 from ...knowledge_plugins.key_definitions.constants import OP_BEFORE, OP_AFTER
 from ...knowledge_plugins.key_definitions.dataset import DataSet
-from ...knowledge_plugins.key_definitions.undefined import Undefined, undefined
+from ...knowledge_plugins.key_definitions.heap_address import HeapAddress
+from ...knowledge_plugins.key_definitions.undefined import Undefined, UNDEFINED
 from ...code_location import CodeLocation
 from .rd_state import ReachingDefinitionsState
 from .external_codeloc import ExternalCodeLocation
 
 if TYPE_CHECKING:
-    from ...calling_conventions import SimCC
     from ...knowledge_plugins import FunctionManager
 
 
@@ -118,11 +119,14 @@ class SimEngineRDVEX(
 
         if any(type(d) is Undefined for d in data):
             l.info('Data to write into register <%s> with offset %d undefined, ins_addr = %#x.',
-                   self.arch.register_names[reg_offset], reg_offset, self.ins_addr)
+                   self.arch.translate_register_name(reg_offset, size=size), reg_offset, self.ins_addr)
 
-        # special handling for references to stack variables
+        # special handling for references to heap or stack variables
         for d in data:
-            if isinstance(d, SpOffset) and isinstance(d.offset, int):
+            if (
+                (isinstance(d, HeapAddress) and isinstance(d.value, int)) or
+                (isinstance(d, SpOffset) and isinstance(d.offset, int))
+            ):
                 self.state.add_use(MemoryLocation(d, 1), self._codeloc())
 
         self.state.kill_and_add_definition(reg, self._codeloc(), data)
@@ -155,7 +159,7 @@ class SimEngineRDVEX(
 
             self._store_core(addr, size, data, data_old=data_old)
 
-    def _store_core(self, addr: Iterable[Union[int,SpOffset,Undefined]], size: int, data, data_old=None):
+    def _store_core(self, addr: Iterable[Union[int,HeapAddress,SpOffset,Undefined]], size: int, data, data_old=None):
         if data_old:
             data.update(data_old)
 
@@ -164,13 +168,35 @@ class SimEngineRDVEX(
                 l.info('Memory address undefined, ins_addr = %#x.', self.ins_addr)
             else:
                 if any(type(d) is Undefined for d in data):
-                    l.info('Data to write at address %#x undefined, ins_addr = %#x.', a, self.ins_addr)
+                    l.info('Data to write at address %s undefined, ins_addr = %s.',
+                           "%#x" if isinstance(a, int) else a,
+                           "%#x" if isinstance(self.ins_addr, int) else self.ins_addr
+                           )
+
+                if (
+                    isinstance(a, int) or
+                    (isinstance(a, SpOffset) and isinstance(a.offset, int)) or
+                    (isinstance(a, HeapAddress) and isinstance(a.value, int))
+                ):
+                    tags: Optional[Set[Tag]]
+                    if isinstance(a, SpOffset):
+                        function_address = (
+                            self.project.kb
+                                .cfgs.get_most_accurate()
+                                .get_all_nodes(self._codeloc().ins_addr, anyaddr=True)[0]
+                                .function_address
+                        )
+                        tags = {LocalVariableTag(
+                            function = function_address,
+                            metadata = {'tagged_by': 'SimEngineRDVEX._store_core'}
+                        )}
+                    else:
+                        tags = None
 
-                if isinstance(a, int) or (isinstance(a, SpOffset) and isinstance(a.offset, int)):
                     memloc = MemoryLocation(a, size)
                     # different addresses are not killed by a subsequent iteration, because kill only removes entries
                     # with same index and same size
-                    self.state.kill_and_add_definition(memloc, self._codeloc(), data)
+                    self.state.kill_and_add_definition(memloc, self._codeloc(), data, tags=tags)
 
     def _handle_LoadG(self, stmt):
         guard: DataSet = self._expr(stmt.guard)
@@ -234,7 +260,7 @@ class SimEngineRDVEX(
         data = super()._expr(expr)
         if data is None:
             bits = expr.result_size(self.tyenv)
-            data = DataSet(undefined, bits)
+            data = DataSet(UNDEFINED, bits)
         return data
 
     def _handle_RdTmp(self, expr: pyvex.IRExpr.RdTmp) -> Optional[DataSet]:
@@ -260,12 +286,12 @@ class SimEngineRDVEX(
             data.update(current_def.data)
         if len(data) == 0:
             # no defs can be found. add a fake definition
-            data.add(undefined)
+            data.add(UNDEFINED)
             self.state.kill_and_add_definition(Register(reg_offset, size), self._external_codeloc(),
                                                DataSet(data, bits))
         if any(type(d) is Undefined for d in data):
             l.info('Data in register <%s> with offset %d undefined, ins_addr = %#x.',
-                   self.arch.register_names[reg_offset], reg_offset, self.ins_addr)
+                   self.arch.translate_register_name(reg_offset, size=size), reg_offset, self.ins_addr)
 
         self.state.add_use(Register(reg_offset, size), self._codeloc())
 
@@ -280,7 +306,7 @@ class SimEngineRDVEX(
 
         return self._load_core(addr, size, expr.endness)
 
-    def _load_core(self, addr: Iterable[Union[int,SpOffset]], size: int, endness: str):  # pylint:disable=unused-argument
+    def _load_core(self, addr: Iterable[Union[int,HeapAddress,SpOffset]], size: int, endness: str):  # pylint:disable=unused-argument
 
         current_defs: Iterable[Definition]
 
@@ -308,16 +334,26 @@ class SimEngineRDVEX(
                 current_defs = self.state.stack_definitions.get_objects_by_offset(a.offset)
                 if current_defs:
                     for def_ in current_defs:
-                        data.update(def_.data)
+                        data.update(def_.data.truncate(self.arch.bits))
                 else:
-                    data.add(undefined)
+                    data.add(UNDEFINED)
+                memory_location = MemoryLocation(a, size)
+                self.state.add_use(memory_location, self._codeloc())
+            elif isinstance(a, HeapAddress) and isinstance(a.value, int):
+                # Load data from the heap
+                current_defs = self.state.heap_definitions.get_objects_by_offset(a.value)
+                if current_defs:
+                    for def_ in current_defs:
+                        data.update(def_.data.truncate(self.arch.bits))
+                else:
+                    data.add(UNDEFINED)
                 memory_location = MemoryLocation(a, size)
                 self.state.add_use(memory_location, self._codeloc())
             else:
                 l.info('Memory address undefined, ins_addr = %#x.', self.ins_addr)
 
         if len(data) == 0:
-            data.add(undefined)
+            data.add(UNDEFINED)
 
         return DataSet(data, size * self.arch.byte_width)
 
@@ -379,7 +415,7 @@ class SimEngineRDVEX(
             if isinstance(e0, int):
                 return DataSet(e0 != 1, expr.result_size(self.tyenv))
 
-        # l.warning('Comparison of multiple values / different types.')
+        l.warning('Comparison of multiple values / different types.')
         return DataSet({True, False}, expr.result_size(self.tyenv))
 
     def _handle_Not(self, expr):
@@ -391,7 +427,7 @@ class SimEngineRDVEX(
             if isinstance(e0, int):
                 return DataSet(e0 == 0, expr.result_size(self.tyenv))
 
-        # l.warning('Comparison of multiple values / different types.')
+        l.warning('Comparison of multiple values / different types.')
         return DataSet({True, False}, expr.result_size(self.tyenv))
 
     #
@@ -414,7 +450,7 @@ class SimEngineRDVEX(
                         head = ((1 << e1) - 1) << (bits - e1)
                     data.add(head | (e0 >> e1))
                 except (ValueError, TypeError) as e:
-                    data.add(undefined)
+                    data.add(UNDEFINED)
                     l.warning(e)
 
         return DataSet(data, expr.result_size(self.tyenv))
@@ -430,7 +466,7 @@ class SimEngineRDVEX(
             if isinstance(e0, int) and isinstance(e1, int):
                 return DataSet(e0 == e1, expr.result_size(self.tyenv))
 
-        # l.warning('Comparison of multiple values / different types.')
+        l.warning('Comparison of multiple values / different types.')
         return DataSet({True, False}, expr.result_size(self.tyenv))
 
     def _handle_CmpNE(self, expr):
@@ -444,7 +480,7 @@ class SimEngineRDVEX(
             if isinstance(e0, int) and isinstance(e1, int):
                 return DataSet(e0 != e1, expr.result_size(self.tyenv))
 
-        # l.warning('Comparison of multiple values / different types.')
+        l.warning('Comparison of multiple values / different types.')
         return DataSet({True, False}, expr.result_size(self.tyenv))
 
     def _handle_CmpLT(self, expr):
@@ -458,7 +494,7 @@ class SimEngineRDVEX(
             if isinstance(e0, int) and isinstance(e1, int):
                 return DataSet(e0 < e1, expr.result_size(self.tyenv))
 
-        # l.warning('Comparison of multiple values / different types.')
+        l.warning('Comparison of multiple values / different types.')
         return DataSet({True, False}, expr.result_size(self.tyenv))
 
     # ppc only
@@ -478,14 +514,14 @@ class SimEngineRDVEX(
                 else:
                     return DataSet(0x02, expr.result_size(self.tyenv))
 
-        # l.warning('Comparison of multiple values / different types.')
+        l.warning('Comparison of multiple values / different types.')
         return DataSet({True, False}, expr.result_size(self.tyenv))
 
     def _handle_CCall(self, expr):
         bits = expr.result_size(self.tyenv)
         for arg_expr in expr.args:
             self._expr(arg_expr)
-        return DataSet(undefined, bits)
+        return DataSet(UNDEFINED, bits)
 
     #
     # User defined high level statement handlers
@@ -498,7 +534,7 @@ class SimEngineRDVEX(
 
     def _handle_function_core(self, func_addr: Optional[DataSet], **kwargs) -> bool:  # pylint:disable=unused-argument
 
-        if len(self._call_stack) + 1 > self._maximum_local_call_depth:
+        if self._call_stack is not None and len(self._call_stack) + 1 > self._maximum_local_call_depth:
             l.warning('The analysis reached its maximum recursion depth.')
             return False
 
@@ -580,8 +616,7 @@ class SimEngineRDVEX(
                     self._visited_blocks = visited_blocks
                     self._dep_graph = dep_graph
             else:
-                # l.warning('Please implement the local function handler with your own logic.')
-                pass
+                l.warning('Please implement the local function handler with your own logic.')
         else:
             l.warning('Could not find function name for external function at address %#x.', func_addr_int)
             handler_name = 'handle_unknown_call'
@@ -595,47 +630,47 @@ class SimEngineRDVEX(
 
         return skip_cc
 
-    def _handle_function_cc(self, func_addr: Optional[DataSet]):  # pylint:disable=unused-argument
-
-        cc: Optional['SimCC'] = None
+    def _handle_function_cc(self, func_addr: Optional[DataSet]):
+        _cc = None
+        func_addr_int: Optional[Union[int,Undefined]] = None
         if func_addr is not None and len(func_addr) == 1 and self.functions is not None:
             func_addr_int = next(iter(func_addr))
             if self.functions.contains_addr(func_addr_int):
-                cc = self.functions[func_addr_int].calling_convention
+                _cc = self.functions[func_addr_int].calling_convention
+        cc: SimCC = _cc or DEFAULT_CC.get(self.arch.name, None)(self.arch)
 
-        if cc is None:
-            cc = DEFAULT_CC.get(self.arch.name, None)(self.arch)
-
-        if cc is not None:
-            # follow the calling convention and:
-            # - add uses for arguments
-            # - kill return value registers
-            # - caller-saving registers
-            if cc.args:
-                code_loc = self._codeloc()
-                for arg in cc.args:
-                    if isinstance(arg, SimRegArg):
-                        reg_offset, reg_size = self.arch.registers[arg.reg_name]
-                        self.state.add_use(Register(reg_offset, reg_size), code_loc)
-                    elif isinstance(arg, SimStackArg):
-                        self.state.add_use(MemoryLocation(SpOffset(self.arch.bits,
-                                                                   arg.stack_offset),
-                                                          arg.size * self.arch.byte_width),
-                                           code_loc)
-            if cc.RETURN_VAL is not None:
-                if isinstance(cc.RETURN_VAL, SimRegArg):
-                    reg_offset, reg_size = self.arch.registers[cc.RETURN_VAL.reg_name]
+        # follow the calling convention and:
+        # - add uses for arguments
+        # - kill return value registers
+        # - caller-saving registers
+        if cc.args:
+            code_loc = self._codeloc()
+            for arg in cc.args:
+                if isinstance(arg, SimRegArg):
+                    reg_offset, reg_size = self.arch.registers[arg.reg_name]
                     atom = Register(reg_offset, reg_size)
-                    if func_addr is not None and len(func_addr) == 1 and self.functions is not None and type(func_addr_int) != Undefined:
-                        self.state.kill_and_add_definition(atom, self._codeloc(), DataSet({undefined}, reg_size * 8), tag=RetValueTag(metadata=hex(func_addr_int)))
-                    else:
-                        self.state.kill_and_add_definition(atom, self._codeloc(), DataSet({undefined}, reg_size * 8), tag=RetValueTag())
+                elif isinstance(arg, SimStackArg):
+                    atom = MemoryLocation(SpOffset(self.arch.bits,
+                                          arg.stack_offset),
+                                          arg.size * self.arch.byte_width)
+                self.state.add_use(atom, code_loc)
+                self._tag_definitions_of_atom(atom, func_addr_int)
 
-            if cc.CALLER_SAVED_REGS is not None:
-                for reg in cc.CALLER_SAVED_REGS:
-                    reg_offset, reg_size = self.arch.registers[reg]
-                    atom = Register(reg_offset, reg_size)
-                    self.state.kill_and_add_definition(atom, self._codeloc(), DataSet({undefined}, reg_size * 8))
+        if cc.RETURN_VAL is not None:
+            if isinstance(cc.RETURN_VAL, SimRegArg):
+                reg_offset, reg_size = self.arch.registers[cc.RETURN_VAL.reg_name]
+                atom = Register(reg_offset, reg_size)
+                tag = ReturnValueTag(
+                    function = func_addr_int if isinstance(func_addr_int, int) else None,
+                    metadata = {'tagged_by': 'SimEngineRDVEX._handle_function_cc'}
+                )
+                self.state.kill_and_add_definition(atom, self._codeloc(), DataSet({UNDEFINED}, reg_size * 8), tags={tag})
+
+        if cc.CALLER_SAVED_REGS is not None:
+            for reg in cc.CALLER_SAVED_REGS:
+                reg_offset, reg_size = self.arch.registers[reg]
+                atom = Register(reg_offset, reg_size)
+                self.state.kill_and_add_definition(atom, self._codeloc(), DataSet({UNDEFINED}, reg_size * 8))
 
         if self.arch.call_pushes_ret is True:
             # pop return address if necessary
@@ -662,4 +697,17 @@ class SimEngineRDVEX(
                 raise TypeError('Invalid type %s for stack pointer.' % type(sp_addr).__name__)
 
             atom = Register(self.arch.sp_offset, self.arch.bytes)
-            self.state.kill_and_add_definition(atom, self._codeloc(), DataSet(sp_addr, self.arch.bits))
+            tag = ReturnValueTag(
+                function = func_addr_int,
+                metadata = {'tagged_by': 'SimEngineRDVEX._handle_function_cc'}
+            )
+            self.state.kill_and_add_definition(atom, self._codeloc(), DataSet(sp_addr, self.arch.bits), tags={tag})
+
+    def _tag_definitions_of_atom(self, atom: Atom, func_addr: int):
+        definitions = self.state.get_definitions(atom)
+        tag = ParameterTag(
+            function = func_addr,
+            metadata = {'tagged_by': 'SimEngineRDVEX._handle_function_cc'}
+        )
+        for definition in definitions:
+            definition.tags |= {tag}
