@@ -12,6 +12,8 @@ class ArithmeticExpression:
     And = 4
     RShift = 8
     LShift = 16
+    Mul = 32
+    Xor = 64
 
     CONST_TYPES = (int, ailment.expression.Const)
 
@@ -34,6 +36,8 @@ class ArithmeticExpression:
             return "%s >> %s" % self.operands
         elif self.op == ArithmeticExpression.LShift:
             return "%s << %s" % self.operands
+        elif self.op == ArithmeticExpression.Mul:
+            return "%s * %s" % self.operands
         else:
             return "Unsupported op %s" % self.op
 
@@ -55,6 +59,16 @@ class ArithmeticExpression:
                 return ArithmeticExpression(self.op, (self.operands[0], self.operands[1] - other,))
         return ArithmeticExpression(self.op, (self, other, ))
 
+    def __rsub__(self, other):
+        if type(other) in ArithmeticExpression.CONST_TYPES:
+            other = self._unpack_const(other)
+            if type(self.operands[0]) is int:
+                return ArithmeticExpression(self.op, other - (self.operands[0], self.operands[1], ))
+            elif type(self.operands[1]) is int:
+                return ArithmeticExpression(self.op, (self.operands[0], other - self.operands[1],))
+        return ArithmeticExpression(self.op, (self, other, ))
+
+
     def __and__(self, other):
         if type(other) in ArithmeticExpression.CONST_TYPES:
             other = self._unpack_const(other)
@@ -73,6 +87,16 @@ class ArithmeticExpression:
                 return ArithmeticExpression(self.op, (self.operands[0], self.operands[1] | other,))
         return ArithmeticExpression(self.op, (self, other, ))
 
+    def __xor__(self, other):
+        if type(other) in ArithmeticExpression.CONST_TYPES:
+            other = self._unpack_const(other)
+            if type(self.operands[0]) is int:
+                return ArithmeticExpression(self.op, (self.operands[0] ^ other, self.operands[1], ))
+            elif type(self.operands[1]) is int:
+                return ArithmeticExpression(self.op, (self.operands[0], self.operands[1] ^ other,))
+        return ArithmeticExpression(self.op, (self, other, ))
+
+
     def __lshift__(self, other):
         if type(other) in ArithmeticExpression.CONST_TYPES:
             other = self._unpack_const(other)
@@ -185,6 +209,26 @@ class RegisterOffset:
                 return RegisterOffset(self._bits, self.reg,
                                       ArithmeticExpression(ArithmeticExpression.Sub, (other, self.offset, )))
 
+    def __mul__(self, other):
+        if not self.symbolic and type(other) is int:
+            return RegisterOffset(self._bits, self.reg, self._to_signed(self.offset * other))
+        else:
+            if self.symbolic:
+                return RegisterOffset(self._bits, self.reg, self.offset * other)
+            else:
+                return RegisterOffset(self._bits, self.reg,
+                                      ArithmeticExpression(ArithmeticExpression.Mul, (self.offset, other, )))
+
+    def __rmul__(self, other):
+        if not self.symbolic and type(other) is int:
+            return RegisterOffset(self._bits, self.reg, self._to_signed(other * self.offset))
+        else:
+            if self.symbolic:
+                return RegisterOffset(self._bits, self.reg, self.offset * other)
+            else:
+                return RegisterOffset(self._bits, self.reg,
+                                      ArithmeticExpression(ArithmeticExpression.Mul, (other, self.offset, )))
+
     def __and__(self, other):
         if not self.symbolic and type(other) is int:
             return RegisterOffset(self._bits, self.reg, self._to_signed(self.offset & other))
@@ -211,6 +255,23 @@ class RegisterOffset:
     def __ror__(self, other):
         return self.__or__(other)
 
+    def __xor__(self, other):
+        if not self.symbolic and type(other) is int:
+            return RegisterOffset(self._bits, self.reg, self._to_signed(self.offset | other))
+        else:
+            if self.symbolic:
+                return RegisterOffset(self._bits, self.reg, self.offset ^ other)
+            else:
+                return RegisterOffset(self._bits, self.reg,
+                                      ArithmeticExpression(ArithmeticExpression.Xor, (self.offset, other,)))
+
+    def __rxor__(self, other):
+        return self.__xor__(other)
+
+    def __floordiv__(self, other):
+        # this should never happen. returning self is obviously incorrect.
+        return self
+
     def __rshift__(self, other):
         if not self.symbolic and type(other) is int:
             return RegisterOffset(self._bits, self.reg, self._to_signed(self.offset >> other))
@@ -278,7 +339,7 @@ class SpOffset(RegisterOffset):
 
     def __repr__(self):
         if type(self.offset) is int:
-            offset_str = '' if self.offset == 0 else "%+x" % self.offset
+            offset_str = '' if self.offset == 0 else "%+#x" % self.offset
         else:
             offset_str = str(self.offset)
         return "%s%s" % ('BP' if self.is_base else 'SP', offset_str)
@@ -294,6 +355,9 @@ class SpOffset(RegisterOffset):
                 return SpOffset(self._bits, ArithmeticExpression(ArithmeticExpression.Add, (self.offset, other, )))
 
     def __sub__(self, other):
+        if isinstance(other, self.__class__):
+            return self.offset - other.offset
+
         other = ArithmeticExpression.try_unpack_const(other)
         if not self.symbolic and type(other) is int:
             return SpOffset(self._bits, self._to_signed(self.offset - other))
