@@ -1,13 +1,12 @@
 # pylint:disable=no-self-use
-from typing import Tuple
-
+from typing import Tuple, Optional
 import logging
 
 import ailment
 import pyvex
 import archinfo
 
-from ...engines.vex.claripy.irop import operations as vex_operations
+from ...engines.vex.claripy.irop import UnsupportedIROpError, vexop_to_simop
 from ...code_location import CodeLocation
 from ...utils.constants import DEFAULT_STATEMENT
 from ..engine import SimEngine
@@ -48,8 +47,13 @@ class SimEngineLight(SimEngine):
     #
 
     @property
-    def _context(self) -> Tuple[int]:
+    def _context(self) -> Optional[Tuple[int]]:
+        if self._call_stack is None:
+            # contextless mode
+            return None
+
         if not self._call_stack:
+            # contextful but the callstack is empty
             return tuple()
 
         # Convert to Tuple to make `context` hashable if not None
@@ -66,9 +70,7 @@ class SimEngineLight(SimEngine):
 
 class SimEngineLightVEXMixin:
 
-    def _process(self, state, successors, *args, block=None, whitelist=None, **kwargs):  # pylint:disable=arguments-differ,unused-argument
-
-        assert block is not None
+    def _process(self, state, successors, *args, block, whitelist=None, **kwargs):  # pylint:disable=arguments-differ,unused-argument
 
         # initialize local variables
         self.tmps = {}
@@ -206,7 +208,12 @@ class SimEngineLightVEXMixin:
         handler = None
 
         # All conversions are handled by the Conversion handler
-        simop = vex_operations.get(expr.op)
+        simop = None
+        try:
+            simop = vexop_to_simop(expr.op)
+        except UnsupportedIROpError:
+            pass
+
         if simop is not None and simop.op_attrs.get('conversion', None):
             handler = '_handle_Conversion'
         # Notice order of "Not" comparisons
@@ -225,6 +232,8 @@ class SimEngineLightVEXMixin:
         handler = None
         if expr.op.startswith('Iop_And'):
             handler = '_handle_And'
+        elif expr.op.startswith('Iop_Mod'):
+            handler = '_handle_Mod'
         elif expr.op.startswith('Iop_Or'):
             handler = '_handle_Or'
         elif expr.op.startswith('Iop_Add'):
@@ -254,11 +263,16 @@ class SimEngineLightVEXMixin:
             handler = '_handle_CmpLT'
         elif expr.op.startswith('Iop_CmpLE'):
             handler = '_handle_CmpLE'
+        elif expr.op.startswith('Iop_CmpGE'):
+            handler = '_handle_CmpGE'
+        elif expr.op.startswith('Iop_CmpGT'):
+            handler = '_handle_CmpGT'
         elif expr.op.startswith('Iop_CmpORD'):
             handler = '_handle_CmpORD'
+        elif expr.op == 'Iop_32HLto64':
+            handler = '_handle_32HLto64'
         elif expr.op.startswith('Const'):
             handler = '_handle_Const'
-
         if handler is not None and hasattr(self, handler):
             return getattr(self, handler)(expr)
         else:
@@ -423,6 +437,25 @@ class SimEngineLightVEXMixin:
         except TypeError as e:
             self.l.warning(e)
             return None
+        except ZeroDivisionError as e:
+            self.l.warning(e)
+            return None
+
+    def _handle_Mod(self, expr):
+        arg0, arg1 = expr.args
+        expr_0 = self._expr(arg0)
+        if expr_0 is None:
+            return None
+        expr_1 = self._expr(arg1)
+        if expr_1 is None:
+            return None
+
+        try:
+            # TODO: Probably should take care of the sign
+            return expr_0 - (expr_0 // expr_1)*expr_1
+        except TypeError as e:
+            self.l.warning(e)
+            return None
 
     def _handle_Xor(self, expr):
         arg0, arg1 = expr.args
@@ -441,18 +474,24 @@ class SimEngineLightVEXMixin:
 
     def _handle_Shl(self, expr):
         arg0, arg1 = expr.args
+
         expr_0 = self._expr(arg0)
         if expr_0 is None:
             return None
         expr_1 = self._expr(arg1)
         if expr_1 is None:
             return None
-
+        #elif expr_1 < 0:
+        #    expr.args = [expr_0, -1*expr_1]
+         #   return self._handle_Shr(expr)
         try:
             if isinstance(expr_0, int) and isinstance(expr_1, int):
                 # self.tyenv is not used
                 mask = (1 << expr.result_size(self.tyenv)) - 1
-                return (expr_0 << expr_1) & mask
+                if expr_1 < 0:
+                    return (expr_0 >> -expr_1) & mask
+                else:
+                    return (expr_0 << expr_1) & mask
             else:
                 return expr_0 << expr_1
         except TypeError as e:
@@ -469,7 +508,10 @@ class SimEngineLightVEXMixin:
             return None
 
         try:
-            return expr_0 >> expr_1
+            if expr_1 < 0:
+                return expr_0 << -expr_1
+            else:
+                return expr_0 >> expr_1
         except TypeError as e:
             self.l.warning(e)
             return None
@@ -534,6 +576,21 @@ class SimEngineLightVEXMixin:
             self.l.warning(ex)
             return None
 
+    def _handle_CmpGE(self, expr):
+        arg0, arg1 = expr.args
+        expr_0 = self._expr(arg0)
+        if expr_0 is None:
+            return None
+        expr_1 = self._expr(arg1)
+        if expr_1 is None:
+            return None
+
+        try:
+            return expr_0 >= expr_1
+        except TypeError as ex:
+            self.l.warning(ex)
+            return None
+
     def _handle_CmpLT(self, expr):
         arg0, arg1 = expr.args
         expr_0 = self._expr(arg0)
@@ -549,16 +606,43 @@ class SimEngineLightVEXMixin:
             self.l.warning(ex)
             return None
 
+    def _handle_CmpGT(self, expr):
+        arg0, arg1 = expr.args
+        expr_0 = self._expr(arg0)
+        if expr_0 is None:
+            return None
+        expr_1 = self._expr(arg1)
+        if expr_1 is None:
+            return None
+
+        try:
+            return expr_0 > expr_1
+        except TypeError as ex:
+            self.l.warning(ex)
+            return None
+
     def _handle_MBE(self, expr):  # pylint:disable=unused-argument
         # Yeah.... no.
         return None
 
+    def _handle_32HLto64(self, expr):
+        arg0, arg1 = expr.args
+        expr_0 = self._expr(arg0)
+        if expr_0 is None:
+            return None
+        expr_1 = self._expr(arg1)
+        if expr_1 is None:
+            return None
+
+        return None
+
+
 class SimEngineLightAILMixin:
 
     def _process(self, state, successors, *args, block=None, whitelist=None, **kwargs):  # pylint:disable=arguments-differ,unused-argument
 
         self.tmps = {}
-        self.block = block
+        self.block: ailment.Block = block
         self.state = state
         self.arch = state.arch
 
@@ -583,9 +667,24 @@ class SimEngineLightAILMixin:
 
     def _expr(self, expr):
 
-        handler = "_ail_handle_%s" % type(expr).__name__
+        expr_type_name = type(expr).__name__
+        if isinstance(expr, ailment.Stmt.Call):
+            # Call can be both an expression and a statement. Add a suffix to make sure we are working on the expression
+            # variant.
+            expr_type_name += "Expr"
+
+        h = None
+        handler = "_handle_%s" % expr_type_name
         if hasattr(self, handler):
-            return getattr(self, handler)(expr)
+            h = getattr(self, handler)
+
+        if h is None:
+            handler = "_ail_handle_%s" % expr_type_name
+            if hasattr(self, handler):
+                h = getattr(self, handler)
+
+        if h is not None:
+            return h(expr)
         self.l.warning('Unsupported expression type %s.', type(expr).__name__)
         return None
 
@@ -594,7 +693,8 @@ class SimEngineLightAILMixin:
     #
 
     def _codeloc(self):
-        return CodeLocation(self.block.addr, self.stmt_idx, ins_addr=self.ins_addr, context=self._context)
+        return CodeLocation(self.block.addr, self.stmt_idx, ins_addr=self.ins_addr, context=self._context,
+                            block_idx=self.block.idx)
 
     #
     # Statement handlers
@@ -620,6 +720,9 @@ class SimEngineLightAILMixin:
     def _ail_handle_Call(self, stmt):
         raise NotImplementedError('Please implement the Call handler with your own logic.')
 
+    def _ail_handle_Return(self, stmt):
+        raise NotImplementedError('Please implement the Return handler with your own logic.')
+
     #
     # Expression handlers
     #
@@ -638,6 +741,9 @@ class SimEngineLightAILMixin:
     def _ail_handle_Load(self, expr):
         raise NotImplementedError('Please implement the Load handler with your own logic.')
 
+    def _ail_handle_CallExpr(self, expr):
+        raise NotImplementedError('Please implement the CallExpr handler with your own logic.')
+
     def _ail_handle_UnaryOp(self, expr):
         handler_name = '_ail_handle_%s' % expr.op
         try:
@@ -853,6 +959,19 @@ class SimEngineLightAILMixin:
         except TypeError:
             return ailment.Expr.BinaryOp(expr.idx, 'Sar', [expr_0, expr_1], expr.signed, **expr.tags)
 
+    def _ail_handle_Concat(self, expr):
+
+        arg0, arg1 = expr.operands
+        expr_0 = self._expr(arg0)
+        expr_1 = self._expr(arg1)
+
+        if expr_0 is None:
+            expr_0 = arg0
+        if expr_1 is None:
+            expr_1 = arg1
+
+        return ailment.Expr.BinaryOp(expr.idx, 'Concat', [expr_0, expr_1], expr.signed, **expr.tags)
+
     #
     # Unary operation handlers
     #
