@@ -69,6 +69,7 @@ class HeavyVEXMixin(SuccessorsMixin, ClaripyDataMixin, SimStateStorageMixin, VEX
         size=None,
         num_inst=None,
         extra_stop_points=None,
+        opt_level=None,
         **kwargs):
         if not pyvex.lifting.lifters[self.state.arch.name] or type(successors.addr) is not int:
             return super().process_successors(successors, extra_stop_points=extra_stop_points, num_inst=num_inst, size=size, insn_text=insn_text, insn_bytes=insn_bytes, **kwargs)
@@ -99,7 +100,8 @@ class HeavyVEXMixin(SuccessorsMixin, ClaripyDataMixin, SimStateStorageMixin, VEX
                     thumb=thumb,
                     size=size,
                     num_inst=num_inst,
-                    extra_stop_points=extra_stop_points)
+                    extra_stop_points=extra_stop_points,
+                    opt_level=opt_level)
 
             if irsb.size == 0:
                 if irsb.jumpkind == 'Ijk_NoDecode' and not self.state.project.is_hooked(irsb.addr):
@@ -116,7 +118,7 @@ class HeavyVEXMixin(SuccessorsMixin, ClaripyDataMixin, SimStateStorageMixin, VEX
                 except errors.SimMemoryError:
                     raise errors.SimSegfaultError(addr, 'exec-miss')
                 else:
-                    if not perms.symbolic:
+                    if not self.state.solver.symbolic(perms):
                         perms = self.state.solver.eval(perms)
                         if not perms & 4 and o.ENABLE_NX in self.state.options:
                             raise errors.SimSegfaultError(addr, 'non-executable')
@@ -210,6 +212,12 @@ class HeavyVEXMixin(SuccessorsMixin, ClaripyDataMixin, SimStateStorageMixin, VEX
             if subaddr + stmt.addr in self.state.scratch.dirty_addrs:
                 raise errors.SimReliftException(self.state)
 
+        # HACK: mips64 may put an instruction which may fault in the delay slot of a branch likely instruction
+        # if the branch is not taken, we must not execute that instruction if the condition fails (i.e. the current guard is False)
+        if self.state.scratch.guard.is_false():
+            self.successors.add_successor(self.state, ins_addr, self.state.scratch.guard, 'Ijk_Boring')
+            raise VEXEarlyExit
+
         self.state.scratch.num_insns += 1
         self.successors.artifacts['insn_addrs'].append(ins_addr)
 
