@@ -1,13 +1,10 @@
 import angr
-from angr.sim_type import SimTypeInt
 
 ######################################
 # getuid
 ######################################
 
-
 class getuid(angr.SimProcedure):
     # pylint: disable=arguments-differ
     def run(self):
-        self.return_type = SimTypeInt(32, True)
-        return 1000
\ No newline at end of file
+        return 1000
