@@ -1,12 +1,8 @@
 import angr
-from angr.sim_type import SimTypeString
 
 class strcat(angr.SimProcedure):
+    # pylint: disable=arguments-differ
     def run(self, dst, src):
-        self.argument_types = {0: self.ty_ptr(SimTypeString()),
-                               1: self.ty_ptr(SimTypeString())}
-        self.return_type = self.ty_ptr(SimTypeString())
-
         strlen = angr.SIM_PROCEDURES['libc']['strlen']
         strncpy = angr.SIM_PROCEDURES['libc']['strncpy']
         src_len = self.inline_call(strlen, src).ret_expr
