@@ -227,7 +227,10 @@ class SimTypeReg(SimType):
 
     def store(self, state, addr, value):
         store_endness = state.arch.memory_endness
-
+        try:
+            value = value.ast
+        except AttributeError:
+            pass
         if isinstance(value, claripy.ast.Bits):  # pylint:disable=isinstance-second-argument-not-valid-type
             if value.size() != self.size:
                 raise ValueError("size of expression is wrong size for type")
@@ -480,6 +483,12 @@ class SimTypeFd(SimTypeReg):
     def copy(self):
         return SimTypeFd(label=self.label)
 
+    def _init_str(self):
+        return "%s(%s)" % (
+            self.__class__.__name__,
+            ('label="%s"' % self.label) if self.label is not None else "",
+        )
+
 
 class SimTypePointer(SimTypeReg):
     """
@@ -599,7 +608,7 @@ class SimTypeFixedSizeArray(SimType):
 
     def store(self, state, addr, values):
         for i, val in enumerate(values):
-            self.elem_type.store(state, addr + i*(self.elem_type.size//8), val)
+            self.elem_type.store(state, addr + i * (self.elem_type.size // state.arch.byte_width), val)
 
     @property
     def size(self):
@@ -823,7 +832,8 @@ class SimTypeFunction(SimType):
         return 4096     # ???????????
 
     def _with_arch(self, arch):
-        out = SimTypeFunction([a.with_arch(arch) for a in self.args], self.returnty.with_arch(arch),
+        out = SimTypeFunction([a.with_arch(arch) for a in self.args],
+                              self.returnty.with_arch(arch) if self.returnty is not None else None,
                               label=self.label,
                               arg_names=self.arg_names,
                               variadic=self.variadic
@@ -1067,7 +1077,12 @@ class SimStruct(NamedTypeMixin, SimType):
 
     @property
     def size(self):
-        return sum(val.size for val in self.fields.values())
+        if not self.offsets:
+            return 0
+
+        last_name, last_off = list(self.offsets.items())[-1]
+        last_type = self.fields[last_name]
+        return last_off * self._arch.byte_width + last_type.size
 
     @property
     def alignment(self):
@@ -1127,24 +1142,45 @@ class SimStructValue:
         self._struct = struct
         self._values = defaultdict(lambda: None, values or ())
 
+    def __indented_repr__(self, indent=0):
+        fields = []
+        for name in self._struct.fields:
+            value = self._values[name]
+            try:
+                f = getattr(value, '__indented_repr__')
+                s = f(indent=indent+2)
+            except AttributeError:
+                s = repr(value)
+            fields.append(' ' * (indent + 2) + '.{} = {}'.format(name, s))
+
+        return '{{\n{}\n{}}}'.format(',\n'.join(fields), ' ' * indent)
+
     def __repr__(self):
-        fields = ('.{} = {}'.format(name, self._values[name]) for name in self._struct.fields)
-        return '{{\n  {}\n}}'.format(',\n  '.join(fields))
+        return self.__indented_repr__()
 
     def __getattr__(self, k):
         return self[k]
 
     def __getitem__(self, k):
         if type(k) is int:
-            return self._values[self._struct.fields[k]]
+            k = self._struct.fields[k]
+        if k not in self._values:
+            for f in self._struct.fields:
+                if isinstance(f, NamedTypeMixin) and f.name is None:
+                    try:
+                        return f[k]
+                    except:
+                        continue
+            else:
+                return self._values[k]
+
         return self._values[k]
 
     def copy(self):
-        return SimStructValue(self._struct, values=defaultdict(self._values))
-
+        return SimStructValue(self._struct, values=defaultdict(lambda: None, self._values))
 
 class SimUnion(NamedTypeMixin, SimType):
-    _fields = ('members', 'name')
+    fields = ('members', 'name')
 
     def __init__(self, members, name=None, label=None):
         """
@@ -1162,6 +1198,24 @@ class SimUnion(NamedTypeMixin, SimType):
     def alignment(self):
         return max(val.alignment for val in self.members.values())
 
+    def _refine_dir(self):
+        return list(self.members.keys())
+
+    def _refine(self, view, k):
+        ty = self.members[k]
+        return view._deeper(ty=ty, addr=view._addr)
+
+    def extract(self, state, addr, concrete=False):
+        values = {}
+        for name, ty in self.members.items():
+            v = SimMemView(ty=ty, addr=addr, state=state)
+            if concrete:
+                values[name] = v.concrete
+            else:
+                values[name] = v.resolved
+
+        return SimUnionValue(self, values=values)
+
     def __repr__(self):
         # use the str instead of repr of each member to avoid exceed recursion
         # depth when representing self-referential unions
@@ -1181,6 +1235,44 @@ class SimUnion(NamedTypeMixin, SimType):
     def copy(self):
         return SimUnion(dict(self.members), name=self.name, label=self.label)
 
+class SimUnionValue:
+    """
+    A SimStruct type paired with some real values
+    """
+    def __init__(self, union, values=None):
+        """
+        :param union:      A SimUnion instance describing the type of this union
+        :param values:      A mapping from union members to values
+        """
+        self._union = union
+        self._values = defaultdict(lambda: None, values or ())
+
+    def __indented_repr__(self, indent=0):
+        fields = []
+        for name, value in self._values.items():
+            try:
+                f = getattr(value, '__indented_repr__')
+                s = f(indent=indent+2)
+            except AttributeError:
+                s = repr(value)
+            fields.append(' ' * (indent + 2) + '.{} = {}'.format(name, s))
+
+        return '{{\n{}\n{}}}'.format(',\n'.join(fields), ' ' * indent)
+
+    def __repr__(self):
+        return self.__indented_repr__()
+
+    def __getattr__(self, k):
+        return self[k]
+
+    def __getitem__(self, k):
+        if k not in self._values:
+            return super().__getitem__(k)
+        return self._values[k]
+
+    def copy(self):
+        return SimUnionValue(self._union, values=self._values)
+
 
 class SimCppClass(SimStruct):
     def __init__(self, members: Dict[str,SimType], name: Optional[str]=None, pack: bool=False, align=None):
@@ -1207,6 +1299,8 @@ BASIC_TYPES = {
     'unsigned short int': SimTypeShort(False),
 
     'int': SimTypeInt(True),
+    'signed': SimTypeInt(True),
+    'unsigned': SimTypeInt(False),
     'signed int': SimTypeInt(True),
     'unsigned int': SimTypeInt(False),
 
@@ -1270,6 +1364,7 @@ ALL_TYPES.update(BASIC_TYPES)
 # this is a hack, pending https://github.com/eliben/pycparser/issues/187
 def make_preamble():
     out = ['typedef int TOP;',
+           'typedef void BOT;',
            'typedef struct FILE_t FILE;',
            'typedef int pid_t;',
            'typedef int sigset_t;',
@@ -1426,6 +1521,16 @@ def parse_file(defn, preprocess=True):
             ty = _decl_to_type(piece.type, extra_types)
             if piece.name is not None:
                 out[piece.name] = ty
+
+            # Don't forget to update typedef types
+            if (isinstance(ty, SimStruct) or isinstance(ty, SimUnion)) and ty.name != '<anon>':
+                for _, i in extra_types.items():
+                    if i.name == ty.name:
+                        if isinstance(ty, SimStruct):
+                            i.fields = ty.fields
+                        else:
+                            i.members = ty.members
+
         elif isinstance(piece, pycparser.c_ast.Typedef):
             extra_types[piece.name] = copy.copy(_decl_to_type(piece.type, extra_types))
             extra_types[piece.name].label = piece.name
@@ -1507,6 +1612,9 @@ def _decl_to_type(decl, extra_types=None):
 
     elif isinstance(decl, pycparser.c_ast.ArrayDecl):
         elem_type = _decl_to_type(decl.type, extra_types)
+
+        if decl.dim is None:
+            return SimTypeArray(elem_type)
         try:
             size = _parse_const(decl.dim)
         except ValueError as e:
@@ -1520,6 +1628,10 @@ def _decl_to_type(decl, extra_types=None):
         else:
             fields = OrderedDict()
 
+        # Don't forget that "type[]" has a different meaning in structures than in functions
+        if len(fields) > 0 and isinstance(fields[next(reversed(fields))], SimTypeArray):
+            raise NotImplementedError("Sorry, we have no support of flexible array members")
+
         if decl.name is not None:
             key = 'struct ' + decl.name
             struct = extra_types.get(key, None)
@@ -1574,12 +1686,16 @@ def _decl_to_type(decl, extra_types=None):
         else:
             raise TypeError("Unknown type '%s'" % ' '.join(key))
 
+    elif isinstance(decl, pycparser.c_ast.Enum):
+        # See C99 at 6.7.2.2
+        return ALL_TYPES['int']
+
     raise ValueError("Unknown type!")
 
 
 def _parse_const(c):
     if type(c) is pycparser.c_ast.Constant:
-        return int(c.value)
+        return int(c.value, base=0)
     elif type(c) is pycparser.c_ast.BinaryOp:
         if c.op == '+':
             return _parse_const(c.children()[0][1]) + _parse_const(c.children()[1][1])
@@ -1589,6 +1705,10 @@ def _parse_const(c):
             return _parse_const(c.children()[0][1]) * _parse_const(c.children()[1][1])
         if c.op == '/':
             return _parse_const(c.children()[0][1]) // _parse_const(c.children()[1][1])
+        if c.op == '<<':
+            return _parse_const(c.children()[0][1]) << _parse_const(c.children()[1][1])
+        if c.op == '>>':
+            return _parse_const(c.children()[0][1]) >> _parse_const(c.children()[1][1])
         raise ValueError('Binary op %s' % c.op)
     else:
         raise ValueError(c)
