@@ -7,6 +7,7 @@ from cle import MetaELF
 from cle.backends.elf.symbol import ELFSymbol, ELFSymbolType
 from cle.backends.elf.elfcore import ELFCore
 from cle.address_translator import AT
+from cle.backends.elf.relocation.arm64 import R_AARCH64_TLSDESC
 from archinfo import ArchX86, ArchAMD64, ArchARM, ArchAArch64, ArchMIPS32, ArchMIPS64, ArchPPC32, ArchPPC64
 
 from ..tablespecs import StringTableSpec
@@ -14,6 +15,7 @@ from ..procedures import SIM_PROCEDURES as P, SIM_LIBRARIES as L
 from ..state_plugins import SimFilesystem, SimHostFilesystem
 from ..storage.file import SimFile, SimFileBase
 from ..errors import AngrSyscallError
+from .. import sim_options as o
 from .userland import SimUserland
 
 _l = logging.getLogger(name=__name__)
@@ -34,6 +36,7 @@ class SimLinux(SimUserland):
         self._loader_addr = None
         self._loader_lock_addr = None
         self._loader_unlock_addr = None
+        self._loader_destructor = 0
         self._error_catch_tsd_addr = None
         self._is_core = None
         self.vsyscall_addr = None
@@ -45,11 +48,13 @@ class SimLinux(SimUserland):
             self._loader_addr = self.project.loader.extern_object.allocate()
             self._loader_lock_addr = self.project.loader.extern_object.allocate()
             self._loader_unlock_addr = self.project.loader.extern_object.allocate()
+            self._loader_destructor = self.project.loader.extern_object.allocate()
             self._error_catch_tsd_addr = self.project.loader.extern_object.allocate()
             self.vsyscall_addr = self.project.loader.extern_object.allocate()
             self.project.hook(self._loader_addr, P['linux_loader']['LinuxLoader']())
             self.project.hook(self._loader_lock_addr, P['linux_loader']['_dl_rtld_lock_recursive']())
             self.project.hook(self._loader_unlock_addr, P['linux_loader']['_dl_rtld_unlock_recursive']())
+            self.project.hook(self._loader_destructor, P['stubs']['ReturnUnconstrained'](return_val=0))
             self.project.hook(self._error_catch_tsd_addr,
                               P['linux_loader']['_dl_initial_error_catch_tsd'](
                                   static_addr=self.project.loader.extern_object.allocate()
@@ -60,6 +65,7 @@ class SimLinux(SimUserland):
             # there are some functions we MUST use the simprocedures for, regardless of what the user wants
             self._weak_hook_symbol('__tls_get_addr', L['ld.so'].get('__tls_get_addr', self.arch))    # ld
             self._weak_hook_symbol('___tls_get_addr', L['ld.so'].get('___tls_get_addr', self.arch))  # ld
+            self._weak_hook_symbol('_dl_get_tls_static_info', L['ld.so'].get('_dl_get_tls_static_info', self.arch))  # ld
             self._weak_hook_symbol('_dl_vdso_vsym', L['libc.so.6'].get('_dl_vdso_vsym', self.arch))  # libc
 
             # set up some static data in the loader object...
@@ -69,11 +75,15 @@ class SimLinux(SimUserland):
                     self.project.loader.memory.pack_word(_rtld_global.rebased_addr + 0xF08, self._loader_lock_addr)
                     self.project.loader.memory.pack_word(_rtld_global.rebased_addr + 0xF10, self._loader_unlock_addr)
                     self.project.loader.memory.pack_word(_rtld_global.rebased_addr + 0x990, self._error_catch_tsd_addr)
+                elif isinstance(self.project.arch, ArchARM):
+                    self.project.loader.memory.pack_word(_rtld_global.rebased_addr + 0x7E8, self._loader_lock_addr)
+                    self.project.loader.memory.pack_word(_rtld_global.rebased_addr + 0x7EC, self._loader_unlock_addr)
 
             # TODO: what the hell is this
             _rtld_global_ro = self.project.loader.find_symbol('_rtld_global_ro')
             if _rtld_global_ro is not None:
-                pass
+                if isinstance(self.project.arch, ArchAMD64):
+                    self.project.loader.memory.pack_word(_rtld_global_ro.rebased_addr + 0x0D0, 2)  # cpu features: kind = amd
 
             tls_obj = self.project.loader.tls.new_thread()
             if isinstance(self.project.arch, ArchAMD64):
@@ -108,12 +118,18 @@ class SimLinux(SimUserland):
                             }
                             # TODO: should this be replaced with hook_symbol?
                             randaddr = self.project.loader.extern_object.allocate()
-                            self.project.hook(randaddr, P['linux_loader']['IFuncResolver'](**kwargs))
+                            self.project.hook(randaddr, P['linux_loader']['IFuncResolver'](display_name='IFuncResolver.' + reloc.symbol.name, **kwargs))
                             self.project.loader.memory.pack_word(gotaddr, randaddr)
 
-
         if isinstance(self.project.arch, ArchARM):
-            self.project.hook(0xffff0fe0, P['linux_kernel']['_kernel_user_helper_get_tls']())
+            # https://www.kernel.org/doc/Documentation/arm/kernel_user_helpers.txt
+            for func_name in P['linux_kernel']:
+                if not func_name.startswith('_kuser_'):
+                    continue
+                func = P['linux_kernel'][func_name]
+                self.project.hook(func.kuser_addr, func())
+        elif isinstance(self.project.arch, ArchAArch64):
+            self.project.hook(R_AARCH64_TLSDESC.RESOLVER_ADDR, P['linux_loader']['tlsdesc_resolver']())
 
         # maybe move this into archinfo?
         if self.arch.name == 'X86':
@@ -135,6 +151,8 @@ class SimLinux(SimUserland):
             syscall_abis = ['ppc']
         elif self.arch.name == 'PPC64':
             syscall_abis = ['ppc64']
+        elif self.arch.name == 'RISCV':
+            syscall_abis = ['riscv32']
         else:
             syscall_abis = [] # ?
 
@@ -157,7 +175,11 @@ class SimLinux(SimUserland):
     # pylint: disable=arguments-differ
     def state_blank(self, fs=None, concrete_fs=False, chroot=None,
             cwd=None, pathsep=b'/', thread_idx=None, init_libc = False, **kwargs):
-        state = super(SimLinux, self).state_blank(thread_idx=thread_idx, **kwargs)
+        state = super().state_blank(thread_idx=thread_idx, **kwargs)
+
+        # pre-grow the stack by 0x20 pages. unsure if this is strictly required or just a hack around a compiler bug
+        if not self._is_core and hasattr(state.memory, 'allocate_stack_pages'):
+            state.memory.allocate_stack_pages(state.solver.eval(state.regs.sp) - 1, 0x20 * 0x1000)
 
         tls_obj = self.project.loader.tls.threads[thread_idx if thread_idx is not None else 0]
         if isinstance(state.arch, ArchAMD64):
@@ -335,8 +357,7 @@ class SimLinux(SimUserland):
                     state.registers.store(reg, state.posix.auxv)
                 elif val == 'ld_destructor':
                     # a pointer to the dynamic linker's destructor routine, to be called at exit
-                    # or NULL. We like NULL. It makes things easier.
-                    state.registers.store(reg, 0)
+                    state.registers.store(reg, self._loader_destructor)
                 elif val == 'toc':
                     if self.project.loader.main_object.is_ppc64_abiv1:
                         state.registers.store(reg, self.project.loader.main_object.ppc64_initial_rtoc)
