@@ -1,5 +1,8 @@
 import logging
 import struct
+from typing import Optional
+
+import angr # for types
 
 import claripy
 from archinfo import ArchMIPS32, ArchS390X
@@ -15,7 +18,6 @@ from ..calling_conventions import DEFAULT_CC
 from ..procedures import SIM_PROCEDURES as P
 from .. import sim_options as o
 from ..storage.file import SimFileStream, SimFileBase
-from ..misc import IRange
 
 
 _l = logging.getLogger(name=__name__)
@@ -26,7 +28,7 @@ class SimOS:
     A class describing OS/arch-level configuration.
     """
 
-    def __init__(self, project, name=None):
+    def __init__(self, project: 'angr.Project', name=None):
         self.arch = project.arch
         self.project = project
         self.name = name
@@ -81,12 +83,14 @@ class SimOS:
             sym = scope.get_symbol(name)
 
         if sym is not None:
-            if self.project.is_hooked(sym.rebased_addr):
-                if not self.project.hooked_by(sym.rebased_addr).is_stub:
+            addr, _ = self.prepare_function_symbol(name, basic_addr=sym.rebased_addr)
+            if self.project.is_hooked(addr):
+                if not self.project.hooked_by(addr).is_stub:
                     return
-            self.project.hook(sym.rebased_addr, hook)
+            self.project.hook(addr, hook)
 
-    def state_blank(self, addr=None, initial_prefix=None, brk=None, stack_end=None, stack_size=1024*1024*8, stdin=None, **kwargs):
+    def state_blank(self, addr=None, initial_prefix=None, brk=None, stack_end=None, stack_size=1024*1024*8, stdin=None,
+                    thread_idx=None, permissions_backer=None, **kwargs):
         """
         Initialize a blank state.
 
@@ -104,7 +108,18 @@ class SimOS:
         # TODO: move ALL of this into the SimState constructor
         if kwargs.get('mode', None) is None:
             kwargs['mode'] = self.project._default_analysis_mode
-        if kwargs.get('permissions_backer', None) is None:
+        if permissions_backer is not None:
+            kwargs['permissions_map'] = permissions_backer[1]
+            kwargs['default_permissions'] = 7 if permissions_backer[0] else 3
+        if kwargs.get('cle_memory_backer', None) is None:
+            kwargs['cle_memory_backer'] = self.project.loader
+        if kwargs.get('os_name', None) is None:
+            kwargs['os_name'] = self.name
+        actual_stack_end = stack_end
+        if stack_end is None:
+            stack_end = self.arch.initial_sp
+
+        if kwargs.get('permissions_map', None) is None:
             # just a dict of address ranges to permission bits
             permission_map = { }
             for obj in self.project.loader.all_objects:
@@ -118,32 +133,34 @@ class SimOS:
                     if seg.is_executable:
                         perms |= 4  # PROT_EXEC
                     permission_map[(seg.min_addr, seg.max_addr)] = perms
-            permissions_backer = (self.project.loader.main_object.execstack, permission_map)
-            kwargs['permissions_backer'] = permissions_backer
-        if kwargs.get('memory_backer', None) is None:
-            kwargs['memory_backer'] = self.project.loader.memory
-        if kwargs.get('os_name', None) is None:
-            kwargs['os_name'] = self.name
+            kwargs['permissions_map'] = permission_map
+        if self.project.loader.main_object.execstack:
+            stack_perms = 1 | 2 | 4  # RWX
+        else:
+            stack_perms = 1 | 2  # RW
 
-        state = SimState(self.project, **kwargs)
+        state = SimState(self.project, stack_end=stack_end, stack_size=stack_size, stack_perms=stack_perms, **kwargs)
 
         if stdin is not None and not isinstance(stdin, SimFileBase):
             if type(stdin) is type:
                 stdin = stdin(name='stdin', has_end=False)
             else:
+                if isinstance(stdin, claripy.Bits):
+                    num_bytes = len(stdin) // self.project.arch.byte_width
+                else:
+                    num_bytes = len(stdin)
+                _l.warning("stdin is constrained to %d bytes (has_end=True). If you are only providing the first "
+                           "%d bytes instead of the entire stdin, please use "
+                           "stdin=SimFileStream(name='stdin', content=your_first_n_bytes, has_end=False).",
+                           num_bytes, num_bytes)
                 stdin = SimFileStream(name='stdin', content=stdin, has_end=True)
 
         last_addr = self.project.loader.main_object.max_addr
         actual_brk = (last_addr - last_addr % 0x1000 + 0x1000) if brk is None else brk
         state.register_plugin('posix', SimSystemPosix(stdin=stdin, brk=actual_brk))
 
-
-        actual_stack_end = state.arch.initial_sp if stack_end is None else stack_end
-        if o.ABSTRACT_MEMORY not in state.options:
-            state.memory.mem._preapproved_stack = IRange(actual_stack_end - stack_size, actual_stack_end)
-
         if state.arch.sp_offset is not None:
-            state.regs.sp = actual_stack_end + (state.arch.stack_change if state.arch.stack_change is not None else 0)
+            state.regs.sp = stack_end
 
         if initial_prefix is not None:
             for reg in state.arch.default_symbolic_registers:
@@ -155,7 +172,6 @@ class SimOS:
                     eternal=True))
 
         for reg, val, is_addr, mem_region in state.arch.default_register_values:
-
             region_base = None  # so pycharm does not complain
 
             if is_addr:
@@ -168,7 +184,7 @@ class SimOS:
                 else:
                     raise AngrSimOSError('You must specify the base address for memory region "%s". ' % mem_region)
 
-            # special case for stack pointer override
+            # special case for stack_end overriding sp default
             if actual_stack_end is not None and state.arch.registers[reg][0] == state.arch.sp_offset:
                 continue
 
@@ -181,9 +197,12 @@ class SimOS:
         if addr is None:
             state.regs.ip = self.project.entry
 
-        for reg, val in self.project.loader.main_object.thread_registers().items():
-            if reg in state.arch.registers or reg in ('flags', 'eflags', 'rflags'):
-                setattr(state.regs, reg, val)
+        thread_name = self.project.loader.main_object.threads[thread_idx] if thread_idx is not None else None
+        for reg, val in self.project.loader.main_object.thread_registers(thread_name).items():
+            if reg in ('fs', 'gs', 'cs', 'ds', 'es', 'ss') and state.arch.name == 'X86':
+                state.registers.store(reg, val >> 16)  # oh boy big hack
+            elif reg in state.arch.registers or reg in ('flags', 'eflags', 'rflags'):
+                state.registers.store(reg, val)
             elif reg == 'fctrl':
                 state.regs.fpround = (val & 0xC00) >> 10
             elif reg == 'fstat':
@@ -303,9 +322,12 @@ class SimOS:
     def syscall(self, state, allow_unsupported=True):
         return None
 
-    def syscall_abi(self, state):
+    def syscall_abi(self, state) -> str:
         return None
 
+    def syscall_cc(self, state) -> Optional[angr.calling_conventions.SimCCSyscall]:
+        raise NotImplementedError()
+
     def is_syscall_addr(self, addr):
         return False
 
