@@ -35,7 +35,7 @@ class PTChunk(Chunk):
     """
 
     def __init__(self, base, sim_state, heap=None):
-        super(PTChunk, self).__init__(base, sim_state)
+        super().__init__(base, sim_state)
 
         # This is necessary since the heap can't always be referenced through the state, e.g. during heap initialization
         self.heap = self.state.heap if heap is None else heap
@@ -60,13 +60,13 @@ class PTChunk(Chunk):
         chunk_flags = self.state.memory.load(self.base + self._chunk_size_t_size, self._chunk_size_t_size) \
                       & CHUNK_FLAGS_MASK
         unsilence_logger(level)
-        self.state.memory.store(self.base + self._chunk_size_t_size, size | chunk_flags)
+        self.state.memory.store(self.base + self._chunk_size_t_size, size | chunk_flags, size=self.state.arch.bytes)
 
     def _set_trailing_size(self, size):
         if self.is_free():
             next_chunk = self.next_chunk()
             if next_chunk is not None:
-                self.state.memory.store(next_chunk.base, size)
+                self.state.memory.store(next_chunk.base, size, self.state.arch.bytes)
 
     def set_size(self, size, is_free=None):  #pylint:disable=arguments-differ
         """
@@ -86,7 +86,7 @@ class PTChunk(Chunk):
                 self.heap._set_final_freeness(is_free)
         if is_free is not None and is_free or self.is_free():
             if next_chunk is not None:
-                self.state.memory.store(next_chunk.base, size)
+                self.state.memory.store(next_chunk.base, size, size=self.state.arch.bytes)
 
     def set_prev_freeness(self, is_free):
         """
@@ -98,9 +98,9 @@ class PTChunk(Chunk):
         size_field = self.state.memory.load(self.base + self._chunk_size_t_size, self._chunk_size_t_size)
         unsilence_logger(level)
         if is_free:
-            self.state.memory.store(self.base + self._chunk_size_t_size, size_field & ~CHUNK_P_MASK)
+            self.state.memory.store(self.base + self._chunk_size_t_size, size_field & ~CHUNK_P_MASK, size=self.state.arch.bytes)
         else:
-            self.state.memory.store(self.base + self._chunk_size_t_size, size_field | CHUNK_P_MASK)
+            self.state.memory.store(self.base + self._chunk_size_t_size, size_field | CHUNK_P_MASK, size=self.state.arch.bytes)
 
     def is_prev_free(self):
         """
@@ -116,7 +116,7 @@ class PTChunk(Chunk):
             return self.state.solver.min_int(flag)
 
         flag = concretize(flag, self.state.solver, sym_flag_handler)
-        return False if flag else True
+        return not flag
 
     def prev_size(self):
         """
@@ -142,7 +142,7 @@ class PTChunk(Chunk):
                 return self.state.solver.min_int(flag)
 
             flag = concretize(flag, self.state.solver, sym_flag_handler)
-            return False if flag else True
+            return not flag
 
     def data_ptr(self):
         return self.base + (2 * self._chunk_size_t_size)
@@ -190,7 +190,7 @@ class PTChunk(Chunk):
             raise SimHeapError("Attempted to access the forward chunk of an allocated chunk")
 
     def set_fwd_chunk(self, fwd):
-        self.state.memory.store(self.base + 2 * self._chunk_size_t_size, fwd.base, endness=self.state.arch.memory_endness)
+        self.state.memory.store(self.base + 2 * self._chunk_size_t_size, fwd.base, endness=self.state.arch.memory_endness, size=self.state.arch.bytes)
 
     def bck_chunk(self):
         """
@@ -206,7 +206,7 @@ class PTChunk(Chunk):
             raise SimHeapError("Attempted to access the backward chunk of an allocated chunk")
 
     def set_bck_chunk(self, bck):
-        self.state.memory.store(self.base + 3 * self._chunk_size_t_size, bck.base, endness=self.state.arch.memory_endness)
+        self.state.memory.store(self.base + 3 * self._chunk_size_t_size, bck.base, endness=self.state.arch.memory_endness, size=self.state.arch.bytes)
 
 class PTChunkIterator:
     def __init__(self, chunk, cond=lambda chunk: True):
@@ -228,9 +228,9 @@ class PTChunkIterator:
                 self.chunk = self.chunk.next_chunk()
             if self.chunk is None:
                 raise StopIteration
-            else:
-                ret = self.chunk
-                self.chunk = self.chunk.next_chunk()
+
+            ret = self.chunk
+            self.chunk = self.chunk.next_chunk()
 
         return ret
 
@@ -250,16 +250,25 @@ class SimHeapPTMalloc(SimHeapFreelist):
     """
 
     def __init__(self, heap_base=None, heap_size=None):
-        super(SimHeapPTMalloc, self).__init__(heap_base, heap_size)
+        super().__init__(heap_base, heap_size)
 
         # All of these depend on the state and so are initialized in init_state
         self._free_head_chunk_exists = True  # Only used during plugin copy due to the dependency on the memory plugin
         self._free_head_chunk_init_base = None  # Same as above
         self._chunk_size_t_size = None  # Size (bytes) of the type used to store a piece of metadata
         self._chunk_min_size = None  # Based on needed fields for any chunk
+        self._chunk_align_mask = 0
         self.free_head_chunk = None
         self._initialized = False
 
+    @SimStatePlugin.memo
+    def copy(self, memo):# pylint: disable=unused-argument
+        o = super().copy(memo)
+        o._free_head_chunk_exists = self.free_head_chunk is not None
+        o._free_head_chunk_init_base = self.free_head_chunk.base if self.free_head_chunk is not None else None
+        o._initialized = self._initialized
+        return o
+
     def chunks(self):
         return PTChunkIterator(PTChunk(self.heap_base, self.state))
 
@@ -316,9 +325,9 @@ class SimHeapPTMalloc(SimHeapFreelist):
         this. Nonetheless, for now it is implemented as if an additional chunk followed the final chunk.
         """
         if flag:
-            self.state.memory.store(self.heap_base + self.heap_size - self._chunk_size_t_size, ~CHUNK_P_MASK)
+            self.state.memory.store(self.heap_base + self.heap_size - self._chunk_size_t_size, ~CHUNK_P_MASK, size=self.state.arch.bytes)
         else:
-            self.state.memory.store(self.heap_base + self.heap_size - self._chunk_size_t_size, CHUNK_P_MASK)
+            self.state.memory.store(self.heap_base + self.heap_size - self._chunk_size_t_size, CHUNK_P_MASK, size=self.state.arch.bytes)
 
     def _make_chunk_size(self, req_size):
         """
@@ -409,9 +418,9 @@ class SimHeapPTMalloc(SimHeapFreelist):
             return
         size = chunk.get_size()
 
-        p_in_use = False if chunk.is_prev_free() else True
+        p_in_use = not chunk.is_prev_free()
         n_ptr = chunk.next_chunk()
-        n_in_use = False if n_ptr is not None and n_ptr.is_free() else True  # Next is taken to be in use if it doesn't
+        n_in_use = n_ptr is None or not n_ptr.is_free()
         if p_in_use and n_in_use:                                            # exist
             # When both adjacent chunks are in use, no merging will be
             # necessary between the freed chunk and another free chunk
@@ -548,15 +557,6 @@ class SimHeapPTMalloc(SimHeapFreelist):
     def _realloc(self, ptr, size):
         return self.realloc(ptr, size)
 
-    @SimStatePlugin.memo
-    def copy(self, memo):# pylint: disable=unused-argument
-        c = SimHeapPTMalloc(heap_base=self.heap_base, heap_size=self.heap_size)
-        c.mmap_base = self.mmap_base
-        c._free_head_chunk_exists = True if self.free_head_chunk is not None else False
-        c._free_head_chunk_init_base = self.free_head_chunk.base if self.free_head_chunk is not None else None
-        c._initialized = self._initialized
-        return c
-
     def _combine(self, others):
         if any(o.heap_base != self.heap_base for o in others):
             raise SimMergeError("Cannot merge heaps with different bases")
@@ -585,11 +585,11 @@ class SimHeapPTMalloc(SimHeapFreelist):
         return self._combine(others)
 
     def init_state(self):
-        super(SimHeapPTMalloc, self).init_state()
+        super().init_state()
 
-        self._chunk_size_t_size = self.state.arch.bits // 8
+        self._chunk_size_t_size = self.state.arch.bytes
         self._chunk_min_size = 4 * self._chunk_size_t_size
-        self._chunk_align_mask = 2 * self._chunk_size_t_size - 1  #pylint:disable=attribute-defined-outside-init
+        self._chunk_align_mask = 2 * self._chunk_size_t_size - 1
 
         # TODO: where are bin metadata stored in reality?
         if self._free_head_chunk_exists and self._free_head_chunk_init_base is None:
@@ -607,7 +607,7 @@ class SimHeapPTMalloc(SimHeapFreelist):
         if not self._initialized:
             self.state.memory.store(self.free_head_chunk.base + self._chunk_size_t_size
                                     , ((self.heap_size - 2 * self._chunk_size_t_size) & ~self._chunk_align_mask)
-                                    | CHUNK_P_MASK)
+                                    | CHUNK_P_MASK, size=self.state.arch.bytes)
             self._set_final_freeness(True)
             self.free_head_chunk.set_fwd_chunk(self.free_head_chunk)
             self.free_head_chunk.set_bck_chunk(self.free_head_chunk)
