@@ -11,7 +11,7 @@ from .sim_action_object import ast_stripping_decorator, SimActionObject
 
 l = logging.getLogger(name=__name__)
 
-#pylint:disable=unidiomatic-typecheck
+#pylint:disable=unidiomatic-typecheck,isinstance-second-argument-not-valid-type
 
 #
 # Timing stuff
@@ -181,8 +181,8 @@ class SimSolver(SimStatePlugin):
     Any top-level variable of the claripy module can be accessed as a property of this object.
     """
     def __init__(self, solver=None, all_variables=None, temporal_tracked_variables=None, eternal_tracked_variables=None): #pylint:disable=redefined-outer-name
-        l.debug("Creating SimSolverClaripy.")
-        SimStatePlugin.__init__(self)
+        super().__init__()
+
         self._stored_solver = solver
         self.all_variables = [] if all_variables is None else all_variables
         self.temporal_tracked_variables = {} if temporal_tracked_variables is None else temporal_tracked_variables
@@ -252,6 +252,8 @@ class SimSolver(SimStatePlugin):
         """
         Given an AST, iterate over all the keys of all the BVS leaves in the tree which are registered.
         """
+        # pylint: disable=stop-iteration-return
+        # ??? wtf pylint
         reverse_mapping = {next(iter(var.variables)): k for k, var in self.eternal_tracked_variables.items()}
         reverse_mapping.update({next(iter(var.variables)): k for k, var in self.temporal_tracked_variables.items() if k[-1] is not None})
 
@@ -407,7 +409,7 @@ class SimSolver(SimStatePlugin):
             return f
 
     def __dir__(self):
-        return sorted(set(dir(super(SimSolver, self)) + dir(claripy._all_operations) + dir(self.__class__)))
+        return sorted(set(dir(super()) + dir(claripy._all_operations) + dir(self.__class__)))
 
     #
     # Branching stuff
@@ -415,7 +417,14 @@ class SimSolver(SimStatePlugin):
 
     @SimStatePlugin.memo
     def copy(self, memo): # pylint: disable=unused-argument
-        return type(self)(solver=self._solver.branch(), all_variables=self.all_variables, temporal_tracked_variables=self.temporal_tracked_variables, eternal_tracked_variables=self.eternal_tracked_variables)
+        c = super().copy(memo)
+
+        c._stored_solver = self._solver.branch()
+        c.all_variables = self.all_variables
+        c.temporal_tracked_variables = self.temporal_tracked_variables
+        c.eternal_tracked_variables = self.eternal_tracked_variables
+
+        return c
 
     @error_converter
     def merge(self, others, merge_conditions, common_ancestor=None): # pylint: disable=W0613
@@ -740,6 +749,10 @@ class SimSolver(SimStatePlugin):
         :return:
         """
         # eval_upto already throws the UnsatError, no reason for us to worry about it
+        concrete_val = _concrete_value(e)
+        if concrete_val is not None:
+            return self._cast_to(e, concrete_val, cast_to)
+
         return self.eval_upto(e, 1, cast_to = cast_to,  **kwargs)[0]
 
     def eval_one(self, e, **kwargs):
